package com.asdf.plugins.pokergames;

/**
 * A class to efficiently represent a traditional playing card from a 52 card deck.
 * 
 *
 */

public class Card {
	
	/**
	 * Default constructor. Initializes the card to improper card.
	 *
	 */
	public Card() {
		card = 0;
	}
	
	/**
	 * Constructs a card with the given integer representation.
	 * @param card_ integer representation of the card.
	 */
	public Card(int card_) {
		card = card_;
	}
	
	/**
	 * Constructs a card with given rank and suit.
	 * @param rank Rank is a value from [0-12] and describes the rank of the card.
	 * @param suit Suit is an integer from [0-3].
	 */
	public Card(int rank, int suit) {
		int r_suit = 0x8000;
		switch(suit) {
		case 0:
			r_suit >>= 1;
		case 1:
			r_suit >>= 1;
		case 2:
			r_suit >>= 1;
		case 3:
			break;
		default:
			r_suit = 0;
		}
		
		card = primes[rank] | rank << 8 | r_suit |  (1 << (16 + rank));
	}
	
	/**
	 * Compares this card to another card.
	 * @param a Another card.
	 * @return 2 if rank of current card is greater than of the other card. -2 if rank is less than that of the other card. 1 if ranks are equal but suit is greater. -1 if ranks are equal but suit is less. 0 if has the same rank and same suit.
	 */
	public int compareTo(Card a) {
		if(rank() > a.rank())
			return 2;
		if(rank() < a.rank())
			return -2;
		if(suit() > a.suit())
			return 1;
		if(suit() < a.suit())
			return -1;
		return 0;
	}
	
	/**
	 * Comparator function. Compares two cards and says which one is bigger.
	 * @param anotherCard A card to which we will compare this card.
	 * @return Returns true if this card is of greater rank than the other card. Otherwise false.
	 */
	
	public boolean lessThan(Card anotherCard) {
		if (rank() < anotherCard.rank())
			return true;
		return false;
	}

	/**
	 * Checks if the card is a valid card
	 * @return False if card value is zero, true otherwise.
	 */
	public boolean isValid() {
		if(card == 0)
			return false;
		return true;
	}
	
	
	/**
	 * Getter function for card rank.
	 * @return Rank of the card
	 */
	public int rank() {
		return ((card >> 8) & 0xF);
	}

	/**
	 * Getter function for card suit.
	 * @return Integer representation of the suit from the range [0-3].
	 */
	public int suit() {
		if ((card & 0x1000) != 0)
			return 0;
		if ((card & 0x2000) != 0)
			return 1;
		if ((card & 0x4000) != 0)
			return 2;
		else
			return 3;
	}

	/**
	 * Swaps the contents of two cards
	 * @param a Another card with which the contents of this card will be swapped.
	 */
	public void swap(Card a) {
		Card temp = new Card(a.card);
		a.card = card;
		card = temp.card;
	}

	/**
	 * Getter for integer representation of the card.
	 * @return Integer representation of the card.
	 */
	public int getInt() {
		return card;
	}
	
	
	/**
	 * Creates a string representation of the card.
	 */
	public String toString() {
		String suit;
		int s = suit();
		
		switch(s) {
		case 0:
			suit = "C";
			break;
		case 1:
			suit = "D";
			break;
		case 2:
			suit = "H";
			break;
		case 3:
			suit = "S";
			break;
		default:
			suit = "";
		}
		
		int rank = rank();
		if(rank == 12) {
			suit += "1";
			return suit;
		} else {
			suit += rank + 2;
			return suit;
		}
	}
	
	public int card;
	public static int primes[] = { 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41 };
}
