package com.asdf.plugins.pokergames;

import java.util.List;

import com.asdf.plugins.pokergames.PokerGame.Blind;

public interface GameInfo {

	
	/**
	 * Get whether the player has sat out some blinds, and has to pay any as a penalty for entering the game
	 * @return missed blinds
	 * @param seat of the player
	 */
	public Blind hasToPostBlind(int seat);
	
	/**
	 * See if given seat is reserved
	 * @param seat
	 * @return true if the seat is reserved
	 */
	
	public boolean isReserved(int seat);
	
	/**
	 * See how much a player must at least bring to table with him, her or it
	 * @return maximum amount
	 */
	public double minBuy();
	
	/**
	 * See how much a player can at most bring to table with him, her or it
	 * @return maximum amount
	 */
	public double maxBuy();
		
	
	/**
	 * See if the player in the given seat is able to raise
	 * Calls the PlayerInfo's corresponding method.
	 * @return true if the player has enough chips to raise after calling
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 */
	public boolean canRaise(int arg0) ; 
			
			 
	

	/**
	 * Get a player's amount to call
	 * Calls the PlayerInfo's corresponding method.
	 * @return the amount given player has to call
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 */
	public double getAmountToCall(int arg0) ; 
		
		 
	

	/**
	 * Get the amount of the ante, if any
	 * @return the value of ante
	 */
	public double getAnte() ; 
		
		 
	

	/**
	 * Get a player's bankroll
	 * Calls the PlayerInfo's corresponding method.
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return the given player's bankroll
	 */
	public double getBankRoll(int arg0) ; 
		

		 
	

	/**
	 * Get a player's amount to call in bets
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 */
	public double getBetsToCall(int arg0) ; 
		
		 
	

	/**
	 * Get the big blind seat
	 * @return the big blind seat
	 */
	public int getBigBlindSeat() ; 
		
		 
	

	/**
	 * Get the amount of the big blind.
	 * @return the amount of the big blind
	 */
	public double getBigBlindSize() ; 
		
		 
	

	/**
	 * Get the board cards
	 * @return new hand with the board cards
	 */
	public Hand getBoard() ; 
		
		 
	

	/**
	 * Get the seat the button is on (could be dead button)
	 * @return button seat
	 */
	public int getButtonSeat() ; 
		
		 
	

	/**
	 * Get the amount of the betting increment (for fixed-limit)
	 * @return current bet size, nothing relevant if not fixed-limit
	 */
	public double getCurrentBetSize() ; 
		
		 
	

	/**
	 * Get the seat for the current player to act
	 * @return the seat for the current player to act
	 */
	public int getCurrentPlayerSeat() ; 
		
		 
	

	/**
	 * Get the amount in the pot that a player is eligible to win.
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return the amount the player is eligible to win
	 */
	public double getEligiblePot(int arg0) ; 
		
		 
	

	/**
	 * The game identifier for this table
	 * @return the gameID
	 */
	public long getGameID() ; 
		


	/**
	 * Get the size of the main pot.
	 * @return the main pot
	 */
	public double getMainPotSize() ; 
		
		 
	

	/**
	 * Get the smallest legal raise
	 * @return the minimum raise
	 */
	public double getMinRaise() ; 
		
		 
	

	/**
	 * Get the number of players still active (not folded) in the hand
	 * @return the number of active players
	 */
	public int getNumActivePlayers() ; 
		
		 
	

	/**
	 * Get the number of active players that are not all-in
	 * @return the number of players not all-in
	 */
	public int getNumActivePlayersNotAllIn() ; 
		
		 
	

	/**
	 * Get the number of players dealt into the hand
	 * @return the number of players dealt in to the hand
	 */
	public int getNumPlayers() ; 
		
		 
	

	/**
	 * Get the number of bets and raises made during the current betting round
	 * @return the number of raises made during current round
	 */
	public int getNumRaises() ; 
		
		 
	

	/**
	 * The number of seats at the table
	 * @return the number of seats
	 */
	public int getNumSeats() ; 
 
	

	/**
	 * Get the number of players still to act in this betting round if there are no further raises.
	 * @return the number of players to act
	 */
	public int getNumToAct() ; 
		
		 
	

	/**
	 * Get the number of players that were awarded a pot in this hand
	 * @return the number of winners
	 */
	public int getNumWinners() ; 
		
		 
	

	/**
	 * Get the number of players in the hand that are all-in
	 * @return number of players all-in
	 */
	public int getNumberOfAllInPlayers() ; 
		
		 
	

	/**
	 * Get state information for a player in a given seat
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return PlayerInfo for given player, or null if seat is empty
	 */
	public PlayerInfo getPlayer(int arg0) ; 
		
		 
	

	/**
	 * Get state information for a player with a given name
	 * @param arg0 the name of desired player
	 * @return PlayerInfo for given player, or null if not found
	 */
	public PlayerInfo getPlayer(String arg0) ; 
		
		 
	

	/**
	 * Get the name of the player in a given seat
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return the name of the player in the seat
	 */
	public String getPlayerName(int arg0) ; 
		
		 
	

	/**
	 * Get the seat number for a player by name
	 * @param arg0 the name of the player
	 * @return the seat the player is in or -1 if not found
	 */
	public int getPlayerSeat(String arg0) ; 
		
		 
	

	/**
	 * Get a list of all players eligible to win a pot with the given minimum stakes
	 * @param arg0 the amount a player must have in the pot to be eligible to win it
	 * @return a list of all players eligible to win the pot
	 */
	public List<PlayerInfo> getPlayersInPot(double arg0) ; 
		
		 

	/**
	 * Get the small blind seat (could be dead small blind)
	 * @return the small blind seat
	 */
	public int getSmallBlindSeat() ; 
		
		 
	

	/**
	 * Get the amount of the small blind
	 * @return small blind size
	 */
	public double getSmallBlindSize() ; 
		
		 
	

	/**
	 * Get the current game stage
	 * @return the current stage
	 */
	public Stage getStage() ; 
		
		 
	

	/**
	 * The stakes are the amount each player must have invested in the pot to stay active in the hand.
	 * @return the stakes
	 */
	public double getStakes() ; 
		
		 
	

	/**
	 * Get the total size of the pot including the main pot and all side pots.
	 * @return total pot size
	 * 
	 */
	public double getTotalPotSize() ; 
		
		 
	

	/**
	 * Get the number of players who have not acted yet this round
	 * @return the number of players not yet acted 
	 */
	public int getUnacted() ; 
		
		 
	

	/**
	 * See if the player in the given seat is dealt into the hand (not sitting out or empty).
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return true if the player is dealt in
	 */
	public boolean inGame(int arg0) ; 
		
		 
	

	/**
	 * See if the player in the given seat is still active in the hand
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return true if player is active
	 */
	public boolean isActive(int arg0) ; 
		
		 
	

	/**
	 * See if a player has invested anything in the current round
	 * @param arg0 position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return true if the player has invested
	 */
	public boolean isCommitted(int arg0) ; 
		
		 
	

	/**
	 * Determine if the game is fixed-limit
	 * @return true if the game is fixed-limit
	 */
	public boolean isFixedLimit() ; 
		
		 	

	/**
	 * Determine if the game is no-limit
	 * @return true if the game is no-limit
	 */
	public boolean isNoLimit() ; 
		
	

	/**
	 * Determine if the game is pot-limit
	 * @return true if the game is pot-limit
	 */
	public boolean isPotLimit() ; 
		
		

	/**
	 * In reverse blinds mode, if the game is Heads-up, the small blind is placed on the button.
	 * @return if we are using the reverse blinds rule for heads up
	 */
	public boolean isReverseBlinds() ; 
		 
	

	/**
	 * Determine if the game is in simulation mode. Simulations are high speed games used to determine approximate values of particular hand scenarios In simulation mode, players may choose to play faster, using cruder approximations, and should avoid all use of disk.
	 * @return true if game is in simulation mode
	 */
	public boolean isSimulation() ; 
		
		 
	

	
	//public boolean isTurn() ; 
		
		 
	

	/**
	 * We should play as fast and crudely as possible because the user has requested we zip to the end of the hand.
	 * @return true is it is Zip mode
	 */
	public boolean isZipMode() ; 
		
		 
	

	/**
	 * Get the seat number of the next active player at the table (skips empty seats and folded players)
	 * @param arg0 position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return the seat number of the next active player
	 */
	public int nextActivePlayer(int arg0) ; 
		
		 
	

	/**
	 * Get the seat number of the next active player at the table (skips empty seats and folded players)
	 * @param arg0 position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return the seat number of the next active player
	 */
	public int nextPlayer(int arg0) ; 
		
		 
	

	/**
	 * Get the seat number of the next seat clockwise at the table
	 * @param arg0 position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return the next seat number
	 */
	public int nextSeat(int arg0) ; 
		
		 
	

	/**
	 * Get the seat number of the previous player at the table (skips empty seats)
	 * @param arg0 position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return seat number of previous player
	 */
	public int previousPlayer(int arg0) ; 
		
		 
	
}
	

	

