package com.asdf.plugins.pokergames;

//import java.util.Random;
import java.util.Vector;

/**
 * This class evaluates poker hands with methods eval_5cards and eval_7cards.
 * Returns an integer representing the "strength" of a hand. A hand with greater number wins
 * against lower value hands.
 *
 */
public class HandEval {

	public EvalTables tables;

	/**
	 * Constructor. Sets the tables to point to the location given as parameter.
	 * Enables the possibility of not having an instance of the tables (which are rather large)
	 * for every instance of HandEval, and thus, for every poker game instance.
	 * @param evalTables Reference to an EvalTables instance.
	 */
	public HandEval(EvalTables evalTables) {
		tables = evalTables;
	}
	
	/**
	 * Describes how a deck should be built if this evaluator is intended to be used with the deck.
	 * @param deck A Vector of Integer representations of Cards.
	 */
	public void init_deck(Vector<Integer> deck) {
		int i, j, n = 0, suit = 0x8000;
		deck = new Vector<Integer>();

		for (i = 0; i < 4; i++, suit >>= 1)
			for (j = 0; j < 13; j++, n++)
				deck.add(new Integer(tables.primes[j] | (j << 8) | suit | (1 << (16 + j))));
	}


	/**
	 * This function performs a binary search on a presorted array of integer representasions of hands.
	 * The index gives the absolute value of the hand, and tables.products contains the prime number product representation of the hand.
	 * @param key Prime number representation of the given hand.
	 * @return Absolute value of queried hand, or -1 if unable to resolve hand value.
	 */
	public int findit(int key) {
		int low = 0, high = 4887, mid;

		while (low <= high) {
			mid = (high + low) >> 1; // divide by two
		
			if (key < tables.products[mid])
				high = mid - 1;
			else if (key > tables.products[mid])
				low = mid + 1;
			else
				return (mid);
		}

		return (-1);
	}

	/**
	 *  This routine will search a deck for a specific card
	 * (specified by rank/suit), and return the INDEX giving
	 * the position of the found card.  If it is not found,
	 * then it returns -1
	 * @param rank Rank of queried card.
	 * @param suit Suit of queried card. (0x8000 || 0x4000 || 0x2000 || 0x1000)
	 * @param deck A deck of cards.
	 * @return The index in the deck where the card is located, or -1 if the card is not in the deck.
	 */

	public int find_card(int rank, int suit, Vector<Integer> deck) {
		int i, c;

		for (i = 0; i < deck.size(); i++) {
			c = deck.elementAt(i).intValue();
			if (((c & suit) != 0) && (RANK(c) == rank))
				return (i);
		}
		return (-1);
	}
	
	
	/**
	 * Cycles through all 5 card combinations of the 6 cards, and returns the best hand and it's value.
	 * @param cards A 6 card hand in integer representation. See com.asdf.utils.Card.getInt().
	 * @return The best combination of 5 cards in this hand, and it's value. If there are multiple hands of the same value, the first one encountered is preferred.
	 */
	public BestHand eval_6cards(int[] cards) {
		
		BestHand handReturner = null;
		
		int[] card_positions = new int[5];
		int[] cards5 = new int[5];
		int best_val = 10000;
		int temp_val = 10000;
		
		for(int out1 = 0; out1 < cards.length; out1++) {

			int counter = 0;
			for(int i=0; i < cards.length; i++) {
				if(i != out1) {
					cards5[counter] = cards[i];
					card_positions[counter] = i;
					counter++;
				}
			}
			
			temp_val = eval_5cards(cards5[0], cards5[1], cards5[2], cards5[3], cards5[4]);
			if(temp_val < best_val)
			{
				best_val = temp_val;
				handReturner = new BestHand(card_positions, best_val);
			}
		}
		
		return handReturner;
	}
	
	

	/**
	 * Cycles through all 5 card combinations of the 7 cards, and returns the best hand and it's value.
	 * @param cards A 7 card hand in integer representation. See com.asdf.utils.Card.getInt().
	 * @return The best combination of 5 cards in this hand, and it's value. If there are multiple hands of the same value, the first one encountered is preferred.
	 */
	public BestHand eval_7cards(int[] cards) {
		
		BestHand handReturner = null;
		
		int[] card_positions = new int[5];
		int[] cards5 = new int[5];
		int best_val = 10000;
		int temp_val = 10000;
		
		for(int out1 = 0; out1 < cards.length; out1++)
			for(int out2 = 1; out2 < cards.length; out2++) {
				if(out1 == out2)
					continue;
				int counter = 0;
				for(int i=0; i < cards.length; i++) {
					if(i != out1 && i != out2) {
						cards5[counter] = cards[i];
						card_positions[counter] = i;
						counter++;
					}
				}
				
				temp_val = eval_5cards(cards5[0], cards5[1], cards5[2], cards5[3], cards5[4]);
				if(temp_val < best_val)
				{
					best_val = temp_val;
					handReturner = new BestHand(card_positions, best_val);
				}
			}
		
		return handReturner;
	}
	
	/**
	 *  Calculates the absolute value of a given 5 card poker-hand.
	 * @param c1 card1
	 * @param c2 card2
	 * @param c3 card3
	 * @param c4 card4
	 * @param c5 card5
	 * @return value of the hand
	 */
	public int eval_5cards(int c1, int c2, int c3, int c4, int c5) {
		int q;
		int s;

		q = (c1 | c2 | c3 | c4 | c5) >> 16;
			
		/// check for Flushes and StraightFlushes
		if ((c1 & c2 & c3 & c4 & c5 & 0xF000) != 0)
			return tables.flushes[q];

		s = tables.unique5[q];
		if (s != 0)
			return (s);

		/* let's do it the hard way
		 */
		q = (c1 & 0xFF) * (c2 & 0xFF) * (c3 & 0xFF) * (c4 & 0xFF) * (c5 & 0xFF);
		q = findit(q);
		
		return tables.values[q];
	}

	/**
	 * Obtains the hand type of a hand, given it's value.
	 * @param val Value of a poker hand.
	 * @return integer representation of the hand type. Use this to index value_str to obtain string representation.
	 */
	public int hand_rank(int val) {
		if (val > 6185)
			return (HIGH_CARD); // 1277 high card
		if (val > 3325)
			return (ONE_PAIR); // 2860 one pair
		if (val > 2467)
			return (TWO_PAIR); //  858 two pair
		if (val > 1609)
			return (THREE_OF_A_KIND); //  858 three-kind
		if (val > 1599)
			return (STRAIGHT); //   10 straights
		if (val > 322)
			return (FLUSH); // 1277 flushes
		if (val > 166)
			return (FULL_HOUSE); //  156 full house
		if (val > 10)
			return (FOUR_OF_A_KIND); //  156 four-kind
		return (STRAIGHT_FLUSH); //   10 straight-flushes
	}

	public final int STRAIGHT_FLUSH = 1;
	public final int FOUR_OF_A_KIND = 2;
	public final int FULL_HOUSE = 3;
	public final int FLUSH = 4;
	public final int STRAIGHT = 5;
	public final int THREE_OF_A_KIND = 6;
	public final int TWO_PAIR = 7;
	public final int ONE_PAIR = 8;
	public final int HIGH_CARD = 9;

	/**
	 * Getter for card rank for a card in integer representation.
	 * @param x a card.
	 * @return Rank of the given card.
	 */
	public int RANK(int x) {
		return ((x >> 8) & 0xF);
	}

	/// String representations for hand types.
	public static String value_str[] = { "", "Straight Flush",
			"Four of a Kind", "Full House", "Flush", "Straight",
			"Three of a Kind", "Two Pair", "One Pair", "High Card" };

}

