package com.asdf.plugins.pokergames;

import com.asdf.plugins.pokergames.PokerGame.Blind;

/**
 * interface for getting all the public information concerning one player
 * @author rimpila
 *
 */
public interface PlayerInfo {

	/**
	 * Get whether the player has sat out some blinds, and has to pay any as a penalty for entering the game
	 * @return missed blinds
	 */
	public Blind hasToPostBlind();
		
	
	/**
	 * Get the amount of the current bet that the player can call
	 * @return how much the player can pay of the amount he should call
	 */
	public double getAmountCallable() ;
		
		
	

	/**
	 * Obtain the amount the player has invested in the pot.
	 * @return the amount the player has in the pot of the current hand
	 */
	public double getAmountInPot() ;
		
		
	

	/**
	 * Obtain the amount the player has invested in the pot this round
	 * @return the amount the player has invested in the pot this round
	 */
	public double getAmountInPotThisRound() ;
		
		
	

	/**
	 * Get the maximum amount the player can raise.
	 * @return the amount the player will be able to raise,
	 * thus, either the upper limit for raises, or the total bankroll
	 */
	public double getAmountRaiseable() ;
		
		
	

	/**
	 * Determine the amount a player must pay to stay in the game
	 * @return the difference between the player who has bet most in the pot, and this players amount in the pot
	 */
	public double getAmountToCall() ;
		
		
	

	/**
	 * Get the current bankroll
	 * @return the amount of money the player has in his disposal in the table
	 */
	public double getBankRoll() ;
		
	


	/**
	 * Get the player's bankroll at the start of the hand (i.e., before any actions including antes and blinds).
	 * @return the amount the player had in his bankroll before any antes, blinds or bets
	 */
	public double getBankRollAtStartOfHand() ;
		
		
	

	/**
	 * Get the current bankroll as a multiple of the big blind
	 * @return the players bankroll divided by the size of the big blind
	 */
	public double getBankRollInSmallBets() ;
		
		
	

	/**
	 * Get the context of the last action made by this player.
	 * @return the GameInfo saved when doing latest action 
	 */
	public GameInfo getGameInfo() ;
		
		
	

	/**
	 * A single integer code for the last action made
	 * @return one of [Holdem.FOLD, Holdem.CALL, Holdem.RAISE]
	 */
	public int getLastAction() ;
		
		
	

	/**
	 * Obtain the player's name
	 * @return the name of the player
	 */
	public String getName() ;
		
		
	

	/**
	 * Get the net amount won or lost since the start of the current hand
	 * @return the difference between current bankroll and the bankroll at the start of hand
	 */
	public double getNetGain() ;
		
		
	

	/**
	 * Return the amount the player can raise, given the desired amount to raise.
	 * @param arg0 the amount desired to raise
	 * @return the amount player is able to raise given the desired
	 */
	public double getRaiseAmount(double arg0) ;
		
		
	

	/**
	 * Obtain the hand revealed by this player
	 * @return the players hole cards, if he has revealed them, otherwise 2 invalid cards, if he has two, otherwise empty hand
	 */
	public Hand getRevealedHand() ;
		
		
	

	/**
	 * Get the player's seat number in the GameInfo
	 * @return the players seat number
	 */
	public int getSeat() ;
		
		
	

	/**
	 * Check if a player has made at least one action this round
	 * @return true if the player has acted this round 
	 */
	public boolean hasActedThisRound() ;
		
		
	

	/**
	 * Check if the player has enough chips to complete a raise
	 * @return true if the player has enough for minimum raise after paying what he has to call
	 */
	public boolean hasEnoughToRaise() ;
		
		
	

	/**
	 * See if the player is dealt into the current game
	 * @return true if the player is dealt in
	 */
	public boolean inGame() ;
		
		
	

	/**
	 *  Check if the player is still active in the hand
	 *  @return true if the player is active in current hand
	 */
	public boolean isActive() ;
		
		
	

	/**
	 * Check if the player is all-in
	 * @return true if the player is all-in
	 */
	public boolean isAllIn() ;
		
		
	

	/**
	 * Check if the player is all-in
	 * @return true if the player is all-in
	 */
	public boolean isButton() ;
		
		


	/**
	 * Check if the player has voluntarily committed this round
	 * @return true if the player has committed
	 */
	public boolean isCommitted() ;
		
		
	

	/**
	 * Check if the player has folded
	 * @return true if the player has folded
	 */
	public boolean isFolded() ;
		
		
	

	/**
	 * Check if the player is sitting out
	 * @return true if the player is sitting out
	 */
	public boolean isSittingOut() ;
		
		
	/**
	 * Returns the account name of this player.
	 * @return Account name.
	 */
	public String getAccountName();
	
	}		


