package com.asdf.plugins.pokergames;

/**
 * Efficient bit scrabble implementation for a Player state.
 * Mostly checking n:th bit from state variable.
 * No comments necessary for methods.
 */
public class PlayerStates {
	
	//for the state of the player
	//default value is 0, so the player will start matching all, and only, the values with 0


	final static int IN_GAME   = 1;   // was dealt into game
	final static int ACTIVE    = 2;   // is still active in the hand
	final static int COMMITTED = 4;   // has committed (bet)
	final static int SIT_OUT   = 8;   // is sitting out
	final static int FOLDED    = 16;  // has folded
	
	
	/**
	 * Checks if the player's state says he is in game.
	 * @param state Integer representation of a player's state.
	 * @return true if the player is in game, false otherwise.
	 */
	public static boolean isInGame(int state) {
		return (state & IN_GAME) != 0;
	}

	/**
	 * Set the state to reflect that the player is in game
	 * @param state Integer representation of a player's state.
	 * @return state so that isInGame is true
	 */
	public static int inGame(int state) {
		return state |= IN_GAME;
	}
	
	/**
	 * Checks if the player's state says he is active.
	 * @param state Integer representation of a player's state.
	 * @return true if the player is active, false otherwise.
	 */
	public static boolean isActive(int state) {
		return (state & ACTIVE) != 0;
	}	
	
	/**
	 * Checks if the player's state says he is folded.
	 * @param state Integer representation of a player's state.
	 * @return true if the player has folded, false otherwise.
	 */
	public static boolean isFolded(int state) {
		return (state & FOLDED) != 0;
	}
	
	/**
	 * Set the state to reflect that the player has folded, and is not active
	 * @param state Integer representation of a player's state.
	 * @return state
	 */
	public static int folded(int state) {
		state |= FOLDED;
		state &= ~ACTIVE;
		return state;
	}
	
	/**
	 * Set the state to reflect that the player is in game, and active
	 * and not folded, nor sitting out
	 * @param state Integer representation of a player's state.
	 * @return state
	 */
	public static int activated(int state) {
		state |= IN_GAME | ACTIVE;
		return state &= ~0 ^ FOLDED ^ SIT_OUT;	
	}
	
	/**
	 * Checks if the player's state says he has committed funds.
	 * @param state Integer representation of a player's state.
	 * @return true if the player has committed, false otherwise.
	 */
	public static boolean isCommitted(int state) {
		return (state & COMMITTED) != 0;
	}
	
	/**
	 * Set the state to reflect that the player has committed
	 * @param state Integer representation of a player's state.
	 * @return state
	 */
	public static int committed(int state) {
		return state |= COMMITTED;
	}
	
	/**
	 * Set the state to reflect that the player not committed
	 * @param state Integer representation of a player's state.
	 * @return state
	 */
	public static int unCommitted(int state) {
		return state &= ~COMMITTED;
	}
	
	/**
	 * Checks if the player's state says he is sitting out.
	 * @param state Integer representation of a player's state.
	 * @return true if the player is sitting out, false otherwise.
	 */
	public static boolean isSittingOut(int state) {
		return (state & SIT_OUT) != 0;
	}

	/**
	 * Set the state to reflect that the player is sitting out
	 * @param state Integer representation of a player's state.
	 * @return state
	 */
	public static int sitOut(int state) {
		return state |= SIT_OUT;
	}
	
	/**
	 * Set the state to reflect that the player is in game
	 * @param state Integer representation of a player's state.
	 * @return state
	 */
	public static int sitIn(int state) {
		return state &= ~SIT_OUT;
	}
	
	
	/**
	 * ugly tests. Don't read this, it is meaningless.
	 * @param args ...
	 */
	
	public static void main(String[] args) {
		int state = 0;
		System.out.println(state);
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		System.out.println("ToGame:\t" + (state = inGame(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		System.out.println("ToPlay:\t" + (state = activated(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		System.out.println("Folds:\t"  + (state = folded(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		System.out.println("Coms:\t"   + (state = committed(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		System.out.println("unComs:\t" + (state = unCommitted(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		//System.out.println("Revs:\t" + (state = reveal(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		//System.out.println("unRevs:\t" + (state = hide(state)));
		System.out.print("\tSitOut:\t" + isSittingOut(state));
		System.out.print("\tInGame:\t" + isInGame(state));
		System.out.print("\tActive:\t" + isActive(state));
		System.out.print("\tFolded:\t" + isFolded(state));
		System.out.print("\tCommit:\t" + isCommitted(state));
		//System.out.println("\tReveal:\t" + isHandRevealed(state));
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println();
		
		System.out.println();
		
	}
}
