package com.asdf.plugins.pokergames;


import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.PriorityQueue;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.JAXBException;

import org.w3c.dom.DOMException;
import org.w3c.dom.Element;

import com.asdf.common.Action;
import com.asdf.common.Event;
import com.asdf.common.Game;
import com.asdf.common.GameTimerTask;
import com.asdf.common.PokerGameTypes;
import com.asdf.parser.ChatMessage;
import com.asdf.parser.Custom;
import com.asdf.parser.GameDetailsResponse;
import com.asdf.parser.GameRequest;
import com.asdf.parser.GameResponse;
import com.asdf.parser.GameStateUpdate;
import com.asdf.parser.GenericResponse;
import com.asdf.parser.Message;
import com.asdf.parser.OptionalParameters;
import com.asdf.parser.OptionalStringParameters;
import com.asdf.parser.Parameters;
import com.asdf.parser.StateUpdate;
import com.asdf.plugins.pokergames.texasholdem.parser.ActionMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.GameModelInfoMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserCard;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserHand;
import com.asdf.plugins.pokergames.texasholdem.parser.PlayerInfoFieldsMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.TexasEventMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.TexasMessage;
import com.asdf.utils.BasicElement;
import com.asdf.utils.XMLTools;

/**
 * Poker game class. Extendable to different poker games by extending to a subclass and implementing the interface stage for that game.
 * @author rimpila
 *
 */

public abstract class PokerGame extends Game {

	//states corresponding to stages
	public static final int WAIT_PLAYERS = 0;
	public static final int ANTE_ROUND = 1;
	public static final int BLIND_ROUND = 2;
	public static final int SET_DEALER = 3;
	public static final int NEW_DEAL = 4;	
	public static final int BET_ROUND = 5;
	public static final int DEAL_N_CARDS_TO_TABLE = 6;
	public static final int DEAL_N_CARDS_TO_HANDS = 7;
	public static final int DISCARD_CARDS = 8;
	public static final int WINNINGS = 9;
	public static final int MUCK_ROUND = 10;
	public static final int SHOWDOWN = 11;
	public static final int LESS_THAN_TWO_PLAYERS = 12;
	
	private int blindRoundCounter = 0;
	
	//blinds to pay as penalties
	public enum Blind {SMALL_BLIND (1,0),
				BIG_BLIND (0,1),
				BOTH_BLINDS (1,1),
				NO_BLINDS (0,0);
				
				public final double amount1;
				public final double amount2;
				
				private Blind(int amount1, int amount2) {
					this.amount1 = amount1;
					this.amount2 = amount2;
				}
				
				public double toPay(double smallBlind, double bigBlind) {
					return amount1*smallBlind+amount2*bigBlind;
					
				}
	};
	
	/// General timertask
	protected GameTimerTask timerTaskMuck;
	protected GameTimerTask timerTaskBlind;
	protected GameTimerTask timerTaskAction;
	protected GameTimerTask timerTaskDiscard;
	protected GameTimerTask timerTaskNextGame;
	protected GameTimerTask[] seatReserveTimerTasks;
	protected GameTimerTask[] freeSeatTimerTasks;
	protected long[] timerSessionBind;
	
	
	
	
	
	@Override
	public void die() {
		for(long sesID : this.gameWatchers)
			this.sendEvent(Event.GAMEOVER_EVENT, null, -1, -1, 0,  sesID);
	}
	
	
	//times to wait the blinds
	public int BLIND_WAIT = 15000;
	public int MUCK_LOSING_WAIT = 3000;
	public int MUCK_WINNING_WAIT = 3000;
	public int ACTION_WAIT = 15000;
	public int NEXT_GAME_WAIT = 10000;
	public int DISCARD_WAIT = 15000;
	public int RESERVE_WAIT = 30000;
	public static final int FREE_SEAT_WAIT = 120000;

	protected XMLTools xmltools = null;

	/** Contains game and player states */
	protected GameModel gameModel;

	/** Used to store hand statistics to database */
	protected StatisticsHand statsHand;
	
	protected int firstPlayer = -1;
	protected int winner = -1;
	protected Deck deck;
	protected Hand[] hands;
	protected Stage generatorStage= null;
	
	protected boolean forceBlind = false; //TODO if quick game wanted, set to true, and blinds will be posted automatically
	protected boolean won[]; //if the player won in the hand
	
	/** State machine is entering current state for the first time */
	protected boolean enterStage = true;
	protected int lastActed = -1;
	
	/** Used to keep track of which sessionID owns which seat */
	protected long[] PlayerSessionIDs;
	protected HandEval handEval;
	
	protected Collection<Long> gameWatchers; //SessionIDs for each client that should see the game
	protected Collection<Long> superUsers; //SessionIDs for each superuser client that should see the game
	protected Collection<Long> playerIDs; //SessionIDs for each player, to send only one message to multiple bots with the same SessionID
	protected SortedSet<PlayerPosition> winners;
	
	/**
	 * Handles requests
	 * @param sessionID Session ID of the requesting entity
	 * @param request GameRequest
	 */
	@Override
	protected void handleRequest(long sessionID, GameRequest request) {
		if (request.getJoin() != null) {
			if (request.getJoin().getParameters() != null && 
					request.getJoin().getParameters().getAny().size() > 0) {
				Element ele = request.getJoin().getParameters().getAny().get(0);
				
				if(ele.getAttribute("LobbyJoin").equals("true")) {
					this.sendUpdatedModel(sessionID);
				} else {
					String name = ele.getAttribute("Name");
					int bankRoll = Integer.parseInt(ele.getAttribute("Cash"));
					int mySeat   = Integer.parseInt(ele.getAttribute("Seat"));
					this.newPlayer(name, this.getAccountName(sessionID), bankRoll, mySeat, sessionID);
				}
			} else {
				this.newWatcher(sessionID);
			}
			GameResponse gameRes = new GameResponse();
			gameRes.setGameID(this.getId());
			GenericResponse gRes = new GenericResponse();
			gameRes.setJoin(gRes);
			gRes.setSuccess(true);
			Element e = new BasicElement("asdf");
			e.setAttribute("timer", "" + this.ACTION_WAIT);			
			Parameters pars = new Parameters();
			pars.getAny().add(getUpdatedModelElement());
			pars.getAny().add(e);
			gRes.setParameters(pars);
			send(sessionID, gameRes);
		} else if (request.getChat() != null) {
			String name = request.getChat().getSender();
			String message = request.getChat().getMessage();
			this.sendMessageToAll(message, name);
		} else if (request.getVoice() != null) {
			//this.sendVoiceToAll(request.getVoice(), sessionID);
		}
	}

	/**
	 * Handler for responses.
	 */
	@Override
	protected void handleResponse(long sessionID, GameResponse response) {

		TexasMessage texmex = null;
		
		try {
			texmex = (TexasMessage) xmltools.unmarshallNode(response.getCustom().getAny());
		} catch (DOMException e) {
			e.printStackTrace();
			return;
		} catch (JAXBException e) {
			e.printStackTrace();
			return;
		}
		
		if(texmex != null && texmex.getAction() != null){
				
			ActionMessage texact = texmex.getAction();
			
			int actIndex = texact.getIndex();
			Action action;
			switch(actIndex){
			case Action.JOIN_GAME:				
				action = Action.joinAction(texact.getName(), texact.getSeat(), texact.getAmount());
				break;
			case Action.RESERVE_SEAT:
				action = Action.reserveAction(texact.getSeat());
				break;
			case Action.LEAVE_GAME:
				action = Action.leaveGameAction(texact.getSeat(), texact.getAmount());
				break;
			case Action.DISCARD:
				int[] discards = new int[texact.getDiscarded().size()];
				for (int i = 0; i < texact.getDiscarded().size(); i++)
					discards[i] = texact.getDiscarded().get(i);
				action = Action.discardAction(discards);
				break;
			default:
				action = new Action(texact.getIndex(), texact.getAmount());
				break;
			
			}
							
			this.receiveAction(action, texact.getSeat(), sessionID);
		
		}
		else{
			
		}

	}
	
	/**
	 * Returns a general level description of the game.
	 */
	@Override
	protected com.asdf.parser.GameListResponse.Games.Game description() {
		if (this.gameModel == null)
			return null;
		com.asdf.parser.GameListResponse.Games.Game gameDesc = new com.asdf.parser.GameListResponse.Games.Game();
		gameDesc.setGameID(this.getId());
		gameDesc.setGameType("TexasHold'em");
		gameDesc.setName(this.getName());
		gameDesc.setPlayers(this.gameModel.getSittingPlayers());
		gameDesc.setMaxPlayers(this.gameModel.getMaxPlayers());
		String variant;
		switch(PokerGameTypes.values()[this.gameModel.getLimitStyle()]) {
		case HOLDEM_FIXED_LIMIT:
			variant = "limit";
			break;
		case HOLDEM_POT_LIMIT:
			variant = "pot-limit";
			break;
		case HOLDEM_NO_LIMIT:
			variant = "no-limit";
			break;
		default: variant = "VARIANT NOT RECOGNIZED!!";
		}
		gameDesc.setGameVariant(variant);

		return gameDesc;
	}

	/**
	 * Returns detailed information about the game.
	 */
	@Override
	protected void details(GameDetailsResponse a) {
		GameDetailsResponse.Players players = new GameDetailsResponse.Players();
		OptionalStringParameters spara = new OptionalStringParameters();
		
		// Columns
		List<String> lstr = spara.getString();
		lstr.add("Seat");
		lstr.add("Name");
		lstr.add("Cash");
		
		PlayerInfoFieldsMessage plrs[] = this.gameModel.getPlayers();
		for (int i = 0; i < plrs.length; i++) {
			if (plrs[i] != null) {
				GameDetailsResponse.Players.Player p = new GameDetailsResponse.Players.Player();
								
				OptionalParameters opars = new OptionalParameters();
				Parameters pars = new Parameters();
				
				Element e = new BasicElement("any");
				e.setAttribute("Name", plrs[i].getMyName());
				e.setAttribute("Seat", ""+plrs[i].getMySeat());
				e.setAttribute("Cash", ""+plrs[i].getBankRoll() +"$");
								
				pars.getAny().add(e);
				
				opars.setParameters(pars);
				p.setInfo(opars);
				
				/*OptionalStringParameters valuesParameters = new OptionalStringParameters();
				
				// One row
				List<String> values = valuesParameters.getString();
				values.add(((Integer)plrs[i].getMySeat()).toString());BLIND_WAI
				values.add(plrs[i].getMyName());
				values.add("human");
				
				p.setValues(valuesParameters);*/
								
				
				players.getPlayer().add(p);
			
			}else{
				GameDetailsResponse.Players.Player p = new GameDetailsResponse.Players.Player();
				
				OptionalParameters opars = new OptionalParameters();
				Parameters pars = new Parameters();
				
				Element e = new BasicElement("any");
				if(gameModel.isReserved(i))
					e.setAttribute("Name", "Reserved");
				else
					e.setAttribute("Name", "Free");
				e.setAttribute("Seat", ""+i);
				e.setAttribute("Cash", "-");
								
				pars.getAny().add(e);
				
				opars.setParameters(pars);
				p.setInfo(opars);
						
				players.getPlayer().add(p);
			}
		}
		a.setColumnNames(spara);
		a.setPlayers(players);
		a.setRules(new com.asdf.parser.OptionalStringParameters());
		
	}
	
	/**
	 * Populate Element named "asdf" with game model
	 * data.
	 * @return Element containing game model
	 */
	protected Element getUpdatedModelElement() {
		GameModelInfoMessage gmi = gameModel.getGameModelInfo();
		TexasMessage tex = new TexasMessage();
		
		tex.setGameInfo(gmi);
	
		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(tex);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
		return paa;
	}
	
	/**
	 * Populate Custom element to contain game model.
	 * @return Populated Custom element
	 */
	protected Custom getUpdatedModelCustom() {
		
		GameModelInfoMessage gmi = GameModel.cloneWithoutPreviousStates(gameModel).getGameModelInfo();
		TexasMessage tex = new TexasMessage();
		
		tex.setGameInfo(gmi);
		Custom custom = new Custom();
	
		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(tex);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
		custom.setAny(paa);
		return custom;
	}
	
	/**
	 * Sends the updated game model to a client specified by sessionId.
	 * @param sessionId Identifier for the client session in question.
	 */
	protected void sendUpdatedModel(long sessionID) {
		
		Message msg = new Message();
		msg.setSessionID(sessionID);
		
		StateUpdate res = new StateUpdate();
		msg.setStateUpdate(res);
		
		GameStateUpdate gres = new GameStateUpdate();
				
		gres.setCustom(getUpdatedModelCustom());
		res.setGame(gres);

		send(sessionID, gres);
	}
	
	/**
	 * Sends the new model to all users.
	 */
	protected void sendUpdatedModelToAll() {
		for (long player : this.playerIDs)
			this.sendUpdatedModel(player);
		for (long watcher : this.gameWatchers)
			if (!this.IDinTable(watcher))
				this.sendUpdatedModel(watcher);
	}
	
	/**
	 * Sends win event to every client observing the game, players included.
	 * @param winner The seat that won the round.
	 * @param amount How much he won.
	 */
	protected void sendWinEventToAll(int winner, double amount, int[] handPositions) {
		for (long player : this.playerIDs)
			this.sendWinEvent(winner, amount, handPositions, player);
		for (long watcher : this.gameWatchers)
			if (!this.IDinTable(watcher))
				this.sendWinEvent(winner, amount, handPositions, watcher);
	}
	
	/**
	 * Sends event to all watching the game.
	 * @param event Event
	 * @param seat Seat of the person relevant to the action (winner, in win event).
	 * @param cardPositions Position of cards for this event. May be null.
	 * @param amount Amount relevant to the event (amount won, in win event).
	 * @param index number corresponding to the action
	 * @param sessionID Session id of target.
	 */
	protected void sendEventToAll(Event event, int[] cardPositions , int seat, double amount, int index) {
		for (long player : this.playerIDs)
			this.sendEvent(event, cardPositions , seat, amount, index, player);
		for (long sessionID : this.gameWatchers)
			if (!this.IDinTable(sessionID))
				this.sendEvent(event, cardPositions , seat, amount, index, sessionID);
	}
	
	/**
	 * Sends event to one session.
	 * @param event Event
	 * @param seat Seat of the person relevant to the action (winner, in win event).
	 * @param cardPositions Position of cards for this event. May be null.
	 * @param amount Amount relevant to the event (amount won, in win event).
	 * @param sessionID Session id of target.
	 * @param index number corresponding to the action
	 */
	protected void sendEvent(Event event, int[] cardPositions , int seat, double amount, int index, long sessionID) {

		
		//TODO add int index to the message
		//GameResponse game = new GameResponse();
		GameStateUpdate up = new GameStateUpdate();	
		Custom custom = new Custom();
		TexasEventMessage texevent = new TexasEventMessage();
		texevent.setAmount(amount);
		texevent.setSeat(seat);
		texevent.setType(event.ordinal());
		if (cardPositions != null)	{
			for(Integer i : cardPositions)
			texevent.getCardPositions().add(i);
		}
		TexasMessage txmsg = new TexasMessage();
		ActionMessage actionMessage = null;
		
		if(index > -1){
			actionMessage = new ActionMessage();
			actionMessage.setIndex(index);
			actionMessage.setAmount(amount);
			actionMessage.setSeat(seat);
			if (cardPositions != null)
				for(int i : cardPositions)
					actionMessage.getDiscarded().add(i);
		}
		texevent.setAction(actionMessage);
		
		txmsg.setEvent(texevent);
		
		
		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(txmsg);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
		custom.setAny(paa);
		up.setCustom(custom);
		//game.setCustom(custom);
		send(sessionID, up);
		//send(sessionID, game);
		
	}
	
	/**
	 * Sends updated game model to the player whose turn it is now.
	 */
	protected void sendReserveConfirmation(boolean issuccess, String name, int seat, long sessionID){
		GameResponse game = new GameResponse();
		Custom custom = new Custom();
		ActionMessage act = new ActionMessage();
		act.setIndex(Action.RESERVE_SEAT);
		
		act.setName(name);
		if(issuccess)
			act.setSeat(seat);
		else
			act.setSeat(-1);
		
		TexasMessage txmsg = new TexasMessage();
		txmsg.setAction(act);
		
		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(txmsg);
		} catch (JAXBException e) {
			e.printStackTrace();
		};
		
		custom.setAny(paa);
		
		// TODO: issuccess??
		
		game.setCustom(custom);
		
		send(sessionID, game);
	}
	
	/**
	 * sends win event to given watcher
	 * @param winnerSeat seat of the winner
	 * @param amount he won
	 * @param cardPositions int[] of length of the hand evaluated for winning, has positions of the cards in all the cards available to the winner, starting with cards in hand
	 * @param sessionID address to send to
	 */
	protected void sendWinEvent(int winnerSeat, double amount, int[] cardPositions, long sessionID) {
		sendEvent(Event.WIN_EVENT, cardPositions, winnerSeat, amount, 0,  sessionID);
	}
	
	/**
	 * sends gamemodel to current player
	 */
	protected void sendUpdatedModelToCurrentPlayer() {
		if (this.gameModel.getCurrentPlayerSeat() >= 0)
		sendUpdatedModel(this.PlayerSessionIDs[this.gameModel.getCurrentPlayerSeat()]);
	}


	/**
	 * sends action to given watcher, used for informing success of reserved seat
	 * @param action given action
	 * @param sessionID
	 */
	protected void sendAction(Action action, long sessionID){

		GameResponse game = new GameResponse();
		Custom custom = new Custom();
		ActionMessage act = new ActionMessage();
		act.setAmount(action.getAmount());
		act.setSeat(action.seat());
		act.setIndex(action.getActionIndex());
		act.setName(action.name());
		act.setDiscards(action.discards());

		TexasMessage txmsg = new TexasMessage();
		txmsg.setAction(act);

		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(txmsg);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
		custom.setAny(paa);			
		game.setCustom(custom);
		
		send(sessionID, game);
	}
	

	
	/**
	 * Sends a chat message to one session.
	 * @param message Contents of the message.
	 * @param amount Amount.
	 * @param sessionID Session ID of the target player.
	 */
	protected void sendChatMessage(String name, String message, long sessionID) {		

		ChatMessage chat = new ChatMessage();
		chat.setSender(name);
		chat.setMessage(message);
		
		GameStateUpdate txmsg = new GameStateUpdate();
		txmsg.setChat(chat);
		send(sessionID, txmsg);
	}
	
	/**
	 * Send voice data to all except the sender.
	 * @param encodedData Voice data
	 * @param sessionID Sender's ID
	 */
	protected void sendVoiceToAll(String encodedData, long sessionID) {
		for (long player : this.playerIDs)
			if (player != sessionID) {
				GameStateUpdate txmsg = new GameStateUpdate();
				txmsg.setVoice(encodedData);
				send(player, txmsg);
			}
		for (long watcher : this.gameWatchers) {
			if (watcher != sessionID && !this.IDinTable(watcher)) {
				GameStateUpdate txmsg = new GameStateUpdate();
				txmsg.setVoice(encodedData);
				send(watcher, txmsg);
			}
		}
	}
	
	/**
	 * Sends a chat message to all clients.
	 * @param string Contents of the message
	 * @param name name of the sender.
	 */
	protected void sendMessageToAll(String string, String name) {
		for (long player : this.playerIDs)
			this.sendChatMessage(name, string, player);
		for (long watcher : this.gameWatchers)	
			if (!this.IDinTable(watcher))
				this.sendChatMessage(name,  string, watcher);
	}
	
	/**
	 * Sends chat from player to all clients
	 * @param string Contents of the message
	 * @param seat seat of the sender.	 
	 */
	protected void sendMessageToAll(String string, int seat) {
		String name = "Guest?";
		if (this.gameModel.getPlayer(seat) != null)
			name = this.gameModel.getPlayerName(seat);
		for (long player : this.playerIDs)
			this.sendChatMessage(name, string, player);
		for (long watcher : this.gameWatchers)			
			if (!this.IDinTable(watcher))
				this.sendChatMessage(name, string, watcher);
	}
	
	
		

	/**
	 * Sends discard event.
	 * @param amount
	 * @param seat The seat of local player.
	 * @param sessionID Session of the target.
	 */
	protected void sendDiscard(int amount, int seat, long sessionID){

		GameResponse game = new GameResponse();
		Custom custom = new Custom();
		ActionMessage act = new ActionMessage();
		act.setAmount((double)amount);
		act.setSeat(seat);		

		TexasMessage txmsg = new TexasMessage();
		txmsg.setAction(act);
		
		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(txmsg);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
		custom.setAny(paa);		
		game.setCustom(custom);
				
		send(sessionID, game);
	}

	/**
	 * Sends a hand to a single session.
	 * @param hand Hand to be sent.
	 * @param seat Seat to which the hand is associated.
	 * @param sessionID Session ID of the target client.
	 */
	protected void sendHand(Hand hand, int seat, long sessionID) {

		GameStateUpdate game = new GameStateUpdate();
		Custom custom = new Custom();
		ParserHand ph = new ParserHand();
		ph.setSeat(seat);
		
		for(Card c : hand.getCards()){
			ParserCard pc = new ParserCard();
			pc.setCard(c.getInt());
			ph.getCards().add(pc);
		}

		TexasMessage txmsg = new TexasMessage();
		txmsg.setHand(ph);

		Element paa = null;
		try {
			paa = xmltools.marshallObjectToElement(txmsg);
		} catch (JAXBException e) {
			e.printStackTrace();
		};
		
		custom.setAny(paa);
		game.setCustom(custom);
		
		send(sessionID, game);
	}
		
	/**
	 * sends hands visible to that user to each user
	 * sends each hand to the player it is dealt to, and to each superuser
	 */
	protected void sendVisibleHandsToAllViewers() {
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {			
			if (this.hands[i] != null) { //if hand exist, also player should exist 
				this.sendHand(hands[i], i, this.PlayerSessionIDs[i]); //sending to that player
				for (long SuperUser: this.superUsers) {
					this.sendHand(this.hands[i], i, SuperUser);
				}
			}
		}
	}

	/**
	 * adds a sessionID to the list of watchers, and sends them a gamemodel
	 * @param sessionID
	 */
	protected void newWatcher(long sessionID) {
		if (this.gameWatchers.contains(sessionID))
			return;
		this.gameWatchers.add(sessionID);
		this.sendUpdatedModel(sessionID);
		if (super.getUserType(sessionID) == com.asdf.common.AccountType.SUPER_USER)
			this.superUsers.add(sessionID);
	}
	
	/**
	 * Adds a player to the table
	 * @param name of the player to add
	 * @param bankRoll money the player brings to table
	 * @param mySeat the seat for the player
	 * @param sessionID of the player to add
 	 */
	protected void newPlayer(String name, String accountName, double bankRoll, int mySeat, long sessionID) {
		if (mySeat < 0 || mySeat >= this.hands.length)
			throw new InvalidSeatException(mySeat);
		if (this.PlayerSessionIDs[mySeat] != 0)
			throw new FullSeatException(mySeat);
		
		this.setBalanceChange("POKER", sessionID, -bankRoll);
		
		this.PlayerSessionIDs[mySeat] = sessionID;
		//this.playerInfos[mySeat] = new OurPlayerInfo(name, bankRoll, mySeat, this.gameModel);
		this.gameModel.setPlayer(mySeat, new OurPlayerInfo(name, accountName ,bankRoll, mySeat, this.gameModel));
		if (this.gameModel.getStage().index() != this.gameModel.getStage().start().index())
			this.gameModel.getOurPlayer(mySeat).setHasToPostBlind(Blind.BIG_BLIND);
		//this.players++;
		this.sendUpdatedModelToAll();
		this.playerIDs.add(sessionID);
		//this.newWatcher(sessionID);
		if (this.gameModel.Stage().correspondingCase() == 0) {
			this.updateStage();
		} 			
		//if this.stage this.updateStage(this.stageReady());		
	}
	
	/**
	 * Called when action is received
	 * @param action the action performed
	 * @param seat of the performer
	 * @param sessionID of the client sending the action
	 */
	private void receiveAction(Action action, int seat, long sessionID) {
		//this.sendMessageToAll("Saatiin: Action " + action.index + " stage on " +this.gameModel.getStage().index() + (seat >= 0 && this.gameModel.getPlayer(seat) != null ? " pelaajalta " +this.gameModel.getPlayerName(seat): "")+ (seat >= 0 && this.gameModel.getCurrentPlayerSeat() >= 0 && this.gameModel.getCurrentPlayer() != null ? " vuorossa on "+this.gameModel.getCurrentPlayer().getName() : ""), "SERVER");
		//these actions we can get at any stage
		switch (action.getActionIndex()) {
			case Action.JOIN_GAME:
				if (this.gameModel.isReserved(seat) && this.timerSessionBind[seat] == sessionID) {
					this.seatReserveTimerTasks[seat].cancel();
					this.seatReserveTimerTasks[seat] = null;
					this.newPlayer(action.name(), this.getAccountName(sessionID), action.getAmount(), action.seat(), sessionID);
				}
				return;
			case Action.RESERVE_SEAT:
				synchronized(this) {
					if (this.seatReserveTimerTasks[seat] != null || 
							this.PlayerSessionIDs[seat] != 0 || 
							this.gameModel.isReserved(seat)) {
						this.sendUpdatedModel(sessionID);
						this.sendAction(Action.reserveAction(-1), sessionID);
						return;
					}
					this.startTimerForReserveSeat(seat, sessionID);
				}
				this.gameModel.reserveSeat(seat);
				this.sendAction(Action.reserveAction(seat), sessionID);
				this.sendUpdatedModelToAll();
				return;
			case Action.LEAVE_GAME:
				if (action.amount < -1) { //wants to leave table
					this.gameWatchers.remove(sessionID);
					this.superUsers.remove(sessionID);
				}
				
				if (seat >= 0 && seat < this.gameModel.getMaxPlayers()) {
					
					// Player is removing himself
					if (this.PlayerSessionIDs[seat] == sessionID) {
						
						// User is leaving, return money on the table to the total balance
						// There might be nothing to return or there are more than was brought to the table
						setBalanceChange("POKER", sessionID, this.gameModel.getPlayer(seat).getBankRoll());
						
						this.PlayerSessionIDs[seat] = 0;
						this.playerIDs.remove(sessionID);
						if (this.IDinTable(sessionID))
							this.playerIDs.add(sessionID);
						this.gameModel.freeSeat(seat);
						
					// Player is removing someone else than himself
					// proper privileges must be checked
					} else if (this.gameModel.getPlayer(seat) != null && 
							(this.getAccountName(sessionID).equals(this.gameModel.getPlayer(seat).getAccountName()) ||
							(this.gameModel.getPlayer(seat).getBankRoll() <= 0 && this.gameModel.getPlayer(seat).getAmountInPot() <= 0) ||
							super.getUserType(sessionID) == com.asdf.common.AccountType.SUPER_USER)) {
						
						// User is leaving, return money on the table to the total balance
						// There might be nothing to return or there are more than was brought to the table
						setBalanceChange("POKER", sessionID, this.gameModel.getPlayer(seat).getBankRoll());
						
						long ID = this.PlayerSessionIDs[seat];
						this.playerIDs.remove(ID);
						this.PlayerSessionIDs[seat] = 0;
						this.gameModel.freeSeat(seat);
						this.sendEvent(Event.LEAVE_EVENT, null, seat, 0, Action.LEAVE_GAME, ID);
						if (this.IDinTable(ID))
							this.playerIDs.add(ID);
					}
				}
				this.sendUpdatedModelToAll();
				
				return;
			case Action.BACK:
				if (this.PlayerSessionIDs[seat] == sessionID && this.gameModel.getOurPlayer(seat) != null) {
					this.gameModel.getOurPlayer(seat).setState(PlayerStates.sitIn(this.gameModel.getOurPlayer(seat).getState()));
					//this.players++;
					if (this.gameModel.Stage().correspondingCase() == this.generatorStage.start().index() /*&& this.players >= this.gameModel.getMinPlayers()*/)
						this.updateNextStage();
				}
				return;
			case Action.SIT_OUT:
				if (this.gameModel.getPlayer(seat) != null)
					this.gameModel.getOurPlayer(seat).setState(PlayerStates.sitOut(this.gameModel.getOurPlayer(seat).getState()));
				break;
			}
	
		switch (this.gameModel.Stage().correspondingCase()) {
			//case WAIT_PLAYERS:
				//gets new players, and sit-ins, but done always?
			//case DEALER:
				//should not get anything
			case BLIND_ROUND:
				if (seat == this.gameModel.getCurrentPlayerSeat() && 
						(action.isPostBlind() ||
						 action.isSitout() || action.isSkipTurn() ||
						 action.isSmallBlind() || action.isBigBlind())) {
					synchronized(this) {
						if (this.timerTaskBlind == null)
							break;
						else {
							this.timerTaskBlind.cancel();
							this.timerTaskBlind = null;
						}
					}
					if (action.isSitout()) {
						this.updateStage();
					} else if (action.isSkipTurn()) {
						this.currentPlayerSkips();
						this.updateStage();
					} else if (action.isSmallBlind() && 
							((this.gameModel.getCurrentPlayer().hasToPostBlind() == Blind.SMALL_BLIND) ||
							 this.gameModel.getCurrentPlayerSeat() == this.gameModel.getSmallBlindSeat())){
						this.currentPlayerPays(this.gameModel.getSmallBlindSize());
						if (this.gameModel.getPlayer(seat) != null)
							this.gameModel.getOurPlayer(seat).setHasToPostBlind(Blind.NO_BLINDS);
						this.lastActed = seat;
						this.updateStage();
					} else if (action.isBigBlind() && 
							((this.gameModel.getCurrentPlayer().hasToPostBlind() == Blind.BIG_BLIND)||
							 this.gameModel.getCurrentPlayerSeat() == this.gameModel.getBigBlindSeat())){

						this.currentPlayerPays(this.gameModel.getBigBlindSize());
						// added 8/20/2008 by teemu:
						this.gameModel.addRaisesThisRound();
						if (this.gameModel.getPlayer(seat) != null)
							this.gameModel.getOurPlayer(seat).setHasToPostBlind(Blind.NO_BLINDS);
						this.lastActed = seat;
						this.updateStage();
					} else if (action.isPostBlind()) {
						if ((this.gameModel.getCurrentPlayerSeat() == this.gameModel.getSmallBlindSeat() &&
								this.gameModel.getCurrentPlayer().hasToPostBlind() == Blind.NO_BLINDS) ||
						  this.gameModel.getCurrentPlayer().hasToPostBlind() == Blind.SMALL_BLIND)
							this.currentPlayerPays(this.gameModel.getSmallBlindSize());
						else if (this.gameModel.getCurrentPlayerSeat() == this.gameModel.getBigBlindSeat() ||
						 this.gameModel.getCurrentPlayer().hasToPostBlind() == Blind.BIG_BLIND) {
							this.currentPlayerPays(this.gameModel.getBigBlindSize());
							// added 8/20/2008 by teemu:
							if (this.gameModel.getCurrentPlayerSeat() == this.gameModel.getBigBlindSeat()) {
								this.gameModel.addRaisesThisRound();
							}
						}
						else
							this.currentPlayerPays(this.gameModel.getCurrentPlayer().hasToPostBlind().toPay(this.gameModel.getSmallBlindSize(), this.gameModel.getBigBlindSize()));
						this.gameModel.getOurPlayer(seat).setHasToPostBlind(Blind.NO_BLINDS);
						this.lastActed = seat;
						this.updateStage();
					} else {
						//this.sendUpdatedModelToCurrentPlayer();
						this.startTimerForBlind();
						return;
					}
				}
				break;
				//gets blind posted or sit-out actions
			//case NEW_DEAL:
				//should not get anything
			//case FLOP:
				//should not get anything
			//case TURN:
				//should not get anything
			//case RIVER:
				//should not get anything
			case BET_ROUND:
				//raises, checks, folds, calls
				if (seat == this.gameModel.getCurrentPlayerSeat() && (action.isCall() || 
						action.isCheck() || action.isRaise() || 
						action.isBet() || action.isFold())) {
					synchronized(this) {
						if (this.timerTaskAction == null)
							break;
						else {
							this.timerTaskAction.cancel();
							this.timerTaskAction = null;
						}
					}
					switch (action.getActionIndex()) {
						case Action.CALL:
							this.currentPlayerCalls();
							break;
						case Action.CHECK:
							this.currentPlayerChecks();
							break;
						case Action.RAISE:
						case Action.BET:
							//in fixed-limit only 4 raises are possible per round
							if (this.gameModel.isFixedLimit() && this.gameModel.getNumRaises() >=4) { 
								this.currentPlayerCalls();
								break;
							}
							// sending only the amount the player wishes to raise, 
							// then adding the amount he needs to call
							this.currentPlayerRaises(action.getAmount());
							break;
						case Action.FOLD:
							this.currentPlayerFolds();
							break;
						default:
							//this.sendUpdatedModelToCurrentPlayer();
							this.startTimerForAction();
							return;
					}
					this.lastActed = seat;
					this.updateStage();
				}
				break;
				
			case LESS_THAN_TWO_PLAYERS:
				if (seat == this.gameModel.getCurrentPlayerSeat() && (action.isMuck() || action.isReveal())) {
					synchronized(this) {
						if (this.timerTaskMuck == null)
							break;
						else {
							this.timerTaskMuck.cancel();
							this.timerTaskMuck = null;
						}
					}
					if (action.isReveal()) {
						this.gameModel.getOurPlayer(seat).setHoleCards(this.hands[seat]);
					}
					this.updateStage();
				}
				break;
				
			//case WINNINGS:
				//should not get anything
				
			case MUCK_ROUND:
				if (action.isReveal() && this.gameModel.getPlayer(seat) != null) {
					this.gameModel.getOurPlayer(seat).setHoleCards(this.hands[seat]);
				}
				break;
			//case SHOWDOWN:
				//shows cards...
			case DISCARD_CARDS:
				if (this.gameModel.getCurrentPlayerSeat() == seat && action.isDiscard()) {
					synchronized(this) {
						if (this.timerTaskDiscard == null)
							break;
						else {
							this.timerTaskDiscard.cancel();
							this.timerTaskDiscard = null;
						}
					}
					if (action.isDiscard()) {
						if (action.discards() >= 1) {
							for (int i = 0; i < action.discards(); i++)
								this.hands[seat].removeCard(action.discards(i));
							for (int i = this.hands[seat].size(); i < this.gameModel.Stage().param(); i++)
								this.hands[seat].addCard(this.deck.dealCard());
						}
						this.sendDiscard((int)Math.round(action.getAmount()), seat, sessionID);
						this.gameModel.setNextActivePlayer();
						this.lastActed = seat;
						this.updateStage();
					} else {
						//this.sendUpdatedModelToCurrentPlayer();
						this.startTimerForDiscard();
					}
				}
				break;
				
				//default:
				//	throw new RuntimeException("Unknown case: " + action.getActionIndex());
		}
	}

	/**
	 * starts a new game with the given values
	 * @param smallBlind
	 * @param bigBlind
	 * @param minPlayers
	 * @param maxPlayers
	 * @param gameID
	 * @param stage
	 */
	protected abstract void newGame(double smallBlind, double bigBlind, int minPlayers, int maxPlayers, int gameID, Stage stage);
	
	/**
	 * calls updatestage with the current stage
	 */
	protected void updateStage() {
		this.updateStage(this.gameModel.getStage());
	}
	
	/**
	 * calls updateStage with the stage after the current stage
	 */
	protected void updateNextStage() {
		this.enterStage = true;
		this.updateStage(this.gameModel.getStage().next());
	}
	
	/**
	 * 
	 * @param stage to move the game to
	 */
	protected void updateStage(Stage stage) {
		int iterations = 0;
		
		while(stage != null) {
			stage = updateStageRepeat(stage);
			iterations++;
			
			if(iterations > 20)
				throw new RuntimeException("updateStage repeats like hell. PokerGame.");
		}
	}
	
	/**
	 * Performs the actions needed for the given current stage and updates
	 * the stage to the one that should be executed next.
	 * @param stage Current stage.
	 * @return The stage that should be executed next.
	 */
	protected Stage updateStageRepeat(Stage stage) {
		this.gameModel.setStage(stage);
		
		switch(stage.correspondingCase()) {
		case WAIT_PLAYERS:		
			this.clearGameTable();
			this.sendUpdatedModelToAll();
			int activePlayers = 0;
			
			for(int i=0; i<this.gameModel.getMaxPlayers(); i++)
				if(this.gameModel.getPlayer(i) != null && !this.gameModel.getPlayer(i).isSittingOut())
					activePlayers++;
			
			if(activePlayers < this.gameModel.getMinPlayers()) {
				return null; // don't repeat anymore. wait for triggers.
			}
			
			this.setRandomDealer();
			this.gameModel.setMinPlayers(2);
			this.activateAllNotSittingOut();
			this.enterStage = true;
					
			return stage.next();
			
		case SET_DEALER:
			this.activateAllNotSittingOut();
			if(!this.isGameActive()) {
				this.enterStage = true;
				return stage.start();
			}
		
			this.sendEventToAll(Event.GAMESTART_EVENT, null, -1, -1, -1);
			this.clearGameTableExceptDealer();

			this.gameModel.setButtonSeat(this.gameModel.nextActivePlayer(this.gameModel.getButtonSeat()));
			if (this.gameModel.getCountablePlayers() > 2) // if game is Heads-up, go to reverse-blinds
				this.firstPlayer = this.gameModel.getButtonSeat();
			else
				this.firstPlayer = this.gameModel.getSmallBlindSeat();
			this.gameModel.setCurrentPlayerSeat(this.firstPlayer);
			this.clearRoundInfo();
			this.enterStage = true;
			return stage.next();
			
			
		case BLIND_ROUND:
			
			blindRoundCounter++;
			
			if (this.enterStage) {
				this.blindRoundCounter = 1;
				this.enterStage = false;
				this.activateAllNotSittingOut();

				this.gameModel.setSmallBlindSeat(this.gameModel.getSmallBlindSeatFromButton());
				this.gameModel.setBigBlindSeat(this.gameModel.getBigBlindSeatFromButton());
				
				// Initialize statistics
				statsHand = new StatisticsHand(this.getDbConnection(), this.gameModel);
				
			} else {
				
				if (/*this.gameModel.getCurrentPlayer() != null && */
						(this.gameModel.getCurrentPlayerSeat() != this.lastActed && 
						this.currentPlayerToPostBlind() && this.gameModel.getCurrentPlayer().inGame()) 
						|| (this.gameModel.getCurrentPlayer().getBankRoll() <= 0 && 
							this.gameModel.getCurrentPlayer().getAmountInPot() <= 0)) {
					this.sitPlayerOut();
				}
			}
			
			this.gameModel.setNextPlayer();
			
			// if this check is true, scream, a lot.
			if(blindRoundCounter > this.gameModel.getMaxPlayers() * 2) {				
				throw new RuntimeException("Seems like a loop forever in BLIND ROUND @ PokerGame.");
			}
			
			if (!this.isGameActive()) {

			
				this.setPenaltiesAway();
				this.gameModel.setCurrentPlayerSeat(-1);
				this.sendUpdatedModelToAll();
				this.setActivePlayerToCurrent();
				this.enterStage = true;
				return stage.lessThanTwoPlayers();
			}

			this.penaltiesForSittingOutBeforeBlind();


			//has paid, round is complete, moving to next stage
			if (/*this.gameModel.getCurrentPlayer() != null &&*/ this.gameModel.getCurrentPlayer().getAmountInPot() > 0) {
				this.enterStage = true;
				return stage.next();
			}
			
			//does not have to pay, going to next player
			if (!this.currentPlayerToPostBlind() || (/*this.gameModel.getCurrentPlayer() != null &&*/ !this.gameModel.getCurrentPlayer().inGame())) {
				return stage;
			}

			if (this.forceBlind) {
				
				double amount = 0;
				if (this.gameModel.getCurrentPlayerSeat() == this.gameModel.getBigBlindSeat()) {
					amount = this.gameModel.getBigBlindSize();
				} else {
					amount = this.gameModel.getCurrentPlayer().hasToPostBlind().toPay(this.gameModel.getSmallBlindSize(), this.gameModel.getBigBlindSize());
					if (this.gameModel.getCurrentPlayerSeat() == this.gameModel.getSmallBlindSeat())
						amount += this.gameModel.getSmallBlindSize();
				}
				this.currentPlayerPays(amount);
			}
					
			this.sendUpdatedModelToAll();
			this.startTimerForBlind();
			return null; // stop repetition
			
			
		case NEW_DEAL:
			
			this.deck = new Deck();
			int playersDealtIn = 0;
			this.dealNCardsToAllPlayers(stage.param());
			for (int i = 0; i < this.hands.length; i++)
				if (this.hands[i] != null)
					playersDealtIn++;
			
			this.sendVisibleHandsToAllViewers();
			this.sendEventToAll(Event.HOLECARDS_EVENT, null, -1, 0, -1);
			if (this.gameModel.getCountablePlayers() > 2) // if game is Heads-up, go to reverse-blinds, setting previous from the player to act
				this.firstPlayer = this.gameModel.getBigBlindSeat();
			else
				this.firstPlayer = this.gameModel.getBigBlindSeat();
			this.gameModel.setCurrentPlayerSeat(this.firstPlayer);

			
			this.enterStage = true;
			return stage.next();

		case DEAL_N_CARDS_TO_HANDS:
						
			this.dealNCardsToAllPlayers(stage.param());
			this.sendVisibleHandsToAllViewers();
			if (this.gameModel.getCountablePlayers() > 2) //game is Heads-up, go to reverse-blinds
				this.firstPlayer = this.gameModel.getBigBlindSeat();
			else
				this.firstPlayer = this.gameModel.getButtonSeat();
			this.gameModel.setCurrentPlayerSeat(this.firstPlayer);
			this.enterStage = true;
			
			
			return stage.next();

		case BET_ROUND:
						
			if (this.enterStage) {
				this.enterStage = false;
				//extra 3.7.:
				this.setMinRaise(this.getStartMinRaise());
				//System.out.println(this.gameModel.getStage().index());
				//System.out.println(this.gameModel.getMinRaise());
				
				
			} else {
				this.gameModel.decreasePlayersToAct();
				if (this.lastActed != this.gameModel.getCurrentPlayerSeat()) {
					this.currentPlayerFolds();
					this.gameModel.decreasePlayersToAct();
				}
			}
				
			if (!this.isGameActive()) {
				this.gameModel.setCurrentPlayerSeat(-1);
				this.sendUpdatedModelToAll();
				this.setActivePlayerToCurrent();
				this.enterStage = true;
				return stage.lessThanTwoPlayers();
			}
			
			if (this.gameModel.getUnacted() <= 0) {
				this.gameModel.setCurrentPlayerSeat(-1);
				this.sendUpdatedModelToAll();
				this.enterStage = true;
				this.clearRoundInfo();
				
				// Store cards on the board
				this.statsHand.setCards(this.gameModel.getBoardCards().getCards());
				
				// Store state at the end of each bet round
				this.statsHand.setBetround(
						(short)this.gameModel.getNumActivePlayers(),
						this.gameModel.getPot());
				
				return stage.next();
			} else if (this.gameModel.getUnacted() < 0)
				throw new RuntimeException("Negative number of people who are expected to act! PokerGame: BET_ROUND");

			this.askForNextBet();
			return null; // wait for the bet message to arrive.
			
			
		case DEAL_N_CARDS_TO_TABLE:
			
			this.dealNCardsToTable(this.gameModel.Stage().param());
			this.firstPlayer = this.gameModel.getButtonSeat();
			this.gameModel.setCurrentPlayerSeat(this.firstPlayer);
			this.enterStage = true;
			return stage.next();

		case DISCARD_CARDS:	
			// This doesn't actually work? But that's ok for TexasHoldem as this round is never visited.
			this.gameModel.setNextActivePlayer();
			if (this.gameModel.getNumToAct() == 0) {
				this.gameModel.maxPlayersToAct();
				return stage.next();
			}
			
			this.gameModel.decreasePlayersToAct();
			this.sendUpdatedModelToAll();
			this.startTimerForDiscard();
			return null;
			
		case LESS_THAN_TWO_PLAYERS:
			if (this.enterStage) {
				this.enterStage = false;
				this.statsHand.setStateAtEnd(this.gameModel, this.hands);
			}
			else {
				this.enterStage = true;
				return stage.next();
			}
			
			if (this.gameModel.getNumActivePlayers() < 1) {
				this.enterStage = true;
				return stage.next();
			}
			if (this.gameModel.getNumActivePlayers() > 0)
				this.setActivePlayerToCurrent();			
				this.winners = this.currentPlayerWinsAll();
			
			if (this.winners != null && this.winners.first() != null && this.hands[winners.first().getSeat()] != null)
				this.sendUpdatedModel(this.PlayerSessionIDs[winners.first().getSeat()]);
			else {
				this.enterStage = true;
				this.clearRoundInfo();
				return stage.next();
			}
			
			this.startTimerForMuckWinning();
			return null; // wait for answer
			
			
		case WINNINGS:
			
			if (this.enterStage) {
				this.enterStage = false;
				this.gameModel.setCurrentPlayerSeat(-1);
				this.sendUpdatedModelToAll();
				this.statsHand.setStateAtEnd(this.gameModel, this.hands);
			}

			SortedSet<PlayerPosition> handOrder= this.orderPlayersByHandAndPot();

			PlayerPosition winner;
			int sharingPot = 1;
			this.winners = new TreeSet<PlayerPosition>();
			double winnings;
			LinkedList<PlayerPosition> sharingList = new LinkedList<PlayerPosition>();
			
			
			while (!handOrder.isEmpty() && this.gameModel.getTotalPotSize() > 0) {
				//get the hand of greatest value and least cash in pot.
				winner = handOrder.first();
				handOrder.remove(winner);
				//see if pot is shared
				PlayerPosition next = null;
				while (!handOrder.isEmpty() && winner.getHandValue() == handOrder.first().getHandValue()) {
					next = handOrder.first();
					sharingList.add(next);
					handOrder.remove(next);
					sharingPot++;
				}
				//for each player remove the amount from that players hand that he should get, if he's all in, he should get less
				winnings = this.countSharedPotForThisPlayersHandValue(winner);
				
				//update winnings pot and bankRoll
				for (PlayerPosition player : sharingList) {
					player.addWinnings(winnings/sharingPot);
				}
				
				winner.addWinnings(winnings/sharingPot);
				this.gameModel.setPot(this.gameModel.getPot() - winnings);
				//this.gameModel.getOurPlayer(winner.getSeat()).increaseBankRoll(winner.getWinnings());
				
				//save winner list to send in showdown
				if (winner.getWinnings() > 0) {
					this.won[winner.getSeat()] = true;
					this.winners.add(winner);
				}
				
				//if pot is empty, also adds the players from sharinglist to the winner, as they won't otherwise be gone through
				if (this.gameModel.getTotalPotSize() <= 0)
					for (PlayerPosition player : sharingList)
						this.winners.add(player);
				else //adds sharingList to the handOrder, so they will be sharing in the remaining pot
					handOrder.addAll(sharingList);
				
				//set the values for next winner
				sharingList.clear();
				sharingPot = 1;
			}
			
			for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
				if (won[i] && this.gameModel.getPlayer(i) != null) {
					this.gameModel.getOurPlayer(i).setHoleCards(this.hands[i]);
				}
				else if (this.gameModel.getPlayer(i) != null && this.gameModel.getPlayer(i).isActive()) {
					this.gameModel.setCurrentPlayerSeat(i);
					this.sendUpdatedModel(this.PlayerSessionIDs[i]);
				}
			}

			this.enterStage = true;
			return stage.next();

			
			
		case MUCK_ROUND:
			
			if (this.enterStage) {
				this.enterStage = false;
			}
			else {
				this.enterStage = true;
				this.clearRoundInfo();
				return stage.next();
			}
			
			this.startTimerForMuckLosing();
			for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
				if (won[i] && this.gameModel.getPlayer(i) != null) {
					this.gameModel.getOurPlayer(i).setHoleCards(this.hands[i]);
				}
				else if (this.gameModel.getPlayer(i) != null && this.gameModel.getPlayer(i).isActive()) {
					this.gameModel.setCurrentPlayerSeat(i);
					this.sendUpdatedModel(this.PlayerSessionIDs[i]);
				}
			}
			
			return null; // wait for answers?
			
		case SHOWDOWN:

			if (this.enterStage) {
				this.enterStage = false;
			}
			else {
				this.enterStage = true;
				return stage.next();
			}

			this.gameModel.setCurrentPlayerSeat(-1);
			this.sendUpdatedModelToAll();
			
			this.sendEventToAll(Event.SHOWDOWN_EVENT, null, -1, -1, -1);
			
			if (this.winners != null) {
				int handValue = this.winners.first().getHandValue();
				for (PlayerPosition player : this.winners) {
					this.playerWins(player, player.getHandValue()<=handValue);
					
					// Store each player winning amount to statistics
					this.statsHand.getPlayer(player.getSeat()).setWinnings(player.getWinnings());
				}
			}
			
			// Store statistics information to database
			this.statsHand.storeToDb();
			
			this.winners = null;
				//this.sendWinEventToAll(player.getSeat(), player.getWinnings(), player.getWinningCardsPositions());
			for (int i=0; i < this.gameModel.getNumSeats(); i++)
				if (this.gameModel.getPlayer(i) != null && this.gameModel.getPlayer(i).getBankRoll() <= 0) {
					this.gameModel.getOurPlayer(i).setState(PlayerStates.sitOut(0));
					//this.startTimerForFreeSeat(i); removed
				}
			startWaitForNextGame();
			this.sendEventToAll(Event.HANDOVER_EVENT, null, -1, -1, -1);
			this.clearRoundInfo();
			return null;
			
			default:
				return null;
		}
		
	}
	
	
	/**
	 * counts the pot the player with this hand would win in this pot given the amount he has invested in the pot, 
	 * if his hand is the only one with this value
	 * @param winner
	 * @return the amount he would win
	 */
	private double countSharedPotForThisPlayersHandValue(PlayerPosition winner) {
		System.out.println(winner);
		double winnings = 0;
		for (int i = 0; i < this.hands.length; i++) {
			if (i != winner.getSeat() && this.gameModel.getPlayer(i) != null) {
				if (this.gameModel.getPlayer(i).getAmountInPot() <= this.gameModel.getPlayer(winner.getSeat()).getAmountInPot()) {
					winnings += this.gameModel.getOurPlayer(i).getInPot();
					this.gameModel.getOurPlayer(i).setInPot(0);
				} else {
					winnings += this.gameModel.getPlayer(winner.getSeat()).getAmountInPot();
					this.gameModel.getOurPlayer(i).setInPot(this.gameModel.getPlayer(i).getAmountInPot() 
						- this.gameModel.getPlayer(winner.getSeat()).getAmountInPot());
				}
			}					
		}
		if (this.gameModel.getPlayer(winner.getSeat()) != null) {
			winnings += this.gameModel.getPlayer(winner.getSeat()).getAmountInPot();
			this.gameModel.getOurPlayer(winner.getSeat()).setInPot(0);
		}
		return winnings;
	}
	
	/**
	 * returns an sorted collection of all players active in pot, starting with the player with the best hand,
	 * for players with equal hands the one who has invested the least is first
	 * @return
	 */
	protected abstract SortedSet<PlayerPosition> orderPlayersByHandAndPot();

	private void activateAllNotSittingOut() {
		//this.gameModel.setPlayersDealtIn(0);
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++)
			if (this.gameModel.getPlayer(i) != null && !this.gameModel.getPlayer(i).isSittingOut()) {
					this.gameModel.getOurPlayer(i).setState(PlayerStates.activated(this.gameModel.getOurPlayer(i).getState()));
					//this.gameModel.increasePlayersDealtIn();
			}
	}

	/**
	 * clear all info relating to the last game from the table,
	 * except keeps the Button seat the same
	 */
	protected void clearGameTableExceptDealer() {
		this.clearRoundInfo();
		for (int i = 0; i < this.hands.length; i++) {
			this.hands[i] = new Hand();
		}
		
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
			this.hands[i] = null;
			if (this.gameModel.getOurPlayer(i) != null) {
				this.gameModel.getOurPlayer(i).setBankRollAtStartOfHand(this.gameModel.getPlayer(i).getBankRoll());
				this.gameModel.getOurPlayer(i).setLastAction(Action.INVALID);
				this.gameModel.getOurPlayer(i).setInPot(0);
				this.gameModel.getOurPlayer(i).setHoleCards(null);
				this.gameModel.getOurPlayer(i).setLastGameInfo((GameModel)null);
			}
		}
		this.gameModel.setBoardCards(new Hand()); //dealt during the game
		
		this.gameModel.setCurrentPlayerSeat(-1);
		//this.gameModel.setPlayersDealtIn(0);
		this.gameModel.setRaisesThisRound(0);
		this.gameModel.setPot(0);
		//this.gameModel.setSidepots(/*new LinkedList<Sidepot>()*/ null);
		//this.gameModel.setStage(stage);
		this.gameModel.setPlayersToAct(0);		
		this.gameModel.zeroWinners();
	}

	/**
	 * clear all info from the table relating to the most recent hand
	 */
	protected void clearGameTable() {
		this.gameModel.setButtonSeat(-2);
		this.clearGameTableExceptDealer();
		
	}
	
	/**
	 * gives the minimum starting raise for the current round
	 * @return minimum raise
	 */
	protected abstract double getStartMinRaise();
	
	/**
	 * clears all info regarding only the previous round
	 */
	protected void clearRoundInfo() {
		this.lastActed = -1;
		this.gameModel.setRaisesThisRound(0);
		this.gameModel.maxPlayersToAct();
		//this.gameModel.setMinRaise(this.getStartMinRaise());
		this.setMinRaise(this.getStartMinRaise());
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
			if (this.gameModel.getOurPlayer(i) != null) {
				this.gameModel.getOurPlayer(i).setHasActedThisRound(false);
				this.gameModel.getOurPlayer(i).setInPotThisRound(0);
			}
		}
	}
	
	/**
	 * 
	 * @return true if the game has enough active players to continue
	 */
	protected boolean isGameActive() {
		return (this.gameModel.getNumActivePlayers() >= this.gameModel.getMinPlayers());
	}
	
	/**
	 * check if the current player has to pay blinds
	 * i.e. he is either in a blind seat, or has to pay some penalty
	 * @return
	 */
	private boolean currentPlayerToPostBlind() {
		if  ( this.gameModel.getCurrentPlayer() != null &&
				(this.gameModel.getCurrentPlayer().hasToPostBlind() != Blind.NO_BLINDS ||
				this.gameModel.getCurrentPlayerSeat() == this.gameModel.getSmallBlindSeat() ||
				this.gameModel.getCurrentPlayerSeat() == this.gameModel.getBigBlindSeat()))
			return true;
		return false;
	}	
	
	/**
	 * deals hand cards to all players
	 * @param n amount of cards to deal
	 */
	protected void dealNCardsToAllPlayers(int n) {
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
			if (this.gameModel.getPlayer(i) != null && this.gameModel.getPlayer(i).inGame()) {
				this.hands[i] = new Hand();
				this.gameModel.getOurPlayer(i).setHoleCards(new Hand());
				this.gameModel.getOurPlayer(i).setState(PlayerStates.inGame(this.gameModel.getOurPlayer(i).getState()));
				for (int j = 0; j < n ;j++) {
					this.hands[i].addCard(deck.dealCard());
					this.gameModel.getOurPlayer(i).getHoleCards().addCard(new Card()); //invalid card for the rest to see back
				}
				this.gameModel.activatePlayer(i);
			}
		}
	}
	
	/**
	 * deals cards to table
	 * @param n amount of cards to deal
	 */
	protected void dealNCardsToTable(int n) {
		for (int i = 0; i < n; i++)
			this.gameModel.addCardToBoard(this.deck.dealCard());		
	}
	
	/**
	 * goes through all the players that can be declared sitting out from the next round without having to ask them
	 * and sets their penalties
	 */
	protected void penaltiesForSittingOutBeforeBlind() {
		while (this.gameModel.getCurrentPlayer() != null && this.isGameActive() && 
				(this.gameModel.getCurrentPlayer().isSittingOut() || this.gameModel.getCurrentPlayer().getBankRoll() <= 0)){
			this.sitPlayerOut();
			this.gameModel.setNextPlayer();
		}
	}
	
	/**
	 * removes blind penalties from each player
	 */
	protected void setPenaltiesAway() {
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
			if (this.gameModel.getPlayer(i) != null)
				this.gameModel.getOurPlayer(i).setHasToPostBlind(Blind.NO_BLINDS);
		}
	}
	
	/**
	 * Should be only called when there is one player active
	 * Sets the active player to be the current one
	 */
	protected void setActivePlayerToCurrent() {
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++) {
			if (this.gameModel.getPlayer(i) != null && this.gameModel.getPlayer(i).isActive()) {
				this.gameModel.setCurrentPlayerSeat(i);
				this.winner = i;
				break;
			}
		}
	}
	

	/**
	 * randomly chooses a dealer from the players not sitting out
	 */
	protected void setRandomDealer() {
		PriorityQueue<SeatCard> heap = new PriorityQueue<SeatCard>();
		Deck deck = new Deck();
		deck.shuffle();
		for (int i=0; i < this.gameModel.getMaxPlayers(); i++) {
			if(this.gameModel.getPlayer(i) != null && !this.gameModel.getPlayer(i).isSittingOut())
				heap.add(new SeatCard(i, deck.dealCard()));
		}

		this.gameModel.setButtonSeat(this.gameModel.previousPlayer(heap.peek().getSeat()));
		this.sendMessageToAll("Randomly selected Dealer to be "+this.gameModel.getPlayerName(this.gameModel.getButtonSeat()), "");
	}
	
	/**
	 * Gets the next active player, and asks for his, her, or its action
	 * If the player is all-in, nothing is asked
	 */
	protected void askForNextBet() {
		this.gameModel.setNextActivePlayer();
		if (this.gameModel.getCurrentPlayer().isAllIn()) {
			this.lastActed = this.gameModel.getCurrentPlayerSeat();
			this.updateStage(this.gameModel.getStage());
			return;
		}
		this.sendUpdatedModelToAll();
		this.startTimerForAction();
	}	
	
	/**
	 * sets the current player to sit out, and sets penalties
	 * if this player is in a blind seat, shift the seats accordingly
	 */
	protected void sitPlayerOut() {
		this.sitPlayerOut(this.gameModel.getCurrentPlayerSeat());
		//this.gameModel.setNextPlayer();
	}
	
	/**
	 * called when a player sends a skip action on the blind round
	 * updates the player so that he does not participate in this round,
	 * but will be asked to join the next one
	 * also adds blind penalties, if applicable
	 */
	private void currentPlayerSkips() {
		int currentSeat = this.gameModel.getCurrentPlayerSeat();
		if (this.gameModel.getPlayer(currentSeat) != null) {
			Blind amount = this.gameModel.getOurPlayer(currentSeat).hasToPostBlind();
			if (currentSeat == this.gameModel.getSmallBlindSeat()) {
				switch (amount) {
				//case SMALL_BLIND:
				case BIG_BLIND:
					amount = Blind.BOTH_BLINDS;
					break;
				case NO_BLINDS:
					amount = Blind.SMALL_BLIND;
					break;
				//case BOTH_BLINDS:					
				}
				this.gameModel.moveSmallBlindSeat();
			}
			else if (currentSeat == this.gameModel.getBigBlindSeat()) {
				switch (amount) {
				case SMALL_BLIND:
					amount = Blind.BOTH_BLINDS;
					break;
				//case BIG_BLIND:
				case NO_BLINDS:
					amount = Blind.BIG_BLIND;
				//case BOTH_BLINDS:
				}
				this.gameModel.moveBigBlindSeat();
			}

			this.lastActed = currentSeat;
			this.gameModel.getOurPlayer(currentSeat).setHasToPostBlind(amount);
			this.gameModel.getOurPlayer(currentSeat).setState(0);
		}
	}
	
	/**
	 * sets the given player to sit out, and sets penalties
	 * if this player is in a blind seat, shift the seats accordingly
	 * @param seat player to sit out
	 */
	protected void sitPlayerOut(int seat) {
		int currentSeat = seat;
		if (this.gameModel.getPlayer(currentSeat) != null) {
			Blind amount = this.gameModel.getOurPlayer(currentSeat).hasToPostBlind();
			if (currentSeat == this.gameModel.getSmallBlindSeat()) {
				switch (amount) {
				//case SMALL_BLIND:
				case BIG_BLIND:
					amount = Blind.BOTH_BLINDS;
					break;
				case NO_BLINDS:
					amount = Blind.SMALL_BLIND;
					break;
				//case BOTH_BLINDS:
				}
				this.gameModel.moveSmallBlindSeat();
			}
			else if (currentSeat == this.gameModel.getBigBlindSeat()) {
				switch (amount) {
				case SMALL_BLIND:
					amount = Blind.BOTH_BLINDS;
					break;
				//case BIG_BLIND:
				case NO_BLINDS:
					amount = Blind.BIG_BLIND;
				//case BOTH_BLINDS:
				}
				this.gameModel.moveBigBlindSeat();
			}
			//this.gameModel.decreasePlayersDealtIn();
			
			this.gameModel.getOurPlayer(currentSeat).setHasToPostBlind(amount);
			this.gameModel.getOurPlayer(currentSeat).setState(
					PlayerStates.sitOut(0));		
		}
	}	

	/**
	 * called when a player sends a raise action, updates his, her, or its structures accordingly
	 * @param amount the amount the player wishes to raise
	 */
	protected void currentPlayerRaises(double amount) {
		OurPlayerInfo current = this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat());
		if (current != null) {
			amount = (this.gameModel.getMinRaise() > amount ? this.gameModel.getMinRaise() : amount);
			amount = (current.getAmountRaiseable() < amount ? current.getAmountRaiseable() : amount);
			if (amount <= 0) {
				this.currentPlayerCalls();
				return;
			}
			this.sendEventToAll(Event.ACTION_EVENT, null, current.getSeat(), amount, Action.RAISE);
			current.setLastAction(Action.RAISE);
			this.gameModel.addRaisesThisRound();
			this.gameModel.maxPlayersToAct();
			double called = current.getAmountCallable(); 
			this.currentPlayerPays(amount+called);
			this.setMinRaise(amount+called);
		}
	}
	
	/**
	 * sets the minimum raise given the last amount paid
	 * @param amount the amount that was last paid
	 */
	protected abstract void setMinRaise(double amount);

	/**
	 * Called when a player sends a call action, updates his, her or its structures accordingly
	 * if there is nothing to call, makes that player check instead
	 */
	protected void currentPlayerCalls() {
		OurPlayerInfo current = this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat());
		if (current != null) {
			if (current.getAmountToCall() <= 0) {
				this.currentPlayerChecks();
				return;
			}
			double amount = current.getAmountCallable();
			this.sendEventToAll(Event.ACTION_EVENT, null, current.getSeat(), amount, Action.CALL);
			current.setLastAction(Action.CALL);
			this.currentPlayerPays(amount);
		}
		
		
	}
	
	/**
	 * Called when a player pays something, for example call, raise or blind
	 * removes the amount from bankroll, and puts it in pot
	 * @param amount amount paid
	 */
	protected void currentPlayerPays(double amount) {
		OurPlayerInfo current = this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat());
		if (current != null) {
			
			amount = (current.getBankRoll() < amount ? current.getBankRoll() : amount);
			
			if(amount > current.getBankRoll()) {
				amount = current.getBankRoll();
				throw new RuntimeException("MUAHHAHAA!! I TOLD YOU!!");
			}
			
			if (amount <= 0)
				amount = 0;
			
			current.setLastGameInfo(this.gameModel.getGameModelInfo());
			current.setState(PlayerStates.committed(current.getState()));
			current.decreaseBankRoll(amount);
			current.increaseInPot(amount);
			current.increaseInPotThisRound(amount);
			
			current.setHasActedThisRound(true);
	
			this.gameModel.setPot(this.gameModel.getPot() + amount);
		}
	}
	
	/**
	 * called when a player sends a Fold action
	 * updates his, her, or its structures accordingly
	 * I also evoked when the player fails to send an action within given time limits
	 */
	protected void currentPlayerFolds() {
		OurPlayerInfo current = this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat());
		if (current != null) {
			this.sendEventToAll(Event.ACTION_EVENT, null, current.getSeat(), 0, Action.FOLD);
			current.setLastGameInfo(GameModel.clone(this.gameModel));
			current.setLastAction(Action.FOLD);
			current.setHasActedThisRound(true);
			current.setHoleCards(null);
			this.hands[current.getSeat()] = null;
			this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat()).setState(PlayerStates.folded(this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat()).getState()));
		}
	}
	
	/**
	 * called when a player sends a Check action
	 * updates his, her or its structures accordingly
	 * If that player has something to call, makes that player call instead
	 */
	protected void currentPlayerChecks() {
		OurPlayerInfo current = this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat());
		if (current != null) {
			if (current.getAmountToCall() > 0) {
				this.currentPlayerCalls();
				return;
			}
			this.sendEventToAll(Event.ACTION_EVENT, null, current.getSeat(), 0, Action.CHECK);
			current.setLastGameInfo(GameModel.clone(this.gameModel));
			current.setLastAction(Action.CHECK);
			current.setHasActedThisRound(true);
		}
	}
	
	/**
	 * Called when only one player remains active, prepares his, her or its winnings to be sent in the Showdown
	 * @return SortedSet containing only the winner
	 */
	protected SortedSet<PlayerPosition> currentPlayerWinsAll() {
		if (this.gameModel.getCurrentPlayer() != null) {
			if (this.gameModel.getPot() == this.gameModel.getSmallBlindSize()) {
				this.gameModel.getOurPlayer(this.gameModel.getCurrentPlayerSeat()).increaseBankRoll(this.gameModel.getPot());
				return null;
			}
			TreeSet<PlayerPosition> onlyWinner = new TreeSet<PlayerPosition>();
			PlayerPosition oneWinner = new PlayerPosition(this.gameModel.getCurrentPlayerSeat(), this.gameModel.getCurrentPlayer().getAmountInPot(), null);
			oneWinner.addWinnings(this.gameModel.getPot());
			onlyWinner.add(oneWinner);
			return onlyWinner;
		}
		return null;
	}
	
	/**
	 * Called when a player is awarded a win
	 * @param player PlayerPosition with the information
	 * @param isBestHandValue boolean for the hand being the best in the current played hand, and should be shown
	 */
	protected void playerWins(PlayerPosition player, boolean isBestHandValue /*int player, double amount, int[] handPositions*/) {
		if (player != null && this.gameModel.getPlayer(player.getSeat()) != null)
			this.playerWins(player.getSeat(), player.getWinnings(), player.getWinningCardsPositions(), player.getHandValue(), isBestHandValue);
	}
	
	/**
	 * Called when a player is awarded a win, updates the bankroll of that player, and sends win messages
	 * @param player winning player
	 * @param amount pot share won
	 * @param handPositions positions of the best hand that can be made from that player's hand cards, and cards from the table
	 * @param handValue value for the best hand
	 * @param isBestHandValue true if the hand was the best hand in this game
	 */
	protected void playerWins(int player, double amount, int[] handPositions, int handValue, boolean isBestHandValue) {
		if (this.gameModel.getPlayer(player) != null) {
			this.gameModel.getOurPlayer(player).increaseBankRoll(amount);
			this.gameModel.increaseWinners();
			this.sendWinEventToAll(player, amount, (isBestHandValue ? handPositions : null)); //not sending hand if it is not the best
			if (handValue >= 0 && isBestHandValue)
				this.sendMessageToAll("*"+this.gameModel.getPlayer(player).getName() + " wins "  +amount+ " with a "+ 
					HandEval.value_str[handEval.hand_rank(handValue)] +"!*", "");
			else
				this.sendMessageToAll("*"+this.gameModel.getPlayer(player).getName()+ " wins " +amount+"*", "");
		}
	}
	/*
	protected void initTimer(long delay) {
		// Cancel possible previous timer
		if (timerTask != null)
			timerTask.cancel();
		
		// Init timer to call updateStage
		timerTask = new GameTimerTask(this) {

			@Override
			public void onTick() {
				//Logger.get().fine("Timer lauennut");
				//sendMessageToAll("timer laukesi", "SERVER");
				updateStage();
			}
			
		};
		super.setTimer(timerTask, delay);
		Logger.get().fine("Laitettiin Timer");
	}
	*/
	
	/**
	 * intitializes a timer for seat reservation
	 */
	protected void initSeatTimer(long delay, int seat) {

		final int a = seat;
		
		this.seatReserveTimerTasks[seat] = new GameTimerTask(this) {
			
			@Override
			public void onTick() {
				cancelSeatReserve(a);
			}	
		};
		
		super.setTimer(this.seatReserveTimerTasks[seat], delay);
	}
	
	/**
	 * removes the reservatio from a given seat
	 * @param seat
	 */
	protected void cancelSeatReserve(int seat) {
		this.gameModel.freeSeat(seat);
		this.seatReserveTimerTasks[seat].cancel();
		this.seatReserveTimerTasks[seat] = null;
		this.sendUpdatedModelToAll();
	}
	
	/**
	 * Empties the given seat, if the player has no bankroll left
	 * @param seat
	 */
	protected void freeSeatIfNoBankroll(int seat) {
		if (this.gameModel.getPlayer(seat) != null &&
				this.gameModel.getPlayer(seat).getBankRoll() <= 0) {
			this.PlayerSessionIDs[seat] = 0;
			this.gameModel.freeSeat(seat);
		}
		this.freeSeatTimerTasks[seat] = null;
	}
	
	/**
	 * starts timer to wait for a blind posting from a player
	 * onTick it updates game stage
	 */
	protected void startTimerForBlind() {
		//initTimer(BLIND_WAIT);
		if (timerTaskBlind != null)
			timerTaskBlind.cancel();
		
		// Init timer to call updateStage
		timerTaskBlind = new GameTimerTask(this) {

			@Override
			public void onTick() {
				//Logger.get().fine("Timer lauennut");
				updateStage();
			}
			
		};
		super.setTimer(timerTaskBlind, BLIND_WAIT);
		//Logger.get().fine("Laitettiin Timer");
	}
	
	/**
	 * starts timer to wait for action from a player
	 * onTick it updates game stage
	 */
	protected void startTimerForAction() {
		//initTimer(ACTION_WAIT);
		if (timerTaskAction != null)
			timerTaskAction.cancel();
		
		// Init timer to call updateStage
		timerTaskAction = new GameTimerTask(this) {

			@Override
			public void onTick() {
				//Logger.get().fine("Timer lauennut");
				updateStage();
			}
			
		};
		super.setTimer(timerTaskAction, ACTION_WAIT);
		//Logger.get().fine("Laitettiin Timer");
	}
	
	/**
	 * starts timer to wait for muck action from the only player still active
	 * onTick it updates game stage
	 */
	protected void startTimerForMuckWinning() {
		//initTimer(ACTION_WAIT);
		if (timerTaskMuck != null)
			timerTaskMuck.cancel();
		
		// Init timer to call updateStage
		timerTaskMuck = new GameTimerTask(this) {

			@Override
			public void onTick() {
				//Logger.get().fine("Timer lauennut");
				updateStage();
			}
			
		};
		super.setTimer(timerTaskMuck, MUCK_WINNING_WAIT);
		//Logger.get().fine("Laitettiin Timer");
	}
	
	/**
	 * starts timer to wait for muck announcements from losing players
	 * onTick it updates game stage
	 */
	protected void startTimerForMuckLosing() {
		//initTimer(MUCK_LOSING_WAIT);
		if (timerTaskMuck != null)
			timerTaskMuck.cancel();
		
		// Init timer to call updateStage
		timerTaskMuck = new GameTimerTask(this) {

			@Override
			public void onTick() {
				//Logger.get().fine("Timer lauennut");
				updateStage();
			}
			
		};
		super.setTimer(timerTaskMuck, MUCK_LOSING_WAIT);
		//Logger.get().fine("Laitettiin Timer");
	}
	
	/**
	 * starts timer to give players time for reading hand results
	 * onTick it updates game stage, moving on to the next hand
	 */
	protected void startWaitForNextGame() {
		//initTimer(NEXT_GAME_WAIT);
		if (timerTaskNextGame != null)
			timerTaskNextGame.cancel();
		
		// Init timer to call updateStage
		timerTaskNextGame = new GameTimerTask(this) {

			@Override
			public void onTick() {
				//Logger.get().fine("Timer lauennut");
				updateStage();
			}
			
		};
		super.setTimer(timerTaskNextGame, NEXT_GAME_WAIT);
		//Logger.get().fine("Laitettiin Timer");
	}
	
	/**
	 * starts timer to wait for discard from a player
	 * onTick it updates game stage
	 */
	protected void startTimerForDiscard() {
		//initTimer(DISCARD_WAIT);
		if (timerTaskDiscard != null)
			timerTaskDiscard.cancel();
		
		// Init timer to call updateStage
		timerTaskDiscard = new GameTimerTask(this) {

			@Override
			public void onTick() {
				updateStage();
			}
			
		};
		super.setTimer(timerTaskMuck, DISCARD_WAIT);
	}
	
	/**
	 * starts timer for a seat reservation for given ID
	 * if the timer runs out, the seat is freed
	 * @param seat
	 * @param sessionID
	 */
	protected void startTimerForReserveSeat(int seat, long sessionID) {
		this.timerSessionBind[seat] = sessionID;
		this.initSeatTimer(RESERVE_WAIT, seat);
	}
	
	/**
	 * starts a timer to free the seat if after the time the player in the seat has no bankroll
	 * unused now, but can be used for cleaning old bots
	 * @param seat
	 */
	protected void startTimerForFreeSeat(int seat) {
		final int a = seat;
		if (this.freeSeatTimerTasks[a] == null) {
			freeSeatTimerTasks[a] = new GameTimerTask(this) {

				@Override
				public void onTick() {
					freeSeatIfNoBankroll(a);
				}
				
			};
			super.setTimer(freeSeatTimerTasks[a], FREE_SEAT_WAIT);
		}
	}
	
	/**
	 * 
	 * @param sessionID
	 * @return true if given sessionID is included in the IDs of players in the table
	 */
	private boolean IDinTable(long sessionID) {
		for (int i = 0; i< this.PlayerSessionIDs.length; i++)
			if (this.PlayerSessionIDs[i] == sessionID)
				return true;
		return false;
	}
	
}

class SeatCard implements Comparable<SeatCard>{
	int seat;
	Card card;
	
	SeatCard(int seat, Card card) {
		this.card = card;
		this.seat = seat;
	}
	
	int getSeat() {
		return this.seat;
	}
	
	Card getCard(){
		return this.card;
	}

	@Override
	public int compareTo(SeatCard arg0) {
		return this.card.compareTo(arg0.getCard());
	}
}