package com.asdf.plugins.pokergames;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

import com.asdf.common.Logger;

/**
 * Class containing hand statistics and functionality
 * to store it to the database. This class is general for
 * current implemented poker variants. It is also used on
 * unimplemented poker variants even it may not make completely
 * sense.
 */
public class StatisticsHand {
	
	private StatisticsPlayer[] players;
	private short dealer;
	private String flop;
	private String turn;
	private String river;
	private String showdown;
	private String[] cards;
	
	private Connection dbConnection;
	private short nrBetRounds = 0; 
	
	public StatisticsHand(Connection conn, GameModel model) {
		this.dbConnection = conn;
		players = new StatisticsPlayer[model.getMaxPlayers()];
		for(int i = 0; i < model.getMaxPlayers(); i++) {
			PlayerInfo player = model.getPlayer(i);
			if (player != null && player.inGame())
				players[i] = new StatisticsPlayer(model.getPlayer(i));
		}
		this.dealer = (short)model.getButtonSeat();
		this.cards = new String[0];
	}
	
	/**
	 * Set board cards.
	 * @param cards_
	 */
	public synchronized void setCards(List<Card> cards_) {
		this.cards = new String[cards_.size()];
		for(int i = 0; i < cards_.size(); i++) {
			this.cards[i] = cards_.get(i).toString();
		}
	}
	
	/**
	 * Set ending state.
	 * @param model
	 * @param hands
	 */
	public synchronized void setStateAtEnd(GameModel model, Hand[] hands) {
		// Store each player's amount in pot
		for (int i = 0; i < model.getMaxPlayers(); i++) {
			PlayerInfo p = model.getOurPlayer(i);
			if (p != null && p.inGame()) {
				StatisticsPlayer plr = players[p.getSeat()];
				plr.setMoneyIn(p.getAmountInPot());
				Hand h = hands[i];
				if (h != null) {
					Card card1 = h.getCard(0);
					Card card2 = h.getCard(1);
					plr.setCards(new String[]{card1.toString(), card2.toString()});
				}
			}	
		}
	}
	
	/**
	 * Set bet round's end situation.
	 * @param activePlayers
	 * @param pot
	 */
	public synchronized void setBetround(short activePlayers, double pot) {
		String description = activePlayers + "/" + pot;
		//System.out.println("setBetround: " + description);
		assert(nrBetRounds >= 0 && nrBetRounds < 4);
		switch(nrBetRounds) {
			case 0:
				flop = description;
				break;
			case 1:
				turn = description;
				break;
			case 2:
				river = description;
				break;
			case 3:
				showdown = description;
				break;
			default:
				break;
		}
		nrBetRounds++;
		
	}
	
	/**
	 * Return player from given seat index
	 * @param index Zero based index to seat
	 * @return StatisticsPlayer instance
	 */
	public synchronized StatisticsPlayer getPlayer(int index) {
		return players[index];
	}
	
	/**
	 * @return object as nice string containing all
	 * fields.
	 */
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("<StatisticsHand");
		sb.append(" dealer=" + dealer);
		sb.append(" flop=" + flop);
		sb.append(" turn=" + turn);
		sb.append(" river=" + river);
		sb.append(" showdown=" + showdown);
		sb.append(" cards=");
		for (String card : cards)
			sb.append("(" + card + ")");
		sb.append(">\n");
		for (StatisticsPlayer p : players) {
			sb.append("\t" + p + "\n");
		}
		return sb.toString();
	}
	
	public void storeToDb() {
		Thread t = new Thread(new Runnable() 
		{
			public void run() {
				storeToDbRun();
			}
		});
		t.start();
	}
	
	/**
	 * Storing collected statistics to database. Database table containing
	 * statistics is defined with following sql script:
	 * 
	 * CREATE TABLE hand (
	 * 	hand_id integer NOT NULL,
	 *  timestamp timestamp NOT NULL,
	 * 	dealer smallint NOT NULL,
	 * 	flop varchar(256),
	 * 	turn varchar(256),
	 * 	river varchar(256),
	 * 	showdown varchar(256),
	 * 	card1 char(3),
	 * 	card2 char(3),
	 * 	card3 char(3),
	 * 	card4 char(3),
	 * 	card5 char(3),
	 * 	PRIMARY KEY (hand_id)
	 * );
	 * 
	 * 
	 */
	public synchronized void storeToDbRun() {
		try {
			int handID = nextFreeHandID();
			PreparedStatement ps = dbConnection.prepareStatement(
					"INSERT INTO hand VALUES(?, NOW(), ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
			ps.setInt(1, handID);
			ps.setShort(2, dealer);
			ps.setString(3, flop);
			ps.setString(4, turn);
			ps.setString(5, river);
			ps.setString(6, showdown);
			for(int i = 0; i < 5; i++) {
				if (i < cards.length) 
					ps.setString(7+i, cards[i]);
				else
					ps.setNull(7+i, Types.CHAR);
			}
			ps.execute();
			
			for (StatisticsPlayer player : players) {
				if (player != null)
					player.storeToDb(dbConnection, handID);
			}
			
		} catch (SQLException e) {
			Logger.get().warning("Database problems, can't store statistics");
			Logger.get().throwing("StatisticsHand", "storeToDB", e);
		}
		
	}
	
	/**
	 * Find out next free hand ID.
	 * @return Free hand ID
	 * @throws SQLException 
	 */
	private int nextFreeHandID() throws SQLException {
		int handID = 0;
		PreparedStatement ps = dbConnection.prepareStatement("SELECT MAX(hand_id) from hand");
		if (ps.execute()) {
			ResultSet rs = ps.getResultSet();
			rs.next();
			handID = rs.getInt(1) + 1;
		}
		
		return handID;
	}
}
