package com.asdf.plugins.pokergames.texasholdem.bots;

import com.asdf.plugins.pokergames.*;

/**
 * Class for comparing hands
 * @author teemu
 *
 */

public class HandEvaluator {

	private HandEval eval;
	
	private final static int AHEAD = 0;
	private final static int TIED = 1;
	private final static int BEHIND = 2;
	
	/**
	 * Default constructor
	 */
	public HandEvaluator() {
	
		eval = new HandEval(new EvalTables());
	}
	
	/**
	 * Gives a numerical ranking of the hand
	 * @param c1 Hole card 1
	 * @param c2 Hole card 2
	 * @param board Current board
	 * @return The numerical ranking of the hand
	 */
	public int rankHand(Card c1, Card c2, Hand board) {
		
		if (board.size() == 3) {
			return eval.eval_5cards(c1.getInt(), c2.getInt(), board.getCard(0).getInt(), board.getCard(1).getInt(), board.getCard(2).getInt());
		}
		else if (board.size() == 4) {
			int[] ii = {c1.getInt(), 
						c2.getInt(), 
						board.getCard(0).getInt(), 
						board.getCard(1).getInt(), 
						board.getCard(2).getInt(), 
						board.getCard(3).getInt()};
			return rankHand(eval.eval_6cards(ii));
		}
		else if (board.size() == 5) {
			int[] ii = {c1.getInt(), 
					c2.getInt(), 
					board.getCard(0).getInt(), 
					board.getCard(1).getInt(), 
					board.getCard(2).getInt(), 
					board.getCard(3).getInt(),
					board.getCard(4).getInt()};
			return rankHand(eval.eval_7cards(ii));
		}
		else 
			return -1;
	}

	/**
	 * Gives a numerical ranking of the best hand
	 * @param bh Best hand
	 * @return The numerical ranking of the best hand
	 */
	private int rankHand(BestHand bh) {
		return bh.value;
	}
	
	/**
	 * Calculates the probability of having the best hand against random opponents
	 * @param c1 Hole card 1
	 * @param c2 Hole card 2
	 * @param board Current board
	 * @param opps Number of opponents
	 * @return hand strength
	 */
	public double handRank(Card c1, Card c2, Hand board, int opps) {
		int ourRank = rankHand(c1, c2, board);
		int oppRank;
		int index;
		double[] hands = new double[3];
		double hs = 0.0;
		
		Deck d = new Deck();
	    d.extractCard(c1);
	    d.extractCard(c2);
	    d.extractHand(board);
		
	    for (int i=0; i<d.getNumberOfCards(); i++) {
	         Card o1 = d.getCard(i);
	         // pick second opponent card
	         for (int j=i+1; j<d.getNumberOfCards(); j++) {
	            Card o2 = d.getCard(j);

	            oppRank = rankHand(o1,o2,board);
	            if (ourRank < oppRank) index = AHEAD;
	            else if (ourRank == oppRank) index = TIED;
	            else index = BEHIND;
	            hands[index]++;
	         }
	    }
	    hs = (hands[AHEAD]+hands[TIED]/2)/(hands[AHEAD]+hands[TIED]+hands[BEHIND]);
	    
	    return Math.pow(hs, opps);
	}
	
	public double potential(Card c1, Card c2, Hand bd, boolean positive) {
	      double[][]  HP = new double[3][3];
	      double[]    HPTotal = new double[3];
	      int         ourrank7, opprank;
	      int         index;
	      Hand        board = new Hand(bd);
	      int         ourrank5 =  rankHand(c1,c2,bd);

	      // remove all known cards
	      Deck d = new Deck();
	      d.extractCard(c1);
	      d.extractCard(c2);
	      d.extractHand(board);

	      // pick first opponent card
	      for (int i=0; i<d.getNumberOfCards(); i++) {
	         Card o1 = d.getCard(i);
	         // pick second opponent card
	         for (int j=i+1; j<d.getNumberOfCards(); j++) {
	            Card o2 = d.getCard(j);

	            opprank = rankHand(o1,o2,bd);
	            if (ourrank5 > opprank) index = AHEAD;
	            else if (ourrank5 == opprank) index = TIED;
	            else index = BEHIND;
	            HPTotal[index]++;

	            // tally all possiblities for next board card
	            for (int k=0; k<d.getNumberOfCards(); k++) {
	               if (i == k || j == k) continue;
	               board.addCard(d.getCard(k));
	               ourrank7 = rankHand(c1,c2,board);
	               try {
	            	   opprank = rankHand(o1,o2,board);
	               }
	               catch (Exception e) {
	            	   System.out.println("Error! c1 - rank, suit: " + o1.rank() +","+ o1.suit());
	            	   System.out.println("c2 - rank, suit: " + o2.rank() +","+ o2.suit());
	               }
	               if (ourrank7 > opprank) HP[index][AHEAD]++;
	               else if (ourrank7 == opprank) HP[index][TIED]++;
	               else HP[index][BEHIND]++;
	               board.removeCard();
	            }
	         }
	      } /* end of possible opponent hands */

	      double ppot = 0, npot = 0;
	      double den1 = (45*(HPTotal[BEHIND] + (HPTotal[TIED]/2.0)));
	      double den2 = (45*(HPTotal[AHEAD] + (HPTotal[TIED]/2.0)));
	      if (den1 > 0) {
	         ppot = (HP[BEHIND][AHEAD] + (HP[BEHIND][TIED]/2.0) +
	                (HP[TIED][AHEAD]/2.0)) / (double)den1;
	      }
	      if (den2 > 0) {
	         npot = (HP[AHEAD][BEHIND] + (HP[AHEAD][TIED]/2.0) +
	                (HP[TIED][BEHIND]/2.0)) / (double)den2;
	      }
	      if (positive) return ppot;
	      else return npot;
	   }
}
