package com.asdf.client;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.nio.channels.spi.SelectorProvider;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.xml.bind.JAXBException;

import com.asdf.common.ChangeRequest;
import com.asdf.parser.Message;
import com.asdf.utils.XMLTools;
import com.asdf.utils.ClientServerConstants;



/**
 * ConnectionHandler is a class which connects to the server, queues and sends
 *  messages to server. ConnectionHandler also receives messages from server
 *  and passes them to gameclients and lobbyclient running on client machine.
 */

public class ConnectionHandler implements Runnable {
    
    public enum State
    {
        /** Waiting for the size of the message */
        WAITINGSIZE,
        /** Waiting for the actual message */
        WAITINGDATA
    }
    
    
     // Are we expecting message size or actual data. Probably not needed.  
    private State receivingState = State.WAITINGSIZE;
    
    //The host:port combination to connect to         
    private InetAddress hostAddress;
    private int port;
 
    public boolean running;
    
    SocketChannel socket;

    /** Observer for connecting and disconnecting events */
    private ClientStateObserver observer;

    /** The selector we'll be monitoring */
    private Selector selector;

    /** The buffer into which we'll read data when it's available */
    private ByteBuffer readBuffer = ByteBuffer.allocate(1024);
    
    /** The buffer into which we'll read size of the message */
    private ByteBuffer sizeBuffer = ByteBuffer.allocate(4);

    /** A list of PendingChange instances */
    private List<ChangeRequest> pendingChanges = new LinkedList<ChangeRequest>();

    /** Maps a SocketChannel to a list of ByteBuffer instances */
    private Map<SocketChannel, List<ByteBuffer>> pendingData = new HashMap<SocketChannel, List<ByteBuffer>>();
    
    /** Maps a SocketChannel to a RspHandler */
    private Map<String, MessageHandler> rspHandlers = Collections.synchronizedMap(new HashMap<String, MessageHandler>());
    
    /** XMLTools instance for (un)marshalling messages */
    private XMLTools xmltools;
    
    private int numRead = 0;
    
    /**
     * Constructor for ConnectionHandler.
     * 
     * @param hostAddress Servers ip address in dot notation for example 98.162.0.27
     * @param port Servers port for example 8080
     * @throws IOException if connection could not be established
     */
    public ConnectionHandler(String hostAddress, int port) throws IOException {
        
    	this.xmltools = new XMLTools();
        this.hostAddress = InetAddress.getByName(hostAddress);
        this.port = port;
        
        // Start a new connection
        this.selector = this.initSelector();
        socket = this.initiateConnection();        
    }
    
    /**
     * 
     * @param observer
     */
    public void registerConnectionObserver(ClientStateObserver observer) {
    	this.observer = observer;
    }
    
    /**
     * Game and lobby clients register their response handlers to ConnectionHandler if
     * they want to receive messages from server.
     * 
     * @param handler Game or lobby clients message handler
     * @return false if response handler is already registered
     */
    public boolean registerMessageHandler(MessageHandler handler){
        if(!this.rspHandlers.containsKey(""+handler.getID())){
            this.rspHandlers.put(""+handler.getID(), handler);
            return true;
        }else{
            return false;
        }
    }

    
    /**
     *  Removes a registered response handler.
     * @param handler response handler to be unregistered
     * @return false if response handler could not be found
     */
    public boolean unRegisterMessageHandler(MessageHandler handler){
       
        if(this.rspHandlers.remove(handler.getID()) == null){           
            return false;
        }else{            
            return true;
        }
    }
    
    /**
     * Queues message to be send to server.
     * 
     * @param msg message to be send
     * @throws IOException  If connection to server is lost.
     * @throws JAXBException If message could not be translated to bytes. This means that the 
     * message did not match our message.xsd schema.
     */

    public void sendMessage(Message msg) throws IOException, JAXBException{
    	send(xmltools.messageToBytes(msg));
    }
    
    /**
     * Queues data to be send to a server. Size of the data
     * is sent as binary network byte order before the actual data.
     * 
     * @param data data to be written
     * @throws IOException If connection to server is lost.
     */
    public void send(byte[] data) throws IOException {
    	//System.out.println("Sent {" + new String(data) + "}");

        // Allocate send buffer and put data length as a "header"       
        ByteBuffer buf = ByteBuffer.allocate(data.length + 4);

        buf.putInt(data.length);
        buf.put(data);
        buf.rewind();

        sendRaw(buf);
    }
    
    /**
     * Queues data to be send to the server as it is.
     * @see send to send data with prepending size of the data
     * @param data to be written
     * @throws IOException If connection to server is lost.
     */
    public void sendRaw(ByteBuffer data) throws IOException {
    	// Queue the data we want written
        synchronized (this.pendingData) {
            List<ByteBuffer> queue = this.pendingData.get(socket);
            if (queue == null) {
                queue = new ArrayList<ByteBuffer>();
                this.pendingData.put(socket, queue);
            }            
            queue.add(data);            
        }

        // Finally, wake up our selecting thread so it can make the required changes              
        synchronized(this.pendingChanges) {
            this.pendingChanges.add(new ChangeRequest(socket, ChangeRequest.CHANGEOPS, SelectionKey.OP_WRITE));            
        }
        
        this.selector.wakeup();    
    }
    
    /**
     * Starts the ConnectionHandler.
     */

    public void run() {
    	running = true;
        while (running) {
            runOnce();
        }
        
//        if (observer != null) {
//    		observer.disconnected(null);
//    	}
    }
    
    /**
     * Runs ConnectionHandler once.
     */
    public void runOnce()
    {
    	try {
            // Process any pending changes
            synchronized (this.pendingChanges) {
                Iterator<ChangeRequest> changes = this.pendingChanges.iterator();
                while (changes.hasNext()) {
                    ChangeRequest change = changes.next();
                    SelectionKey key;
                    switch (change.type) {                            
                    case ChangeRequest.CHANGEOPS:                            
                        key = change.socket.keyFor(this.selector);                            
                        key.interestOps(change.ops);                            
                        break;
                    case ChangeRequest.REGISTER:
                        change.socket.register(this.selector, change.ops);
                        break;
                    }
                }
                this.pendingChanges.clear();
            }

            // Wait for an event one of the registered channels
            this.selector.select(ClientServerConstants.SELECT_TIMEOUT);           

            // Iterate over the set of keys for which events are available
            Iterator<SelectionKey> selectedKeys = this.selector.selectedKeys().iterator();
            while (selectedKeys.hasNext()) {
                SelectionKey key = selectedKeys.next();
                selectedKeys.remove();
                
                if (!key.isValid()) {
                    continue;
                }                   
                // Check what event is available and deal with it
                if (key.isConnectable()) {
                    this.finishConnection(key);
                } else if (key.isReadable()) {
                    this.read(key);
                } else if (key.isWritable()) {                    
                    this.write(key);
                }
            }
        } catch (Exception e) {
        	running = false;
        	if (observer != null) {
        		observer.disconnected(e);
        	}
            e.printStackTrace();
        }
    }

    /**
     * Reads data from socket. Attempts to read message size first and then
     * makes readBuffer ready to receive correct amount of data from the socket.    
     * 
     * @param key selection key associated to operation
     * @throws IOException if message is garbage
     * @throws JAXBException if data could not be translated to XML
     */
    
    private void read(SelectionKey key) throws IOException, JAXBException {
        SocketChannel socketChannel = (SocketChannel) key.channel();

		int msgSize = 0;
		int bytesRead;
		
		// Read message size from socket to sizeBuffer
		if(this.receivingState == State.WAITINGSIZE) { 
			
	
			// Attempt to read off the channel
			if (sizeBuffer.remaining() > 0) {
				try {
					bytesRead = socketChannel.read(sizeBuffer);
				} catch (IOException e) {
					// The remote forcibly closed the connection, cancel
					// the selection key and close the channel.
					key.cancel();
					socketChannel.close();
					return;
				}
				if (bytesRead == -1) {
					// Remote entity shut the socket down cleanly. Do the
					// same from our end and cancel the channel.
					key.channel().close();
					key.cancel();
					return;
				}
			}
			if (sizeBuffer.remaining() == 0) {
	
				sizeBuffer.rewind();
				msgSize = sizeBuffer.getInt();
	
				/*
				 * Perform sanity check for message size. Exception causes Server to
				 * disconnect this client.
				 */
				if (msgSize > ClientServerConstants.TOO_BIG_MESSAGE) {
					throw new IOException("This is bullshit");
				}
	
				// Clear out our read buffer so it's ready for new data and allocate
				// more space if necessary
				if (readBuffer == null
						|| readBuffer.capacity() < msgSize
						|| readBuffer.capacity() > (msgSize * ClientServerConstants.WASTING_MEM_RATIO)) {
					readBuffer = ByteBuffer.allocate(msgSize);
				} else {
					readBuffer.clear();
					readBuffer.limit(msgSize);
				}
	
				sizeBuffer.clear();
				receivingState = State.WAITINGDATA;
			}
		}

		// Read data from socket to readBuffer
		if(this.receivingState == State.WAITINGDATA) {
	
			// Attempt to read off the channel
			if (readBuffer.remaining() > 0) {
	
				try {
					bytesRead = socketChannel.read(readBuffer);
				} catch (IOException e) {
					// The remote forcibly closed the connection, cancel
					// the selection key and close the channel.
					key.cancel();
					socketChannel.close();
					return;
				}
				if (numRead == -1) {
					// Remote entity shut the socket down cleanly. Do the
					// same from our end and cancel the channel.
					key.channel().close();
					key.cancel();
					return;
				}
				numRead += bytesRead;
			}
	
			// When whole message is read from socket send it to response handler
			if (readBuffer.remaining() == 0) {
				this.handleResponse(key, readBuffer.array(), numRead);
				readBuffer.clear();
				// synchronized(this.receivingState) TODO delete if unnecessary
				this.receivingState = State.WAITINGSIZE;
				numRead = 0;
			}
		}
    }
    /**
	 * Translates data bytes to XML message and looks up message handler for
	 * this message.
	 * 
	 * @param key
	 * @param data data read from socket
	 * @param numRead length of data read from socket
	 * @throws JAXBException if data could not be translated to XML
	 */
    private void handleResponse(SelectionKey key, byte[] data, int numRead) throws JAXBException {
        // Make a correctly sized copy of the data before handing it to the
		// client
        byte[] rspData = new byte[numRead];
        System.arraycopy(data, 0, rspData, 0, numRead);       

        // Look up the handler for this message
        Message msg = xmltools.bytesToMessage(rspData);

        //System.out.println("Received {" + new String(rspData) + "}");
        
        MessageHandler handler = this.rspHandlers.get(""+msg.getSessionID());       

        // And pass the message to it
        if(handler != null){
        	if(msg.getResponse() != null)
        		handler.handleResponse(msg.getResponse());         
        	else if(msg.getRequest() != null)
        		handler.handleRequest(msg.getRequest());
        	else if(msg.getStateUpdate() != null)
        		handler.handleStateUpdate(msg.getStateUpdate());
            
        }else {
            //TODO no response handler found. Inform server?
        	//System.out.println("Response handler not found.");
            rspData = null;
        }
    }

    /**
     * Writes data bytes from queue to socket until queue is empty.
     * 
     * @param key associated selection key
     * @throws IOException if could not write to socket
     */
    
    private void write(SelectionKey key) throws IOException {
        SocketChannel socketChannel = (SocketChannel) key.channel();
        
        synchronized (this.pendingData) {
            List<ByteBuffer> queue = this.pendingData.get(socketChannel);

            // Write until there's not more data ...
            while (!queue.isEmpty()) {
                ByteBuffer buf = queue.get(0);
                socketChannel.write(buf);
                if (buf.remaining() > 0) {
                    // ... or the socket's buffer fills up
                    break;
                }
                queue.remove(0);
            }                       

            if (queue.isEmpty()) {
                
                // We wrote away all data, so we're no longer interested
                // in writing on this socket. Switch back to waiting for
                // data.                
                key.interestOps(SelectionKey.OP_READ);                
            }
        }        
    }

    /**
     * Finishes connection establishment.
     * 
     * @param key associated selection key
     * @throws IOException if connection operation failed
     */
    private void finishConnection(SelectionKey key) throws IOException {
        SocketChannel socketChannel = (SocketChannel) key.channel();
    
        // Finish the connection. If the connection operation failed
        // this will raise an IOException.
        socketChannel.finishConnect();
    
        // Register an interest in writing on this channel
        key.interestOps(SelectionKey.OP_READ);
        
        if (observer != null) {
        	observer.connected();
        }
    }

    /**
     * Starts connection establishment.
     * 
     * @return socketChannel of the connection
     * @throws IOException if connection failed
     */
    private SocketChannel initiateConnection() throws IOException {
        // Create a non-blocking socket channel
        SocketChannel socketChannel = SocketChannel.open();
        socketChannel.configureBlocking(false);
        socketChannel.socket().setSoTimeout(ClientServerConstants.CONNECT_TIMEOUT);
        socketChannel.socket().setKeepAlive(true);
    
        // Kick off connection establishment
        socketChannel.connect(new InetSocketAddress(this.hostAddress, this.port));
    
        // Queue a channel registration since the caller is not the 
        // selecting thread. As part of the registration we'll register
        // an interest in connection events. These are raised when a channel
        // is ready to complete connection establishment.
        synchronized(this.pendingChanges) {
            this.pendingChanges.add(new ChangeRequest(socketChannel, ChangeRequest.REGISTER, SelectionKey.OP_CONNECT));
        }
        
        return socketChannel;
    }

    /**
     * Creates new selector.
     * @return selector
     * @throws IOException if failed
     */
    
    private Selector initSelector() throws IOException {
        // Create a new selector
        return SelectorProvider.provider().openSelector();
    }
    
    /**
     * Closes connection.
     * @throws IOException if the connection was already closed
     */
    public void breakConnection() throws IOException{
        socket.close();
        socket.keyFor(this.selector).cancel();
    }
    

}
