package com.asdf.client;

import java.io.IOException;
import java.util.Random;

import javax.swing.SwingUtilities;
import javax.xml.bind.JAXBException;

import com.asdf.common.AccountType;
import com.asdf.common.GameController;
import com.asdf.common.PluginLoadException;
import com.asdf.gui.lobby.CreateGamePanel;
import com.asdf.gui.lobby.LobbyFrame;
import com.asdf.parser.AccountManagementRequest;
import com.asdf.parser.AccountManagementResponse;
import com.asdf.parser.AuthenticationRequest;
import com.asdf.parser.AuthenticationResponse;
import com.asdf.parser.AvailableGamesResponse;
import com.asdf.parser.CreateGameRequest;
import com.asdf.parser.GameDetailsResponse;
import com.asdf.parser.GameListResponse;
import com.asdf.parser.GameRequest;
import com.asdf.parser.GameResponse;
import com.asdf.parser.GenericResponse;
import com.asdf.parser.LobbyRequest;
import com.asdf.parser.LobbyResponse;
import com.asdf.parser.LoginResponse;
import com.asdf.parser.Message;
import com.asdf.parser.OptionalParameters;
import com.asdf.parser.Request;
import com.asdf.parser.Response;
import com.asdf.parser.StateUpdate;
import com.asdf.parser.AccountManagementRequest.Create;
import com.asdf.parser.AuthenticationRequest.Login;
import com.asdf.parser.LobbyRequest.GameDetails;
import com.asdf.parser.LobbyRequest.GameList;
import com.asdf.utils.Filter;
import com.asdf.utils.XMLTools;

/**
 * LobbyController is the centerpiece of the programs client side. It handles starting of 
 * game clients, gets available games on server, lists running games and gets details of
 * selected games. It also handles logging in into server. It runs the whole multigame browser.
 * It is also task of this class to send join and create game requests to the server.
 * 
 * Commands through LobbyView are processed in this LobbyController. Communication to server
 * is done through ConnectionHandler using XML -protocol defined by message.xsd.
 * 
 * @author asdf
 *
 */
public class LobbyController implements MessageHandler {

    private ConnectionHandler connection = null;
    private long sessionID = 0;
    private Filter gameListFilter = null;
    
    /**
     * View of the lobby.
     */
    static LobbyFrame frame;
    
    public static final String TEST_PATH = "testmaterial/";
    public static final String PLUGIN_PATH = "com.asdf.plugins";
    
    private LobbyModel model = null;
    
    // XMLTool used to marshall and unmarshall messages send and received from server
    private XMLTools xmltools;
    
    //private List<GameController> gameClients;
    
    private PluginLoader loader = new PluginLoader();

    /**
     * Sole constructor for LobbyController.
     * 
     * @param connection ConnectionHandler to handle communication between client's lobby and server.
     */
    public LobbyController(ConnectionHandler connection) {
    
    	xmltools = new XMLTools();

        //gameClients = new LinkedList();

    	//Setting a random session id.
        Random r = new Random(System.currentTimeMillis());
        sessionID = r.nextLong();
        if(sessionID < 0)
        	sessionID = -sessionID;
               
        model = new LobbyModel();        
        gameListFilter = new Filter();
        
        //Register as a response message handler to connection handler.
         this.connection = connection;
         if(connection != null)
             if(!connection.registerMessageHandler(this))
             	;
        
        
        requestAvailableGames();
        
        //Initialize GUI
        frame = new LobbyFrame(this);               
        frame.setVisible(true);    
    }  

    /**
     *  Brings lobby's view to front.
     */
    public static void lobbyToFront(){
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	frame.toFront();//No filter
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);		

    }

    
    /**
     *  Loads dynamically a game client plugin of a type chosen from LobbyView, initializes it with
     *  parameters received from server, sets its connection handler and starts it.
     * @param rsp Parameters used to initialize game client.
     */
    public void startGameClient(GameResponse rsp){
		GameController ctrl;
		try {
			ctrl = loader.loadGame(model.getListedGameControllerName(), 0, "noname");
			ctrl.setConnectionHandler(connection);			
			ctrl.init(rsp, model.getUser().getName(), model.getUser().getType());				
			ctrl.start();		
			//System.out.println(connection.registerMessageHandler(ctrl));
				
		} catch (PluginLoadException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Game Client " + model.getListedGameType() +" could not be started. Either it is not installed or it is not configured properly. Please check installation instructions for this game.", true); //No filter
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);				
		}

    }
   
    
    /**
     * Sets available games visible in GUI and updates LobbyModel if necessary.
     * 
     * @param response AvailableGamesResponse sent by server.
     */
    public void setAvailableGames(AvailableGamesResponse response){
    	model.setAvailableGames(response);
        frame.getGameTreePanel().initGUI(response);
    }
    
    /**
     * Sets details of selected game visible in GUI or shows panel for creating games.
     * 
     * @param response GameDetailsResponse sent by server.
     */
    public void setGameDetails(GameDetailsResponse response) {
    	final GameDetailsResponse tmpResponse = response;
    	
    	if(response == null)
    		setCreateGameDetails();
    	else{
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.showGameDetailsPanel(tmpResponse); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);		
    	}
    		
    }
    
    /**
     * Loads dynamically CreateGamePanel for the chosen game type (chosen in LobbyView)
     * and sets it visible.
     */
    public void setCreateGameDetails() {
    	try {    	
      		final CreateGamePanel panel = loader.loadCreateGamePanel(model.getListedCreatePanelName(), 0, "noname");
			panel.setController(this);
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.showCreateGamePanel(panel); 
			    	frame.getLoginPanel().setMessage("", false); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);			
      		
      		//frame.repaint();
		} catch (PluginLoadException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Create game panel for " + model.getListedGameType() + " not found. It is not possible to create games of this type.", false); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
		}
    }
    
    /**
     * Sets list of selected game visible in GUI and updates LobbyModel if necessary.
     * 
     * @param response GameListResponse sent by server.
     */
    public void setGameList(GameListResponse response, Filter filter){
    	final GameListResponse tmpResponse = response;
        model.setGameList(response);        
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	frame.getGameListPanel().setData(tmpResponse, gameListFilter); 
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
        
        this.setCreateGameDetails();
    }
    
    /**
     * Request a list of available games from server.
     */
    public void requestAvailableGames() {
        
    	//System.out.println("Requesting available games.");
    	
        Message msg = new Message();
        Request req = new Request();
        LobbyRequest lobby = new LobbyRequest();
        lobby.setAvailableGames("available games");
        req.setLobby(lobby);
        msg.setRequest(req);
        msg.setSessionID(sessionID);
                
        try {            
            connection.send(xmltools.marshallObject(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }


    /**
     * Requests a list of games of the given gameType and gameVariant.
     * 
     * @param gameType Game type available on server.
     * @param gameVariant Game variant of the given game type.
     */
    public void requestGameList(String gameType, String gameVariant) {
    	
    	//System.out.println("Requesting gamelist of " + gameType +"," + gameVariant);
        
        Message msg = new Message();
        Request req = new Request();
        LobbyRequest lobby = new LobbyRequest();
        GameList list = new GameList();
        //list.setGameType(model.getListedServerGameName()); //gameType      
        //System.err.println(model.getServerGameName(gameType));
        list.setGameType(model.getServerGameName(gameType));
        list.setGameVariant(gameVariant);
        lobby.setGameList(list);
        req.setLobby(lobby);
        msg.setRequest(req);
        msg.setSessionID(sessionID);
        
        try {            
            connection.send(xmltools.marshallObject(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        
        model.setListedGame(gameType, gameVariant);
        
    }

    /**
     * Requests details of selected game from server.
     * 
     * @param gameID GameID of selected game.
     */
    public void requestGameDetails(String gameID) {        
        
    	//System.out.println("Requesting game details "+gameID+".");
    	     
        Message msg = new Message();
        Request req = new Request();
        LobbyRequest lobby = new LobbyRequest();
        GameDetails details = new GameDetails();
        details.setGameID(new Long(gameID));
        lobby.setGameDetails(details);
        req.setLobby(lobby);
        msg.setRequest(req);
        msg.setSessionID(sessionID);
                
        try {        
            connection.send(xmltools.marshallObject(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        
        model.setSelectedGame(Long.parseLong(gameID));
        
    }        

    public XMLTools getXMLParserInstance(){
    	return xmltools;
    }

    /**
     * Requests logout from server.
     * 
     */
    public void requestLogOut() {
        
        Message msg = new Message();
        Request req = new Request();
        AuthenticationRequest auth = new AuthenticationRequest();
        auth.setLogout(new String("Logout"));
        req.setAuthentication(auth);
        msg.setRequest(req);
        msg.setSessionID(sessionID);
        
        try {
            connection.send(xmltools.marshallObject(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        
    }

    /**
     * Requests login from server.
     * 
     * @param account
     * @param password
     */
    public void requestLogIn(String account, String password) {
        //System.err.println("requestlogin");
        Message msg = getMessageTemplate();
        Request req = new Request();
        AuthenticationRequest auth = new AuthenticationRequest();
        Login login = new Login();
        login.setAccountName(account);
        login.setPassword(password);
        auth.setLogin(login);
        req.setAuthentication(auth);
        msg.setRequest(req);
        msg.setSessionID(sessionID);
        
        try {
            connection.send(xmltools.marshallObject(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        
    }
    
    /**
     * Sets user logged in in LobbyModel and LobbyFrame.
     * 
     * @param response Response from server to login request.
     */
    public void setLogin(LoginResponse response){
    	
    	final LoginResponse tmpResponse = response;
    	
        if(response.isSuccess()){
    		Runnable doWorkRunnable = new Runnable() {
    		    public void run() { 
    		    	frame.getLoginPanel().setMessage("Login OK", false);
    		    }
    		};
    		SwingUtilities.invokeLater(doWorkRunnable);
        	
            final String name = response.getName();
            final double balance = response.getBalance();
            
    		Runnable doWorkRunnable2 = new Runnable() {
    		    public void run() { 
    		    	//AccountType.REGULAR_USER
    		    	frame.getLoginPanel().setLoggedIn(name, balance, AccountType.values()[tmpResponse.getType()]);
    		    }
    		};
    		SwingUtilities.invokeLater(doWorkRunnable2);
            
            model.setLoggedIn(name, balance, AccountType.values()[tmpResponse.getType()]);   
          
        } else {
    		Runnable doWorkRunnable = new Runnable() {
    		    public void run() { 
    		    	frame.getLoginPanel().setMessage("Couldn't login: " + tmpResponse.getDescription(), true);
    		    }
    		};
    		SwingUtilities.invokeLater(doWorkRunnable);
        	
        }
        
    }
    
    /**
     * Sets user logged out in LobbyModel and LobbyFrame.
     * 
     * @param response Response from server to logout request.
     */
    public void setLogout(GenericResponse response) {        
        if(response.isSuccess()) {
    		Runnable doWorkRunnable = new Runnable() {
    		    public void run() { 
    		    	frame.getLoginPanel().setLoggedOut();
    		    }
    		};
    		SwingUtilities.invokeLater(doWorkRunnable);
            model.setLoggedOut();
        } else{
        	
        }
            //System.err.println("FAIL");
    }

    /**
     * Sends a join game request to server.
     * 
     * @return false if user was not logged in, true if request was sent 
     */
    public boolean requestJoinGame() {

        Message msg = new Message();
        Request req = new Request();
        GameRequest game = new GameRequest();
        OptionalParameters oPars = new OptionalParameters();
        game.setJoin(oPars);
        game.setGameID(model.getSelectedGame());
        msg.setRequest(req);
        msg.setSessionID(getID());
        req.setGame(game);

        
        try {
            connection.send(xmltools.marshallObject(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        return true;
 
    }

    /**
     * If b, then sets full games hidden, if not b set full games visible in LobbyView.
     * @param b a boolean
     */
    public void setFullGamesHidden(boolean b) {
        gameListFilter.setHideFull(b);
        setGameList(model.getGameList(), gameListFilter);        
        
    }

    /**
     * If b, then sets empty games hidden, if not b set empty games visible in LobbyView.
     * @param b a boolean
     */
    public void setEmptyGamesHidden(boolean b) {
    	gameListFilter.setHideEmpty(b);
    	setGameList(model.getGameList(), gameListFilter);
        
    }
    public void showGamesWithPlayers(int min, int max){    	
    	gameListFilter.setPlayers(min, max);
    	setGameList(model.getGameList(), gameListFilter);
    }
    
    /**
     * Called to get member frame disappear.
     */
    public void dispose() {
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	frame.dispose();
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
    	
    }

    /**
     * Create Message already containing correct sessionID
     * @return New Message object
     */
    public Message getMessageTemplate() {
    	Message msg = new Message();
    	msg.setSessionID(sessionID);
    	return msg;
    }
    
    /**
     * Handle LobbyResponse. Different message types are handled with different methods.
     * @param lobby LobbyResponse sent by the server.
     */
    protected void handleLobbyResponse(LobbyResponse lobby) {
		if (lobby.getAvailableGames() != null) {
			//System.out.print("available games.");
			//System.out.println();
			this.setAvailableGames(lobby.getAvailableGames());
		} else if (lobby.getGameDetails() != null) {
			//System.out.print("game details.");
			//System.out.println();
			this.setGameDetails(lobby.getGameDetails());
		} else if (lobby.getGameList() != null) {
			//System.out.print("game list.");
			//System.out.println();
			this.setGameList(lobby.getGameList(), gameListFilter);
		}
	}

    /**
     * Handle AuthenticationResponse. Possible subtypes are login and logout
     * types.
     * @param auth AuthenticationResponse sent by the server.
     */
	protected void handleAuthenticationResponse(AuthenticationResponse auth) {
		if (auth.getLogin() != null) {
			this.setLogin(auth.getLogin());
		} else if (auth.getLogout() != null) {
			this.setLogout(auth.getLogout());
		}
	}

	/**
	 * Handle AccountManagementResponse. Handled subtypes include
	 * account create, delete and change password functions.
	 * @param acc AccountManagementResponse sent by the server.
	 */
	protected void handleAccountManagementResponse(AccountManagementResponse acc) {
		if (acc.getCreate() != null) {
			final boolean success = acc.getCreate().isSuccess();
			final String description = acc.getCreate().getDescription();
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().createAccountSuccess(success, description); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);
			
		} else if (acc.getDelete() != null) {
			// TODO not implemented yet
		} else if (acc.getChangePassword() != null) {
			// TODO not implemented yet
		}
	}

	/**
	 * Handles GameResponse sent by server. Expects only GameResponses of type
	 * CreateGameResponse and JoinGameResponse.
	 * @param game GameResponse sent by server.
	 */
	protected void handleGameResponse(GameResponse game) {
		//System.out.println("create or join game message.");
		final GameResponse tmpGame = game;
		
		if (game.getCreate() != null) {
			if (game.getCreate().isSuccess()) {				
				requestGameList(model.getListedGameType(), model
						.getListedGameVariant());
			} else {
				Runnable doWorkRunnable = new Runnable() {
				    public void run() { 
						frame.getLoginPanel().setMessage(
								"Game creation failed: "
										+ tmpGame.getCreate().getDescription(), true);
				    }
				};
				SwingUtilities.invokeLater(doWorkRunnable);

			}
		} else if (game.getJoin() != null) {
			this.startGameClient(game);
		} else {
			//System.out.println("Unhandled game response");
		}
    }
    
    /**
	 * Handle response sent by the server.
	 */
	@Override
	public void handleResponse(Response rsp) {

		//System.out.print("Received message: ");
		if (rsp != null) {

			LobbyResponse lobby = rsp.getLobby();
			AuthenticationResponse auth = rsp.getAuthentication();
			AccountManagementResponse acc = rsp.getAccountManagement();
			GameResponse game = rsp.getGame();
			if (lobby != null) {
				handleLobbyResponse(lobby);
			} else if (auth != null) {
				handleAuthenticationResponse(auth);
			} else if (acc != null) {
				handleAccountManagementResponse(acc);
			} else if (game != null) {
				handleGameResponse(game);
			} else {
				//System.out.println("Response was not valid for lobby.");
			}
		}
	}

	/**
	 * Sends a create game request to server. Note that CreateGameRequests must be given
	 * by CreateGamePanel of the chosen game type. LobbyController has no idea what kind of
	 * parameters are used in creating any kind of games.
	 * 
	 * @param greq Request to create a game received from CreateGamePanel.
	 */
	public void requestCreateGame(CreateGameRequest greq) {
		
		greq.setGameType(model.getListedServerGameName());
		Request req = new Request();
		GameRequest gr = new GameRequest();
		gr.setCreate(greq);
		req.setGame(gr);
		
		//System.out.println("Requesting create game.");
		
		Message msg = new Message();
		msg.setSessionID(sessionID);
		msg.setRequest(req);
		try {
            connection.send(xmltools.messageToBytes(msg));
        } catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
		
	}

	@Override
	public long getID() {
		return this.sessionID;
	}

	/**
	 * Sends a create account request to server.  
	 * 
	 * @param account Name of the account.
	 * @param passwd Password for the account.
	 * @param accountType Type of the account. Currently only AccountType.REGULAR_USER.
	 */
	public void requestCreateAccount(String account, String passwd, String accountType) {
		Message msg = new Message();
		Request req = new Request();
		AccountManagementRequest acc = new AccountManagementRequest();
		Create create = new Create();
		create.setAccountName(account);
		create.setPassword(passwd);
		create.setAccountType(AccountType.REGULAR_USER.ordinal());
		
		acc.setCreate(create);
		req.setAccountManagement(acc);
		msg.setRequest(req);
		msg.setSessionID(this.sessionID);
		
		try {
			connection.send(xmltools.messageToBytes(msg));
		} catch (IOException e) {
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	frame.getLoginPanel().setMessage("Connection to server lost.",true); 
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);	
        } catch (JAXBException e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
		
	}

	/**
	 * Stub method, since LobbyController does not serve any requests.
	 */
	@Override
	public void handleRequest(Request req) {
		//System.err.println("Received a response, but I serve no master so nothing done.");
		
	}

	/**
	 * Stub method, since LobbyController does not handle any stateUpdates.
	 */
	@Override
	public void handleStateUpdate(StateUpdate stateUp) {
		
	}

	/**
	 * Sets sessionID for lobby.
	 */
	@Override
	public void setID(long id) {
		this.sessionID = id;
	}

	/**
	 * Sets a ConnectionHandler for lobby.
	 */
	@Override
	public void setConnectionHandler(ConnectionHandler connection) {
		this.connection = connection;		
	}
	/**
	 * Gets the ConnectionHandler used by lobby.
	 */
	@Override
	public ConnectionHandler getConnectionHandler() {
	
		return this.connection;
	}

	public void showGamesWithNameContaining(String s) {
		gameListFilter.setNameContains(s);
		setGameList(model.getGameList(), gameListFilter);				
	}
    
}
