package com.asdf.client;

import java.util.HashMap;
import java.util.Map;

import com.asdf.common.AccountType;
import com.asdf.common.GameController;
import com.asdf.parser.AvailableGamesResponse;
import com.asdf.parser.GameListResponse;

/**
 * Bookkeeping class used by LobbyController. Keeps up to date information about which
 * game is selected in lobby, which game type is selected and also the current gamelist.
 * Also includes getter methods which are useful for getting actual class names of selected
 * games create game panels and game clients.
 * 
 * @author asdf
 *
 */

public class LobbyModel {
    private Map<String, GameController> runningGames = null;
    
    private String listedGame = "Game";
    private String listedVariant = "Game";
    private long selectedGame = 0;
    
    private GameListResponse gameList = null;
    private AvailableGamesResponse availGames = null;
    
    private User user = null;
    
    
    /**
     * Sole constructor.
     */
    public LobbyModel(){
        runningGames = new HashMap<String, GameController>();
        user = new User();

    }
    

    /**
     * Sets the type and variant of listed game.
     * 
     * @param gameName
     * @param variant
     */
    public void setListedGame(String gameName, String variant){
        listedGame = gameName;
    	listedVariant = variant;
    	//this.listedGameController = gameController;
    	//this.listedCreatePanel = createGamePanel;
 
    }
    /**
     *  Gets the user object.
     * @return User
     */
    public User getUser(){
    	return user;
    }
    
    /**
     * Sets the id of selected game.
     * @param id GameID of selected game.
     */
    public void setSelectedGame(long id){
    	selectedGame = id;
    }
    /**
     * Gets the id of selected game.
     * @return GameID of selected game.
     */
    public long getSelectedGame(){
    	return selectedGame;
    }

    /**
     * Gets the game type which is listed.
     * 
     * @return Game type which is listed.
     */
    public String getListedGameType(){
    	return this.listedGame;
    }
    
    /**
     * Gets the game variant which is listed.
     * 
     * @return Game variant which is listed.
     */
    public String getListedGameVariant(){
    	return this.listedVariant;
    }
    
    /**
     * Gives the class name of the GameController for listed game type which starts
     * the game client at client side.
     * 
     * @return Class name of GameController or null, if it was not found.
     */
    public String getListedGameControllerName(){
    	for(AvailableGamesResponse.Game g : availGames.getGame()){
    		if(g.getType().equalsIgnoreCase(this.listedGame))
    			return g.getClientGame();
    	}
    	return null;
    }
    
    /**
     * Gives the class name of the GameController for this game type which starts
     * the game client at client side.
     * 
     * @param gameType Game type name.
     * @return Class name of GameController or null, if it was not found.
     */
    public String getGameControllerName(String gameType){
    	for(AvailableGamesResponse.Game g : availGames.getGame()){
    		if(g.getType().equalsIgnoreCase(gameType))
    			return g.getClientGame();
    	}
    	return null;
    }
    /**
     * Gives the class name of the CreateGamePanel for listed game type which is shown
     * in lobby.
	 *
     * @return Class name of CreateGamePanel or null, if it was not found.
     */
    public String getListedCreatePanelName(){
    	for(AvailableGamesResponse.Game g : availGames.getGame()){
    		if(g.getType().equalsIgnoreCase(this.listedGame))
    			return g.getCreateGamePanel();
    	}
    	return null;
    }
    
    
    /**
     * Gives the class name of the CreateGamePanel for this game type which is shown
     * in lobby.
     * 
     * @param gameType Game type name.
     * @return Class name of CreateGamePanel or null, if it was not found.
     */
    public String getCreatePanelName(String gameType){
    	for(AvailableGamesResponse.Game g : availGames.getGame()){
    		if(g.getType().equalsIgnoreCase(gameType))
    			return g.getCreateGamePanel();
    	}
    	return null;
    }
    /**
     * Gives the class name of the Game running on server side for listed game type which is shown
     * in lobby.
     * 
     * @return Class name of Game running on server side or null, if it was not found.
     */
    public String getListedServerGameName(){
    	for(AvailableGamesResponse.Game g : availGames.getGame()){
    		//System.err.println("getListedServerGameName " + g.getType());
    		if(g.getType().equalsIgnoreCase(this.listedGame))
    			return g.getServerGame();
    	}
    	return null;
    }
    
    /**
     * Gives the class name of the Game running on server side for listed game type which is shown
     * in lobby.
     * 
     * @param gameType Game type name.
     * @return Class name of Game running on server side or null, if it was not found.
     */    
    public String getServerGameName(String gameType){
    	for(AvailableGamesResponse.Game g : availGames.getGame()){
    		//System.err.println("getListedServerGameName " + g.getType());
    		if(g.getType().equalsIgnoreCase(gameType))
    			return g.getServerGame();
    	}
    	return null;
    }

    /**
     * Adds a game to the list of started game clients.
     * 
     * @param game Game which was started in lobby.
     */
    public void addGame(GameController game){
        runningGames.put(""+"GameControllerin id", game);
    }
    
    /**
     * Sets user logged in.
     * @param name Account name.
     * @param balance Account balance.
     * @param type Account type. See AccounType enumeration for details.
     */
    public void setLoggedIn(String name, double balance, AccountType type){  
        user.setLogged(name, balance, type);
    }
    /**
     * Sets user logged out.
     */
    public void setLoggedOut(){
        user.setLoggedOut();
    }

    /**
     * Sets games listed in lobby.
     * @param response GameListResponse sent by server.
     */
    public void setGameList(GameListResponse response) {    	
        gameList = response;        
    }
    
    /**
     * Sets list of available games given by server.
     * @param response AvailableGamesResponse sent by server.
     */
    public void setAvailableGames(AvailableGamesResponse response) {
    	availGames = response;
    }
    /**
     * Gets listed games.
     * 
     * @return GameListResponse containing original gamelist sent by the server.
     */
    public GameListResponse getGameList() {
    	return gameList;        
    }
    /**
     * Gets available games.
     * 
     * @return AvailableGamesResponse containing original available gamelist sent by the server.
     */
    public AvailableGamesResponse getAvailableGames() {
        return availGames;        
    }
}
