package com.asdf.client;

import com.asdf.parser.Request;
import com.asdf.parser.Response;
import com.asdf.parser.StateUpdate;

/**
 * This interface is used by client program to define the communication interface
 * that ConnectionHandler uses to pass messages received from server to Lobby and
 * to GameControllers started from the lobby. 
 *  
 * Messages are passed through this interface according to implementing objects
 * sessionID.
 * 
 * If you wish to know more about the messaging protocol used, please see protocol.xsd
 * schema file in protocols directory for further details and additional comments for different
 * components in the message.
 * 
 * @author asdf
 *
 */
public interface MessageHandler {
	
	/**
	 * Handles a Response sent by the server. You may wish to leave this method as a
	 * stub in your implementation if you are not interested in receiving this kind
	 * of messages.
	 * 
	 * @param rsp Response sent by server. Response is an object generated from
	 * XML message using JAXB.
	 */
	public void handleResponse(Response rsp);
	
	/**
	 * Handles a Request sent by the server. You may wish to leave this method as a
	 * stub in your implementation if you are not interested in receiving this kind
	 * of messages.
	 * 
	 * @param req Request sent by server. Request is an object generated from
	 * XML message using JAXB.
	 */
	public void handleRequest(Request req);
	
	/**
	 * Handles StateUpdates sent by server. You may wish to leave this method as a
	 * stub in your implementation if you are not interested in receiving this kind
	 * of messages.
	 * 
	 * @param stateUp StateUpdate sent by server. StateUpdate is an object generated from
	 * XML message using JAXB.
	 */
	public void handleStateUpdate(StateUpdate stateUp);
	
	/**
	 * Sets the ConnectionHandler which passes messages to the implementing class
	 * trough this interfaces methods. Also ConnectionHandler passed down by this method can
	 * be used to send messages by calling it through getConnectionHandler() method.
	 * 
	 * @param connection ConnectionHandler to be used.
	 */
	public void setConnectionHandler(ConnectionHandler connection);
	
	/**
	 * Returns the ConnectionHandler used by implementing class.
	 * When you want to send messages to server, use this method to get
	 * the associated ConnectionHandler and call its send -methods.
	 * 
	 * @return ConnectionHandler used by the implementing class.
	 */
	public ConnectionHandler getConnectionHandler();
	
	/**
	 * Returns the sessionID of this MessageHandler.
	 * 
	 * @return SessionID of this MessageHandler.
	 */
	public long getID();
	/**
	 * Sets a sessionID for this MessageHandler.
	 * 
	 * @param id Semi-Unique sessionID.
	 */
	public void setID(long id);
}
