package com.asdf.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import com.asdf.common.Logger;

/**
 * Handles playing and recording of sounds.
 */
public class SoundManager {
	
	/** Thread used for recording voice */
	private Thread recordingThread;
	
	/** Stream to which the voice data is pushed */
	private ByteArrayOutputStream out;
	
	/** Status variable for recoding */
	private boolean recording = false;
	
	/** Used audioformat */
	private AudioFormat format;
	
	/** Shared instance of SoundPlayer */
	private static SoundManager instance;
	
	/** Paths to sound files */
	private static final String[] paths = {
		"sound/times_up.wav",
		"sound/cards.wav",
		"sound/hmm.wav",
		"sound/money.wav",
		"sound/your_turn.wav"
	};
	
	/** Sound files loaded to clips */
	private Clip[] clips;
	
	/**
	 * Constructs SoundPlayer by trying to load sounds from
	 * files first and then from resources.
	 */
	protected SoundManager() {
		out = new ByteArrayOutputStream();
		format = new AudioFormat(8000, 8, 1, true, true);
		
		int index = 0;
		clips = new Clip[paths.length];
		for(String path : paths) {
			try {
				AudioInputStream ain;
				URL url = getClass().getResource("/" + path);
				if (url != null) {
					ain = AudioSystem.getAudioInputStream(url);
				} else {
					File f = new File(path);
					ain = AudioSystem.getAudioInputStream(f);
				}
				DataLine.Info info = new DataLine.Info(Clip.class, ain.getFormat());
				clips[index] = (Clip)AudioSystem.getLine(info);
				clips[index].open(ain);
				index++;
	            ain.close();
			} catch (LineUnavailableException e) {
				Logger.get().throwing("SoundManager", "Constructor", e);
			} catch (UnsupportedAudioFileException e) {
				throw new RuntimeException(e);
			} catch (IOException e) {
				throw new RuntimeException(e);
			}
		}
	}
	
	/**
	 * Play file specified by index
	 * @param index
	 */
	protected void playFile(int index) {
		try {
			assert(index >= 0 && index < clips.length);
			clips[index].setMicrosecondPosition(0);
			clips[index].start();
		} catch (Exception e) {
			Logger.get().fine("Sounds disabled: " + e.getMessage());
		}
	}
	
	protected void startRecording_() {
		assert (recording == false);
		try {
			DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);
			final TargetDataLine line = (TargetDataLine) AudioSystem.getLine(info);
			line.open(format);
			line.start();
			Runnable runner = new Runnable() {
				int bufferSize = (int) format.getSampleRate()
						* format.getFrameSize();
				byte buffer[] = new byte[bufferSize];

				public void run() {
					out.reset();
					recording = true;
					try {
						while (recording) {
							int count = line.read(buffer, 0, buffer.length);
							if (count > 0) {
								out.write(buffer, 0, count);
							}
						}
						out.close();
					} catch (IOException e) {
						System.err.println("I/O problems: " + e);
						System.exit(-1);
					}
				}
			};
			recordingThread = new Thread(runner);
			recordingThread.start();
		} catch (LineUnavailableException e) {
			System.err.println("Line unavailable: " + e);
			System.exit(-2);
		}
	}
	
	protected byte[] stopRecording_() {
		if (recording) {
			recording = false;
			byte[] data = null;
			try {
				recordingThread.join();
				data = out.toByteArray();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return data;
		} else {
			return null;
		}
	}
	
	protected void playCustomData_(byte[] data) {
		try {
			byte audio[] = out.toByteArray();
			InputStream input = new ByteArrayInputStream(audio);
			final AudioInputStream ais = new AudioInputStream(input, format,
					audio.length / format.getFrameSize());
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
			final SourceDataLine line = (SourceDataLine) AudioSystem.getLine(info);
			line.open(format);
			line.start();

			Runnable runner = new Runnable() {
				int bufferSize = (int) format.getSampleRate() * format.getFrameSize();
				byte buffer[] = new byte[bufferSize];

				public void run() {
					try {
						int count;
						while ((count = ais.read(buffer, 0, buffer.length)) != -1) {
							if (count > 0) {
								line.write(buffer, 0, count);
							}
						}
						line.drain();
						line.close();
					} catch (IOException e) {
						System.err.println("I/O problems: " + e);
						System.exit(-3);
					}
				}
			};
			Thread playThread = new Thread(runner);
			playThread.start();
		} catch (LineUnavailableException e) {
			System.err.println("Line unavailable: " + e);
			System.exit(-4);
		}
	}
	
	public static void startRecording() {
		//getInstance().startRecording_();
	}
	
	public static byte[] stopRecording() {
		return null/*getInstance().stopRecording_()*/;
	}
	
	public static void playCustomData(byte[] data) {
		//getInstance().playCustomData_(data);
	}
	
	/**
	 * Get the shared instance of SoundPlayer
	 * 
	 * @return
	 */
	protected synchronized static SoundManager getInstance() {
		if (instance == null) {
			instance = new SoundManager();
		}
		return instance;
	}
	
	/**
	 * Play annoying time's up sound.
	 */
	public synchronized static void playTimesUp() {
		getInstance().playFile(0);
	}
	
	/**
	 * Play sound coming from rubbing cards together.
	 */
	public synchronized static void playCardsRubbing() {
		getInstance().playFile(1);
	}
	
	/**
	 * Play humming sound
	 */
	public synchronized static void playHumming() {
		getInstance().playFile(2);		
	}
	
	/**
	 * Play sound coming from placing money on fabric
	 * surface.
	 */
	public synchronized static void playBlingBling() {
		getInstance().playFile(3);
	}
	
	/**
	 * Say "Sir, It's your turn"
	 */
	public synchronized static void playYourTurn() {
		getInstance().playFile(4);
	}
	public static void testi() throws InterruptedException {
//		getInstance().captureAudio();
//		Thread.sleep(2000);
//		getInstance().playAudio();
//		Thread.sleep(20000);
	}
	
	
	public static void main(String[] args) throws InterruptedException {
		System.out.println("Start");
		startRecording();
		Thread.sleep(2000);
		byte[] data = stopRecording();
		playCustomData(data);
		Thread.sleep(4000);
		System.out.println("End");
	}
}
