package com.asdf.common;

/**
 * Class representing the action that a Poker player might perform
 * @author rimpila
 *
 */
public class Action {

	public static final int CALL = 0;
	public static final int CHECK = 1;
	public static final int RAISE = 2;
	public static final int FOLD = 3;
	public static final int ALLIN_PASS = 4;
	public static final int BET = 5;
	public static final int BIG_BLIND = 6;
	public static final int MUCK = 7;
	public static final int POST_ANTE = 8;
	public static final int POST_BLIND = 9;
	public static final int POST_DEAD_BLIND = 10;
	public static final int SIT_OUT = 11;
	public static final int SMALL_BLIND = 12;
	public static final int INVALID = -1;
	
	public static final int JOIN_GAME = 13;
	public static final int RESERVE_SEAT = 14;
	public static final int LEAVE_GAME = 15;
	
	public static final int REVEAL = 16;
	public static final int DISCARD = 17;
	public static final int BACK = 18;
	public static final int SKIP_TURN = 19;

	
	public int index;
	public double amount = 0;
	public int[] discarded = null;
	public String name = null;
	public int seat = -1;

	/**
	 * Empty constructor
	 */
	public Action() {}
	
	/**
	 * Constructor for an action with the given index and amount
	 * @param index
	 * @param amount
	 */
	public Action(int index, double amount) {
		this.index = index;
		this.amount = amount;
	}
	
	/**
	 * Constructor for the given action
	 * @param index
	 */
	public Action(int index) {
		this.index = index;
		this.amount = 0;
	}
	
	/**
	 * Getter for the index
	 * @return integer corresponding to the action
	 */
	public int getActionIndex() {
		return this.index;
	}

	/**
	 * Getter for the amount
	 * @return amount
	 */
	public double getAmount() {
		return this.amount;
	}
	
	/**
	 * 
	 * @return true if the action is call, false otherwise
	 */
	public boolean isCall() {
		return this.index == CALL;
	}
	
	/**
	 * 
	 * @return true if the action is check, false otherwise
	 */
	public boolean isCheck(){
		return this.index == CHECK;
	}
	
	/**
	 * 
	 * @return true if the action is raise, false otherwise
	 */
	public boolean isRaise() {
		return this.index == RAISE;
	}
	
	/**
	 * 
	 * @return true if the action is fold, false otherwise
	 */
	public boolean isFold() {
		return this.index == FOLD;
	}
		
	/**
	 * 
	 * @return true if the action is all in pass, false otherwise
	 */
	public boolean isAllInPass() {
		return this.index == ALLIN_PASS;
	}
		
	/**
	 * 
	 * @return true if the action is bet, false otherwise
	 */
	public boolean isBet() {
		return this.index == BET;
	}
	
	/**
	 * 
	 * @return true if the action is posting big blind, false otherwise
	 */
	public boolean isBigBlind() {
		return this.index == BIG_BLIND;
		
	}
		
	/**
	 * 
	 * @return true if the action is muck, false otherwise
	 */
	public boolean isMuck() {
		return this.index == MUCK;
	}
		
	/**
	 * 
	 * @return true if the action is posting ante, false otherwise
	 */
	public boolean isPostAnte() {
		return this.index == POST_ANTE;
	}
		
	/**
	 * 
	 * @return true if the action is posting a blind, false otherwise
	 */
	public boolean isPostBlind() {
		return this.index == POST_BLIND;
	}
		
	/**
	 * 
	 * @return true if the action is posting a dead blind, false otherwise
	 */
	public boolean isPostDeadBlind() {
		return this.index == POST_DEAD_BLIND;
	}
	
	/**
	 * 
	 * @return true if the action is sitting out, false otherwise
	 */
	public boolean isSitout() {
		return this.index == SIT_OUT;
	}
	
	/**
	 * 
	 * @return true if the action is posting small blind, false otherwise
	 */
	public boolean isSmallBlind() {
		return this.index == SMALL_BLIND;
	}

	
	/**
	 * 
	 * @return true if the action is join game, false otherwise
	 */
	public boolean isJoin() {
		return this.index == JOIN_GAME;
	}
	
	/**
	 * The name should be send with join game actions
	 * @return the name for this action
	 */
	public String name() {
		return this.name;
	}
	
	/**
	 * Seat of the acting player, or the seat the player wishes to join
	 * @return seat
	 */
	public int seat() {
		return this.seat;
	}
	
	/**
	 * 
	 * @return a check action
	 */
	public static Action checkAction() {
		return new Action(CHECK);
	}
	
	/**
	 * 
	 * @return a fold action
	 */
	public static Action foldAction() {
		return new Action(FOLD);
	}
	
	/**
	 * 
	 * @return an all in pass action
	 */
	public static Action allInPassAction() {
		return new Action(ALLIN_PASS);
	}
	
	/**
	 * 
	 * @return a call action
	 */
	public static Action callAction() {
		return new Action(CALL);
	}
	
	/**
	 * 
	 * @param amount2 amount to bet
	 * @return a bet action
	 */
	public static Action betAction(double amount2) {
		return new Action(BET, amount2);
	}
	
	/**
	 * 
	 * @param amount2 amount to ante
	 * @return a post ante action
	 */
	public static Action postAnte(double amount2) {
		return new Action(POST_ANTE, amount2);
	}
	
	/**
	 * 
	 * @return a big blind action
	 */
	public static Action bigBlindAction() {
		return new Action(BIG_BLIND);
	}
	
	/**
	 * 
	 * @return a muck action
	 */
	public static Action muckAction() {
		return new Action(MUCK);
	}
	
	/**
	 * 
	 * @return a post dead blind action
	 */
	public static Action postDeadBlindAction() {
		return new Action(POST_DEAD_BLIND);
	}
	
	/**
	 * 
	 * @return a post blind action
	 */
	public static Action postBlindAction() {
		return new Action(POST_BLIND);
	}
	
	/**
	 * 
	 * @return a sit out action
	 */
	public static Action sitOutAction() {
		return new Action(SIT_OUT);
	}
	
	/**
	 * 
	 * @return a sit out action
	 */
	public static Action sitout() {
		return new Action(SIT_OUT);
	}
	
	/**
	 * 
	 * @param amount2 amount to raise
	 * @return a raise action with the given amount
	 */
	public static Action raiseAction(double amount2) {
		return new Action(RAISE, amount2);
	}
	
	/**
	 * 
	 * @return a small blind action
	 */
	public static Action smallBlindAction() {
		return new Action(SMALL_BLIND);
	}
	
	/**
	 * 
	 * @param name with which the player wishes to play
	 * @param seat at which the player wishes to sit
	 * @param cash the amount to bring to the table
	 * @return a join action
	 */
	public static Action joinAction(String name, int seat, double cash) {
		Action action = new Action(JOIN_GAME, cash);
		action.name = name;
		action.seat = seat;
		return action;
	}
	
	/**
	 * 
	 * @param seat to reserve
	 * @return a reserve action
	 */
	public static Action reserveAction(int seat) {
		Action action = new Action(RESERVE_SEAT);
		action.seat = seat;
		return action;
	}
	
	/**
	 * 
	 * @param seat from which to leave
	 * @param amount value should be < -1 if the player wishes to also be removed from the watcher list (i.e. closing the GUI)
	 * @return a leave game action
	 */
	public static Action leaveGameAction(int seat, double amount) {
		Action action = new Action(LEAVE_GAME);
		//action.name = name;
		action.amount = amount;
		action.seat = seat;
		return action;
	}
	
	/**
	 * 
	 * @param discarded positions of discarded cards
	 * @return discard action
	 */
	public static Action discardAction(int[] discarded) {
		Action action = new Action(DISCARD);
		//action.discards = amount;
		action.discarded = discarded;
		return action;

	}
	
	/**
	 * 
	 * @return true if the action is reserve, false otherwise
	 */
	public boolean isReserve() {
		return this.index == RESERVE_SEAT;
	}
	

	/**
	 * 
	 * @return true if the action is leave game, false otherwise
	 */
	public boolean isLeaveGame() {
		return this.index == LEAVE_GAME;
	}
	
	/**
	 * 
	 * @return reveal action
	 */
	public static Action revealAction() {
		return new Action(REVEAL);
	}
	
	/**
	 * 
	 * @return true if the action is reveal, false otherwise
	 */
	public boolean isReveal() {
		return this.index == REVEAL;
	}
	
	/**
	 * 
	 * @return true if the action is discard, false otherwise
	 */
	public boolean isDiscard() {
		return this.index == DISCARD;
	}

	/**
	 * 
	 * @return number of discarded cards
	 */
	public int discards() {
		if (discarded != null)
			return discarded.length;
		return 0;
	}

	/**
	 * 
	 * @param i
	 * @return position of discarded card number i
	 */
	public int discards(int i) {
		return discarded[i];
	}
	
	/**
	 * 
	 * @return a back action from sit out
	 */
	public static Action backAction() {
		return new Action(BACK);
	}
	
	/**
	 * 
	 * @return true if the action is back, false otherwise
	 */
	public boolean isBack() {
		return this.index == BACK;
	}
	
	/**
	 * 
	 * @return a skip turn action
	 */
	public static Action skipTurnAction() {
		return new Action(SKIP_TURN);
	}
	
	/**
	 * 
	 * @return true if action is skip turn
	 */
	public boolean isSkipTurn() {
		return this.index == SKIP_TURN;
	}
	
	

}
