package com.asdf.common;

import com.asdf.parser.*;
import com.asdf.server.Account;
import com.asdf.server.ClientConnection;
import com.asdf.server.Server;

import java.io.IOException;
import java.sql.Connection;

/**
 * Game interface. Game plugin developers implement abstract
 * methods such as handleRequest and handleResponse to receive
 * messages from clients. Plugins can send data using provided
 * send methods.
 */
public abstract class Game {
	
	/** Tells us how many times this game has been checked for removal. */
	public int age = 0;
	
	/** Server instance used to access various services from server */
	private Server server;
	
	/** Game identification number */
	protected long gameID;
	
	/** Name of the game */
	protected String name;
	
	/**
	 * Handle incoming GameRequest. Responses may be sent with the
	 * send method. Game requests contain information the Game
	 * developer may define him self, and some basic information that
	 * every game is assumed to have, such as name, players, etc..
	 * See protocol description for more information.
	 * @param sessionID Identifies session on which the message was received
	 * @param request GameRequest
	 */
	public synchronized final void handleRequestSync(long sessionID, GameRequest request) {
			handleRequest(sessionID, request);
	}
	
	/**
	 * Handle incoming GameRequest. Responses may be sent with the
	 * send method. A GameRequest is a message from a client, who requests
	 * the game to react to his actions.
	 * @param sessionID Identifies session on which the message was received
	 * @param request GameRequest
	 */
	protected abstract void handleRequest(long sessionID, GameRequest request);
	
	/**
	 * Requests the game to return a description of itself.
	 */
	public synchronized final GameListResponse.Games.Game descriptionSync() {
			return description();
	}

	/**
	 * Requests the game to return a description of itself.
	 */
	protected abstract GameListResponse.Games.Game description();
	
	/**
	 * Requests the game to return a description of itself.
	 */
	public synchronized final void detailsSync(GameDetailsResponse a) {
			details(a);
	}
	
	/**
	 * Requests the game to return a list of columns this type of game
	 *  wants to list in the game browser.
	 *  @return List of columns
	 */
	protected abstract String[] columnNames();
	
	/**
	 * Requests the game to return a list of columns this type of game
	 *  wants to list in the game browser.
	 *  @return List of columns
	 */
	public synchronized final String[] columnNamesSync() {
		return columnNames();
	}
	
	/**
	 * Requests the game to return a description of itself.
	 */
	protected abstract void details(GameDetailsResponse a);
	
	/**
	 * Handle incoming GameResponse.
	 * @param sessionID Identifies session on which the message was received
	 * @param response GameResponse
	 */
	public synchronized final void handleResponseSync(long sessionID, GameResponse response) {
			handleResponse(sessionID, response);
	}
	
	/**
	 * Handle incoming GameResponse.
	 * @param sessionID Identifies session on which the message was received
	 * @param response GameResponse
	 */
	protected abstract void handleResponse(long sessionID, GameResponse response);
	
	/**
	 * Only constructor. Initializes member variables to given values.
	 * @param server 
	 * @param gameID
	 * @param name
	 */
	public synchronized final void initialize(Server server, long gameID, String name) {
		
		this.server = server;
		this.gameID = gameID;
		this.name = name;
		
		final Server temp_serv = server;
		
		GameTimerTask task = new GameTimerTask(this) {
		
			@Override
			public void onTick() {
				temp_serv.getGameManager().checkDelete(getId(), this);
			}
		};
		
		this.setTimer(task , 60000, 10000);
		
	}
	
	/**
	 * Getter for the game ID
	 * @return ID
	 */
	public synchronized final long getId() {
		return gameID;
	}
	
	/**
	 * Getter for the game's name.
	 * @return Name of the game
	 */
	public synchronized final String getName() {
		return name;
	}
	
	/**
	 * Get type of the user.
	 * @param sessionID
	 * @return the AccountType
	 */
	public synchronized AccountType getUserType(long sessionID) {
		Account acc = this.server.getSessionManager().getBySessionID(sessionID).getAccount();
		if (acc != null)
			return acc.getType();
		else
			return AccountType.GUEST;
	}
	
	/**
	 * Get account name of the user.
	 * @param sessionID
	 * @return Name of the account or "Guest" if sessionID is not logged in with any account.
	 */
	public synchronized String getAccountName(long sessionID) {
		Account acc = this.server.getSessionManager().getBySessionID(sessionID).getAccount();
		if (acc != null)
			return acc.getName();
		else
			return "Guest";
	}
	
	/**
     * Schedules the specified task for execution after the specified delay.
     *
     * @param task  task to be scheduled.
     * @param delay delay in milliseconds before task is to be executed.
     * @throws IllegalArgumentException if <tt>delay</tt> is negative, or
     *         <tt>delay + System.currentTimeMillis()</tt> is negative.
     * @throws IllegalStateException if task was already scheduled or
     *         cancelled, or timer was cancelled.
     */
	protected final void setTimer(GameTimerTask callback, long delay) {
		server.getGameManager().getTimer().schedule(callback, delay);
	}
	
	/**
     * Schedules the specified task for repeated <i>fixed-delay execution</i>,
     * beginning after the specified delay.  Subsequent executions take place
     * at approximately regular intervals separated by the specified period.
     *
     * <p>In fixed-delay execution, each execution is scheduled relative to
     * the actual execution time of the previous execution.  If an execution
     * is delayed for any reason (such as garbage collection or other
     * background activity), subsequent executions will be delayed as well.
     * In the long run, the frequency of execution will generally be slightly
     * lower than the reciprocal of the specified period (assuming the system
     * clock underlying <tt>Object.wait(long)</tt> is accurate).
     *
     * <p>Fixed-delay execution is appropriate for recurring activities
     * that require "smoothness."  In other words, it is appropriate for
     * activities where it is more important to keep the frequency accurate
     * in the short run than in the long run.  This includes most animation
     * tasks, such as blinking a cursor at regular intervals.  It also includes
     * tasks wherein regular activity is performed in response to human
     * input, such as automatically repeating a character as long as a key
     * is held down.
     *
     * @param task   task to be scheduled.
     * @param delay  delay in milliseconds before task is to be executed.
     * @param period time in milliseconds between successive task executions.
     * @throws IllegalArgumentException if <tt>delay</tt> is negative, or
     *         <tt>delay + System.currentTimeMillis()</tt> is negative.
     * @throws IllegalStateException if task was already scheduled or
     *         cancelled, timer was cancelled, or timer thread terminated.
     */
	protected final void setTimer(GameTimerTask callback, long delay, long period) {
		server.getGameManager().getTimer().schedule(callback, delay, period);
	}
	
	/**
	 * Send GameRequest to recipient specified by sessionID.
	 * @param sessionID
	 * @param request
	 * @throws IOException
	 */
	protected synchronized final void send(long sessionID, GameRequest request) {
		Message m = new Message();
		m.setSessionID(sessionID);
		Request r = new Request();
		request.setGameID(gameID);
		r.setGame(request);
		m.setRequest(r);
		send(sessionID, m);
	}
	
	/**
	 * Send GameResponse to recipient specified by sessionID.
	 * @param sessionID
	 * @param response
	 * @throws IOException
	 */
	protected synchronized final void send(long sessionID, GameResponse response) {
		Message m = new Message();
		m.setSessionID(sessionID);
		Response r = new Response();
		response.setGameID(gameID);
		r.setGame(response);
		m.setResponse(r);
		send(sessionID, m);
	}
	
	/**
	 * Send GameResponse to recipient specified by sessionID.
	 * @param sessionID
	 * @param response
	 * @throws IOException
	 */
	protected synchronized final void send(long sessionID, GameStateUpdate gsup) {
		Message m = new Message();
		m.setSessionID(sessionID);
		StateUpdate r = new StateUpdate();
		gsup.setGameID(gameID);
		r.setGame(gsup);
		m.setStateUpdate(r);
		send(sessionID, m);
	}
	
	/**
	 * Returns DB connection that can be used for logging
	 * in Game instances.
	 * @return DB connection
	 */
	protected final Connection getDbConnection() {
		return this.server.getGameManager().getDbGameConnection();
	}
	
	/**
	 * Send Message to recipient specified by sessionID.
	 * @param sessionID
	 * @param message Message to send
	 * @throws IOException
	 */
	protected synchronized final void send(long sessionID, Message message) {
		ClientConnection client = server.getSessionManager().getBySessionID(sessionID);
		if (client != null)
			client.send(message);
	}
	
	
	/**
	 * Offers the game a chance to notify all players and observers about the imminent destruction
	 * of this game. Synchronized version.
	 */
	public synchronized final void dieSync() {
		die();
	}

	/**
	 * Offers the game a chance to notify all players and observers about the imminent destruction
	 * of this game.
	 */
	public abstract void die();
	
	
	/**
	 * toString method to ease debugging.
	 * @return Nice human readable description
	 */
	public synchronized String toString() {
		return "<Game id=\"" + gameID + "\" name=\"" + name + "\">";
	}

	/**
	 * Change user credit balance. Positive amount increases the
	 * balance by given amount and negative does the other way around.
	 * @param amount Difference in balance
	 */
	protected final void setBalanceChange(String gameName, long sessionID, double amount) {
		//ClientConnection client = this.server.getSessionManager().getBySessionID(sessionID);
		//this.server.getAccountManager().setBalanceChange(gameName, amount, client.getAccount());
	}
}
