package com.asdf.common;

import java.util.Random;

import com.asdf.client.ConnectionHandler;
import com.asdf.client.MessageHandler;
import com.asdf.parser.GameResponse;


/**
 * GameController is an abstract class which implements some basic functionality necessary
 * for client side game controllers. It defines few abstract methods which are used as an
 * interface to communicate with our client program and this client side game controller.
 * 
 * If you wish to implement a new client side game controller to be used with game implemented
 * at server side, you must use this class (or some other sub-class of this class) as super class
 * of your own implementation.
 * 
 * @author asdf
 *
 */
public abstract class GameController implements MessageHandler {
	
	/**
	 * Randomly generated sessionID, which identifies this session in ConnectionHandler and
	 * at server.
	 */
	private long sessionID = 0;
	/**
	 * GameID is the id of the game running at server side.
	 */
	private long gameID = 0;
	
	private ConnectionHandler connection = null;
	
	/**
	 * Since GameControllers are loaded dynamically in lobby it is not possible to use
	 * a constructor with parameters, which GameController implementations might need.
	 * This is used after the default constructor is called.
	 * 
	 * @param joinResponse Parameters received from server required to initialize this 
	 * @param accountName Account name of the user who is associated with this GameController.
	 * @param accountType Type of the user account. Some users might have more privileges than
	 *  others.
	 * GameController
	 */
	public abstract void init(GameResponse joinResponse, String accountName, AccountType accountType);
	/**
	 * Start is called from lobby when this GameController is ready to be started.
	 * That means you should implement your game starting procedures using this method.
	 */
	public abstract void start();
		
	/**
	 * Default constructor for GameController which sets a random id for itself.
	 */
	public GameController(){
        Random r = new Random(System.currentTimeMillis());
        sessionID = r.nextLong();
        if(sessionID < 0)
        	sessionID = -1 * sessionID;
	}

	/**
	 * Gets this games sessionID.
	 */
	public final long getID(){
		return sessionID;
	}
	/**
	 * Sets this games sessionID. 
	 */
	public final void setID(long id){
		this.sessionID = id;
	}
	/**
	 * Sets this games gameID.
	 * 
	 * @param id GameID received from server.
	 */
	public final void setGameID(long id){
		this.gameID = id;
	}
	/**
	 * Gets this games gameID.
	 * 
	 * @return gameID
	 */
	public final long getGameID(){
		return this.gameID;
	}

	/**
	 * Sets this games ConnectionHandler and registers this GameController as a listener
	 * for messages received from server for this GameControllers sessionID. ConnectionHandler
	 * is also used to send messages and you can get it by calling superclass method "getConnection()".
	 */
	public void setConnectionHandler(ConnectionHandler connection){
		if(connection != null){
			this.connection = connection;
			connection.registerMessageHandler(this);
		}
	}
	/**
	 * Returns ConnectionHandler associated with this GameController. If you want to send
	 * messages to server call this method to get the ConnectionHandler.
	 * 
	 * @return ConnectionHandler associated with this GameController.
	 */
	public ConnectionHandler getConnectionHandler(){
		return connection;
		
	}
	
	
}
