package com.asdf.gui.connect;

import java.awt.event.*;
import java.awt.*;

import javax.swing.*;
import java.net.URL;

import com.asdf.client.ConnectionHandler;
import com.asdf.client.ClientStateObserver;
import com.asdf.client.LobbyController;
import com.asdf.utils.ClientServerConstants;

import java.io.*;

/**
 * Panel containing green background image and connecting
 * fields and buttons.
 */
public class ConnectPanel extends JPanel implements ActionListener, ComponentListener, ClientStateObserver, KeyListener {

	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
	private JLabel lblTitle;
	private JLabel lblMessage;
	private JTextField txtAddress;
	private JButton btnConnect;
	
	/** Green background image */
	private Image backgroundImage;
	
	private ConnectFrame frame;
	
	/** Value to determine whether client is already connected */
	private boolean connected;
	
	private ConnectionHandler connHandler;
	private LobbyController ctrl;
	
	public ConnectPanel(ConnectFrame frame) {
		super();
		
		this.connected = false;
		
		this.frame = frame;
		
		ImageIcon ii;
		
		/* Try first to load image from resource and then from file
		 * if that fails.
		 */
		URL url = getClass().getResource("/images/ConnectFrameBackground.png");
		if (url == null)
			ii = new ImageIcon("images/ConnectFrameBackground.png");
		else
			ii = new ImageIcon(url);
		backgroundImage = ii.getImage();
		
		initGUI();
	}
	
	/**
	 * Initialize GUI contents
	 */
	private void initGUI() {
		// Disable layout manager
		setLayout(null);
		addComponentListener(this);
		lblTitle = new JLabel("Connect to server");
		lblTitle.setFont(new Font("Verdana", Font.PLAIN, 16));
		lblTitle.setForeground(Color.BLACK);
		
		lblMessage = new JLabel("",JLabel.CENTER);
		lblMessage.setForeground(new Color(217, 26, 26));
		lblMessage.setFont(new Font("verdana", Font.PLAIN, 16));
		
		txtAddress = new JTextField(25);
		txtAddress.setBackground(new Color(90, 210, 100));
				
		btnConnect = new JButton("CONNECT");
		btnConnect.setFont(new Font("courier", Font.PLAIN, 10));

		btnConnect.setForeground(Color.black);
		btnConnect.addActionListener(this);
		// Connect by pressing enter
		btnConnect.addKeyListener(this);
		txtAddress.addKeyListener(this);
		
		updateControlPositions();
		
		this.add(lblTitle);
		this.add(txtAddress);
		this.add(btnConnect);
		this.add(lblMessage);
	}
	
	/**
	 * Locates components to the center of the panel
	 */
	private void updateControlPositions() {
		int center_x = this.getWidth() / 2;
		int center_y = this.getHeight() / 2;
		int space = 5;
		int textbox_x = center_x - ((txtAddress.getWidth() + btnConnect.getWidth() + space) / 2);
		setLocation(lblTitle, textbox_x+(txtAddress.getWidth()-lblTitle.getWidth())/2, center_y - 20);
		setLocation(txtAddress, textbox_x, center_y);
		btnConnect.setBounds(textbox_x + txtAddress.getWidth() + space,center_y,100,20);
		lblMessage.setPreferredSize(new Dimension(txtAddress.getWidth(), 20));
		lblMessage.setBounds(textbox_x, center_y + 30, lblMessage.getPreferredSize().width, 20);
		repaint();
	}
	
	/**
	 * Takes X and Y parameters and positions given
	 * component to those with component's preferred
	 * width and height.
	 * @param c Component to position
	 * @param x X coordinate
	 * @param y Y coordinate
	 */
	private void setLocation(Component c, int x, int y) {
		Dimension d = c.getPreferredSize();
		c.setBounds(x, y, d.width, d.height);
	}
	
	/**
	 * Button handler.
	 */
	@Override
	public void actionPerformed(ActionEvent action) {
		if (action.getSource() == btnConnect) {
			connect();
		}
	}
	
	/**
	 * connect to a server with address defined in txtAddress
	 * text field.
	 */
	private void connect() {
		
		if (!connected) {
		
			this.btnConnect.setEnabled(false);
			this.txtAddress.setEnabled(false);
			int port = ClientServerConstants.PORT;
			
			try {
				connHandler = new ConnectionHandler(txtAddress.getText(), port);
				connHandler.registerConnectionObserver(this);
				
				Thread t = new Thread(connHandler);
				t.start();
				
			} catch (IOException e) {
				lblMessage.setText("Couldn't connect: " + e.getMessage());
				return;
			}
		}
	}
	
	/**
	 * Is called when ConnectionHandler finishes connecting
	 * successfully. This method hides connecting screen and
	 * displays lobby.
	 */
	@Override
	public void connected() {
		this.frame.setVisible(false);
		this.frame.setTitle("Connected to " + txtAddress.getText());
		ctrl = new LobbyController(connHandler);
		connected = true;
	}
	
	/**
	 * Is called when ConnectionHandler disconnects because
	 * of exception. Hides lobby if it's visible and display
	 * connecting screen. If exception is given, it's displayed
	 * to user.
	 */
	@Override
	public void disconnected(Exception e) {
		if (ctrl != null)
			ctrl.dispose();
		ctrl = null;
		this.frame.setVisible(true);
		this.btnConnect.setEnabled(true);
		this.txtAddress.setEnabled(true);
		if (e != null) {
			this.lblMessage.setText(e.getMessage());
		} else {
			this.lblMessage.setText("");
		}
		connected = false;
	}
	
	/**
	 * Override to draw background.
	 */
	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.drawImage(backgroundImage, 0, 0, this.getWidth(), this.getHeight(), this);
	}
	
	/**
	 * Catch resizes to update control positions
	 */
	@Override
	public void componentResized(ComponentEvent e) {
		updateControlPositions();
	}
	
	@Override
	public void componentHidden(ComponentEvent e) {}
	
	@Override
	public void componentMoved(ComponentEvent e) {}
	
	@Override
	public void componentShown(ComponentEvent e) {}

	public void keyPressed(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_ENTER) 
				connect();
	}
	public void keyReleased(KeyEvent e) {
	}
	public void keyTyped(KeyEvent e) {
	}
}
