package com.asdf.server;

import java.sql.*;
import java.util.TreeMap;

import com.asdf.common.AccountType;
import com.asdf.common.Logger;
import com.asdf.parser.AccountManagementResponse;
import com.asdf.parser.AuthenticationResponse;
import com.asdf.parser.GenericResponse;
import com.asdf.parser.LoginResponse;

/**
 * Handles creating, deleting, modifying and accessing
 * of accounts.
 */
public class AccountManager {
	private TreeMap<String, Account> accounts;
	private boolean dbAvailable;
	
	public AccountManager() {
		accounts = new TreeMap<String, Account>();
		dbAvailable = (getDbConnection() != null);
	}
	
	/**
	 * Get opened database connection
	 * @return Database connection
	 */
	private Connection getDbConnection() {
		Logger.get().fine("Opening with string " + Configuration.getAuthConnectionString());
		//Added for MySQL:
		try {
		Class.forName(Configuration.getAuthDriverString());
		} catch (Exception e) {System.out.println(e);}
		
		try {
			return DriverManager.getConnection(
					Configuration.getAuthConnectionString(),
					Configuration.getAuthUsername(),
					Configuration.getAuthPassword());
		} catch (SQLException se) {
			Logger.get().throwing("DriverManager", "getConnection", se);
			return null;
		}
	}

	
	/**
	 * Changes password of a given user, if the user performing this action is allowed to do so.
	 * @param username Account name of the user whose password is to be changed.
	 * @param password Old password of this user.
	 * @param newPassword New password.
	 * @return Returns true on success, false otherwise.
	 */
	public boolean changePassword(String username, String password, String newPassword) {
		if(!authenticate(username, password))
			return false;
		
		if (dbAvailable) {
			try {
				Connection c = getDbConnection();
				PreparedStatement s = c.prepareStatement(
						"update account set password=? where username=? and password=md5(?)");
				s.setString(1, newPassword);
				s.setString(2, username);
				s.setString(3, password);
				s.executeQuery();
				return true;
				
			} catch (SQLException e) {
				Logger.get().throwing("SQL", "", e);
				return false;
			}
		}
				
		return false;
	}
	
	
	/**
	 * Deletes an account from database and memory.
	 * @param username Account name to be deleted.
	 * @param password Password for the account, to check that this user is allowed to perform this action.
	 * @return returns true on successful deletion, false otherwise.
	 */
	public boolean deleteAccount(String username, String password) {
		
		if(!authenticate(username, password))
			return false;
		
		if (dbAvailable) {
			try {
				Connection c = getDbConnection();
				PreparedStatement s = c.prepareStatement(
						"delete * from account where username=? and password=md5(?)");
				s.setString(1, username);
				s.setString(2, password);
				s.executeQuery();
				return true;
				
			} catch (SQLException e) {
				Logger.get().throwing("SQL", "", e);
				return false;
			}
		}
		
		return false;
	}
	
	
	
	/**
	 * Try to authenticate user.
	 * @param username
	 * @param password
	 * @return Status
	 */
	private boolean authenticate(String username, String password) {
		try {
			Connection c = getDbConnection();
			PreparedStatement s = c.prepareStatement(
					"select * from account where username=? and password=md5(?)");
			s.setString(1, username);
			s.setString(2, username + password);
			ResultSet rs = s.executeQuery();
			int count = 0;
			short type = -1;
			double balance = 0.0;
			while (rs.next()) {
				type = rs.getShort("type");
				balance = rs.getDouble("balance");
				count++;
			}
			if (count == 1) {
				Account acc = accounts.get(username);
				if (acc == null) {
					if (type >= AccountType.values().length && type >= 0)
						throw new RuntimeException(
								"Database contained invalid type on user " + username);
					
					acc = new Account(username, AccountType.values()[type]);
					acc.setBalance(balance);
					
					accounts.put(username, acc);
				}
				return true;
			}
		} catch (SQLException e) {
			Logger.get().throwing("SQL", "", e);
			return false;
		}
		return false;
	}
	
	
	/**
	 * ConnectionInfo conn wants to log in with Account named account, using password passwd
	 * @param conn Specifies the client who is performing the action
	 * @param username The name of the account with which the user wishes to login
	 * @param passwd The password the user is sending to login
	 */
	public AuthenticationResponse logIn(ClientConnection conn, String username, String passwd) {
		AuthenticationResponse resp = new AuthenticationResponse();
		LoginResponse gresp = new LoginResponse();
		gresp.setName(username);
		gresp.setBalance(0.0d);
		gresp.setType((short)0);
		
		resp.setLogin(gresp);
		
		if (authenticate(username, passwd)) {
			Logger.get().info("User " + username + " logged in");
			Account acc = accounts.get(username);
			conn.login(acc);
			gresp.setType((short)acc.getType().ordinal());
			gresp.setBalance(acc.getBalance());
			gresp.setSuccess(true);
			
		} else {
			Logger.get().warning("User " + username + " tried to log in with wrong password/account");
			// log in failed
			gresp.setSuccess(false);
			gresp.setDescription("Authentication failed, wrong username/password");
		}
		
		return resp;
	}
	
	/**
	 * Logout method.
	 * @param conn
	 */
	public AuthenticationResponse logOut(ClientConnection conn) {
		AuthenticationResponse resp = new AuthenticationResponse();
		GenericResponse gresp = new GenericResponse();
		resp.setLogout(gresp);
		
		if (conn.getAccount() != null) {
			Logger.get().info(
					conn.getAccount().getName() + " has logged out.");
			conn.logout();
			gresp.setSuccess(true);
		} else {
			Logger.get().info("Guest user logged out.");
			gresp.setSuccess(false);
			gresp.setDescription("Guest can't logout");
		}
		return resp;
	}

	/**
	 * Handles account creating requests.
	 * @param conn The client connection used by the user.
	 * @param accountName Account name that is requested for creation.
	 * @param password Password for the new account.
	 * @param type Type of the account.
	 * @return Response message to the attempt.
	 */
	public AccountManagementResponse createAccount (
			ClientConnection conn, String accountName, String password, int type) {
		AccountManagementResponse resp = new AccountManagementResponse();
		GenericResponse gresp = new GenericResponse();
		resp.setCreate(gresp);
		gresp.setSuccess(false);
		
		try {
			Connection c = getDbConnection();
			Logger.get().fine("Creating account " + accountName + " with password " + password);
			PreparedStatement s = c.prepareStatement(
					"insert into account values(?, MD5(?), ?, ?)");
			s.setString(1, accountName);
			s.setString(2, accountName+password);
			s.setInt(3, type);
			s.setDouble(4, 0.0);
			s.execute();
			gresp.setSuccess(true);
			Logger.get().info("Created account " + accountName);
		} catch (SQLException e) {
			Logger.get().throwing("SQL", "", e);
			gresp.setDescription(e.getMessage());
		}
		
		return resp;
	}
	
	/**
	 * Change user credit balance. Positive amount increases the
	 * balance by given amount and negative does the other way around.
	 * @param amount Difference in balance
	 * @param account Account on which to make the change
	 */
	public void setBalanceChange(String gameName, double amount, Account account) {
		if(account != null) {
			
			account.scores.addScore(gameName, (int)amount);
			
			try {
				Connection c = getDbConnection();
				Logger.get().fine("Updating user " + account.getName() + " balance.");
				PreparedStatement psCurrBalance = c.prepareStatement(
					"select balance from account where username=?");
				psCurrBalance.setString(1, account.getName());
				ResultSet rsCurrBalance = psCurrBalance.executeQuery();
				if (rsCurrBalance.next()) {
					double oldAmount = rsCurrBalance.getDouble(1);
					double newAmount = oldAmount + amount;
					
					rsCurrBalance.close();
					psCurrBalance.close();
					PreparedStatement psSetBalance = c.prepareStatement(
							"update account set balance=? where username=?");
					psSetBalance.setDouble(1, newAmount);
					psSetBalance.setString(2, account.getName());
					psSetBalance.execute();
					Logger.get().fine("Updated balance on user " + account.getName() + " to " +
							newAmount);
				} else {
					Logger.get().warning("Tried to set balance on non-existing account!");
				}
			} catch (SQLException e) {
				Logger.get().throwing("SQL", "", e);
			}
		}
	}
}
