package com.asdf.server;

import java.io.*;
import java.net.InetSocketAddress;
import java.net.InetAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

import javax.xml.bind.JAXBException;

import com.asdf.utils.ClientServerConstants;
import com.asdf.common.Logger;
import com.asdf.parser.Message;
import com.asdf.utils.XMLTools;

/**
 * Contains functionality required to manage single channel
 * with multiple recipients and senders.
 * This class is accessed from multiple threads. Send is accessed from
 * Game instances and don't need synchronization as it doesn't modify
 * class's data. Server's single thread is the only one that modifies data
 * so synchronization isn't needed.
 */
public class ClientConnection {
	
	/**
	 * Possible states for data receiving.
	 */
	public enum State {
		/** Waiting for the size of the message */
		WAITINGSIZE,
		/** Waiting for the actual message */
		WAITINGDATA
	}

	
	/**  The account to which this connection is mapped to */
	private Account account;

	/** Server instance used for various manager */
	private Server server;
	
	/** SelectionKey of the client, connection identifier for java.nio libraries */
	private SelectionKey key;

	/** Receiving state of the object */
	private State receivingState;
	
	/** Handler for incoming messages */
	private MessageHandler handler;
	
	/** Remote address of the client */
	private InetSocketAddress address;

	/** XMLTools handling (un)marshalling of messages */
	private XMLTools xmltools = new XMLTools();
	
	public ByteBuffer sizeBuffer = ByteBuffer.allocate(4);
	public ByteBuffer msgBuffer;
	public int size;
	
	public boolean disposing = false;

	

	/**
	 * Constructs ClientConnection. Sets receivingState to
	 * default State.WAITINGSIZE.
	 * @param server
	 */
	public ClientConnection(Server server, SelectionKey key, InetSocketAddress address) {
		this.account = null;
		this.server = server;
		this.key = key;
		this.address = address;
		this.handler = new MessageHandler(this);
		receivingState = State.WAITINGSIZE;
	}
	
	/**
	 * Getter for server
	 * @return Server instance
	 */
	public Server getServer() {
		return server;
	}

	/**
	 * Getter for account
	 * @return Client account
	 */
	public Account getAccount() {
		return account;
	}

	/**
	 * Getter for receivingState
	 * @return Receiving state
	 */
	public State getReceivingState() {
		return receivingState;
	}
	
	/**
	 * Getter method for connection channel key.
	 */
	public SelectionKey getKey() {
		return key;
	}
	
	/**
	 * Getter for client remote address string
	 */
	public String getAddressString() {
		return address.toString();
	}
	
	/**
	 * Getter for client ip string
	 */
	public String getAddress() {
		return address.getAddress().getHostAddress();
	}
	
	/**
	 * Getter for client ip address
	 * @return IP address in byte array in network byte order
	 */
	public InetAddress getRawAddress() {
		return address.getAddress();
	}

	/**
	 * SizeReceived
	 * Is called when size has been read from the
	 * socket. Function initializes message buffer
	 * to be large enough for to be received message.
	 * Function allocates new buffer if previous one doesn't
	 * exist, is small enough or too large (more than 
	 * WASTING_SPACE_RATIO times as large as the current 
	 * message).
	 * @throws IOException
	 */
	public void sizeReceived() throws IOException {
		sizeBuffer.rewind();
		size = sizeBuffer.getInt();
		
		/*
		 * Perform sanity check for message size.
		 * Exception causes Server to disconnect this
		 * client.
		 */
		if (size > ClientServerConstants.TOO_BIG_MESSAGE) {
			throw new IOException("Message maximum size exceeded");
		}

		if (msgBuffer == null
				|| msgBuffer.capacity() < size
				|| msgBuffer.capacity() > (size * ClientServerConstants.WASTING_MEM_RATIO)) {
			msgBuffer = ByteBuffer.allocate(size);
			msgBuffer.clear();
		} else {
			msgBuffer.clear();
			msgBuffer.limit(size);
		}

		receivingState = State.WAITINGDATA;
	}

	/**
	 * MsgReceived
	 * Called when message has been received.
	 */
	public void msgReceived() throws IOException {
		byte[] data = new byte[this.size];
		msgBuffer.rewind();		
		msgBuffer.get(data, 0, this.size);
		Logger.get().finest("Received: {" + new String(data) + "}");
		handler.process(data);
		sizeBuffer.clear();
		
		receivingState = State.WAITINGSIZE;
	}
	
	/**
	 * Sends message with it's size to client.
	 * @param msg Message to send
	 * @throws IOException
	 */
	public void send(Message msg) {
		try {
			send(xmltools.messageToBytes(msg));
		} catch (JAXBException e) {
			Logger.get().throwing("ClientConnection", "send", e);
		} catch (IOException e) {
			Logger.get().throwing("ClientConnection", "send", e);
		}
	}

	/**
	 * Sends data to recipients.
	 * @param data
	 * @throws IOException
	 */
	public void send(byte[] data) throws IOException {
		
		Logger.get().finest("Sending {" + new String(data) + "}");
		
		ByteBuffer sendBuffer = ByteBuffer.allocate(data.length + 4);
		sendBuffer.putInt(data.length);
		sendBuffer.put(data);
		sendBuffer.rewind();
		SocketChannel socket = (SocketChannel)key.channel();
		server.send(socket, sendBuffer);
		Logger.get().finest("Sent " + data.length + " bytes of data to client " +
				this.address.toString());
	}

	/**
	 * Mark connection authenticated.
	 * @param acc Valid account
	 */
	public synchronized void login(Account acc) {
		account = acc;
		Logger.get().info(acc.getName() + " has logged in.");
	}

	/**
	 * Mark connection unauthenticated.
	 */
	public synchronized void logout() {
		account = null;
	}

}
