package com.asdf.server;

import java.util.Properties;
import java.io.FileInputStream;

import com.asdf.common.Logger;

/**
 * Configuration class handling loading
 * of configuration files.
 */
public class Configuration {
	
	/** Shared Configuration instance */
	private static Configuration configuration = null;
	
	/** Properties instance */
	private Properties properties;
	
	/**
	 * Returns instance of Configuration and creates
	 * one if it doesn't already exist. Tries 
	 * @return Configuration instance
	 */
	public static Configuration getInstance() {
		if (configuration == null) {
			configuration = new Configuration();
		}
		return configuration;
	}
	
	/**
	 * Protected constructor. Can't be used from outside of the
	 * class. Tries to load configuration from server.conf from
	 * working directory.
	 */
	protected Configuration() {
		properties = new Properties();
		try {
			FileInputStream fis = new FileInputStream("server.conf");
			properties.load(fis);
		}
		catch (Exception e) {
			Logger.get().throwing("Configuration", "Constructor", e);
		}
	}
	
	/**
	 * Plugin classpaths. Paths are went through in order.
	 * Paths are limited by comma.
	 * @return Order of classpaths
	 */
	public static String[] pluginPaths() {
		Configuration c = getInstance();
		String paths = c.properties.getProperty("pluginpaths");
		if (paths != null) {
			return paths.split(",");
		}
		else {
			return new String[]{};
		}
	}
	
	/**
	 * Getter for database connection string
	 * @return Connection string
	 */
	public static String getAuthConnectionString() {
		Configuration c = getInstance();
		return c.properties.getProperty("auth_db_conn");
	}
	
	/**
	 * Getter for game database connection string
	 * @return Connection string
	 */
	public static String getGameConnectionString() {
		Configuration c = getInstance();
		return c.properties.getProperty("game_db_conn");
	}
	
	/**
	 * Getter for database driver string
	 * @return Driver string
	 */
	public static String getAuthDriverString() {
		Configuration c = getInstance();
		return c.properties.getProperty("auth_db_driver");
	}
	
	/**
	 * Getter for game database driver string
	 * @return Driver string
	 */
	public static String getGameDriverString() {
		Configuration c = getInstance();
		return c.properties.getProperty("game_db_driver");
	}
	
	/**
	 * Getter for database username
	 * @return Username
	 */
	public static String getAuthUsername() {
		Configuration c = getInstance();
		return c.properties.getProperty("auth_db_username");
	}
	
	/**
	 * Getter for game database username
	 * @return Username
	 */
	public static String getGameUsername() {
		Configuration c = getInstance();
		return c.properties.getProperty("auth_db_username");
	}
	
	/**
	 * Getter for database password
	 * @return Password
	 */
	public static String getAuthPassword() {
		Configuration c = getInstance();
		return c.properties.getProperty("auth_db_password");
	}
	
	/**
	 * Getter for database password
	 * @return Password
	 */
	public static String getGamePassword() {
		Configuration c = getInstance();
		return c.properties.getProperty("auth_db_password");
	}
	
	/**
	 * Logging configuration file. If configuration doesn't
	 * define a loggingconf settings then "logging.properties" is
	 * returned.
	 * @return Path to the logging configuration file
	 */
	public static String loggingConfigurationFile() {
		Configuration c = getInstance();
		return c.properties.getProperty("loggingconf", "logging.properties");
	}
}
