package com.asdf.server;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.io.File;
import java.util.Scanner;
import java.io.FileNotFoundException;

import com.asdf.common.Game;
import com.asdf.common.GameTimerTask;
import com.asdf.common.Logger;
import com.asdf.common.PluginLoadException;
import com.asdf.parser.AvailableGamesResponse;
import com.asdf.parser.GameDetailsResponse;
import com.asdf.parser.GameRequest;
import com.asdf.parser.GameResponse;
import com.asdf.parser.CreateGameResponse;
import com.asdf.parser.OptionalStringParameters;
import com.asdf.parser.GameListResponse;
import com.asdf.parser.GameDetailsResponse.Players;

/**
 * Handles creating, deleting and listing of games.
 */
public class GameManager {
	
	/** Games indexed by game ID */
	private HashMap<Long, Game> games;
	
	/** GameID counter telling next free game ID */
	private long gameIdCounter = 0;
	
	/** Timer used in games */
	private final GameTimer timer = new GameTimer();
	
	private static final String META_FILES_DIRECTORY = "plugins/meta";
	
	public class GameJoinException extends Exception {
		private static final long serialVersionUID = 1L;

		public GameJoinException(String reason) {
			super(reason);
		}
	}
	
	public GameManager() {
		this.games = new HashMap<Long, Game>();
	}
	
	/**
	 * Returns next free game ID. Simple
	 * increment on long variable.
	 * @return game ID
	 */
	public long getFreeGameID() {
		return gameIdCounter++;
	}
	
	/**
	 * Returns the Timer object that can be used for synchronization
	 * in Game implementations.
	 * @return Timer
	 */
	public GameTimer getTimer() {
		return timer;
	}
	
	/**
	 * Gives a user action to a game instance for processing.
	 * @param request GameRequest object containing possible custom commands needed by the game
	 * @param sessionID Session ID of the source of the current message
	 * @param client Client information
	 */
	public void deliverAction(GameRequest request, long sessionID, ClientConnection client) {
		Game game = games.get(request.getGameID());
		Logger.get().fine("Delivering action to game " + game.toString());
		if(game != null) {
			game.handleRequestSync(sessionID, request);
		}
	}
	
	/**
	 * Create game with specified parameters.
	 * @param request GameRequest object containing possible custom commands 
	 * needed by Game
	 * @param sessionID Session ID of the source of the current message
	 * @param client Client information to obtain a Server instance
	 * @return boolean Whether game was created successfully
	 */
	public GameResponse createGame(GameRequest request, long sessionID, ClientConnection client) {
		
		GameResponse resp = new GameResponse();
		CreateGameResponse gres = new CreateGameResponse();
		gres.setSuccess(false);
		resp.setCreate(gres);
		Server server = client.getServer();
		
		Account acc = client.getAccount();
		if (acc != null) {
		
			try {
				long newGameID = getFreeGameID();
				Game newGame = GameFactory.createGame(
						request.getCreate().getGameType(), server, newGameID, request.getCreate().getName());
				newGame.handleRequestSync(sessionID, request);
				games.put(newGameID, newGame);
				newGame.handleRequestSync(sessionID, request);
				gres.setSuccess(true);
				gres.setGameID(newGameID);
				
			}
			catch(PluginLoadException e) {
				Logger.get().throwing("GameFactory", "createGame", e);
			}
		} else {
			Logger.get().warning("Fail: guest user tried to create a game: " + 
					request.getCreate().getGameType());
			gres.setSuccess(false);
			gres.setDescription("Login required");
		}
		
		return resp;
	}
	
	
	/**
	 * Join game.
	 * @param request GameRequest containing optional parameters
	 * @param sessionID Session ID of the current message
	 * @param client ClientConnection from which the message was received.
	 * @throws GameJoinException 
	 */
	public void joinGame(GameRequest request, long sessionID, ClientConnection client) throws GameJoinException {
	
		Game myGame = games.get(request.getGameID());

		// If game is found, join
		if (myGame == null) {
			throw new GameJoinException("Game doesn't exist");
		}
		else {
			myGame.handleRequestSync(sessionID, request);
		}
		
		
	}
	
	/**
	 * Part game.
	 * @param request 
	 * @param sessionID
	 * @param client
	 */
	public void partGame(GameRequest request, long sessionID, ClientConnection client) {
		Account acc = client.getAccount();
		if (acc != null) {
			Game myGame = acc.getSession(sessionID);
			if (myGame != null) {
				acc.removeSession(sessionID);
				myGame.handleRequestSync(sessionID, request);
			} else {
				// TODO: Error, client sent session ID that does not exist. Notify client.
			}
		} else {
			Logger.get().warning(
					"Guest user tried to leave a game. Operation not allowed.");
		}
	}
	
	/**
	 * Forcefully deletes an active game.
	 * @param request A GameRequest delivering the gameID to be deleted.
	 * @param sessionID SessionID of the user who is performing this action.
	 * @param client Client information of the user who is performing this action.
	 */
	public void deleteGame(GameRequest request, long sessionID, ClientConnection client) {
		games.remove(request.getGameID());
	}

	/**
	 * Send chat message to Game
	 * @param request GameRequest
	 * @param sessionID Session ID
	 * @param client ClientConnection
	 */
	public void chat(GameRequest request, long sessionID, ClientConnection client) {
		Game g = games.get(request.getGameID());
		if (g != null) {
			g.handleRequestSync(sessionID, request);
			Logger.get().fine("Delivered chat message to game " + g);
		}
	}
	
	/**
	 * Send voice message to Game
	 * @param request GameRequest
	 * @param sessionID Session ID
	 * @param client ClientConnection
	 */
	public void voice(GameRequest request, long sessionID, ClientConnection client) {
		Game g = games.get(request.getGameID());
		if (g != null) {
			g.handleRequestSync(sessionID, request);
			Logger.get().fine("Delivered voice message to game " + g);
		}
	}
	
	/**
	 * Handles game responses.
	 * @param response GameResponse delivering user information.
	 * @param sessionID Session ID of the user who sent the response.
	 * @param client Client information of the user.
	 */
	public void handleGameResponse(GameResponse response, long sessionID, ClientConnection client) {
		Game game = games.get(response.getGameID());
		if(game != null) {
			game.handleResponseSync(sessionID, response);
		}
	}
	
	/**
	 * Reads meta data about games available in this server and sends this to client.
	 * @param gameTypes a reference to a list of game meta data containers.
	 */
	public void getGameTypes(List<AvailableGamesResponse.Game> gameTypes) {
		File gamesMetaFilesDir = new File(META_FILES_DIRECTORY);
		if(gamesMetaFilesDir.isDirectory()) {
			String metaFiles[] = gamesMetaFilesDir.list();
			Logger.get().finest("Found " + metaFiles.length + " game descriptor meta files.");
			for(int i = 0; i < metaFiles.length; i++) {
				File metaFile = new File("plugins/meta/" + metaFiles[i]);
				if(true) {
					Scanner sc;
					try {
						sc = new Scanner(metaFile);
					} catch (FileNotFoundException e) {
						Logger.get().severe("Game meta descriptor ("+ metaFile.getAbsolutePath() + 
								") returned by file list not found??!");
						Logger.get().info(metaFiles[i]);
						continue;
					}
					
					AvailableGamesResponse.Game game = new AvailableGamesResponse.Game();
					Logger.get().finest("Reading meta file " + i);
					
					while(sc.hasNextLine()) {
						String line = sc.nextLine();
						Scanner in_s = new Scanner(line);
						while(in_s.hasNext()) {
							String s = in_s.next();
							if(s.charAt(0) == '#')
								break;
							if(s.toLowerCase().equals("gamename") && in_s.hasNext()) {
								if(!in_s.hasNext())
									continue;
								String s_temp = in_s.next();
								Logger.get().finest("Found game name tag. Name was: " + s_temp);
								game.setType(s_temp);
								break;
							}
							if(s.toLowerCase().equals("panelname") && in_s.hasNext()) {
								if(!in_s.hasNext())
									continue;
								String s_temp = in_s.next();
								Logger.get().finest("Found game panel tag. Panel is found from: " + s_temp);
								game.setCreateGamePanel(s_temp);
								break;
							}
							if(s.toLowerCase().equals("servergame") && in_s.hasNext()) {
								if(!in_s.hasNext())
									continue;
								String s_temp = in_s.next();
								Logger.get().finest("Found server game tag. Name was: " + s_temp);
								game.setServerGame(s_temp);
								break;
							}
							if(s.toLowerCase().equals("clientgame") && in_s.hasNext()) {
								if(!in_s.hasNext())
									continue;
								String s_temp = in_s.next();
								Logger.get().finest("Found client game tag. Name was: " + s_temp);
								game.setClientGame(s_temp);
								break;
							}
							if(s.toLowerCase().equals("variantnames")) {
								OptionalStringParameters val = new OptionalStringParameters();
								List<String> variants = val.getString();
								while(in_s.hasNext())
								{
									String s_temp = in_s.next();
									Logger.get().finest("Game variant name found. Name was: " + s_temp);
									variants.add(s_temp);
								}
								game.setVariants(val);
								break;
							}
						}
					}
					if(game.getCreateGamePanel() != null && game.getServerGame() != null &&
							game.getType() != null && game.getClientGame() != null) {
						gameTypes.add(game);
					}
				} else {
					Logger.get().severe("Game information meta file not found.");
				}
			}
		} else {
			Logger.get().severe("Game meta files directory not found!");
		}
	}
	
	/**
	 * Collects a list of game descriptions to be sent back to client. Method
	 * also queries list of column names from the first encountered instance.
	 * @param gameList A list of Game description objects.
	 * @param columnNames A list of column names
	 * @param gameType Type of the game
	 * @throws ClassNotFoundException 
	 */
	@SuppressWarnings("unchecked")
	public void listGames(
			List<GameListResponse.Games.Game> gameList, List<String> columnNames, String gameType)
	throws ClassNotFoundException {
		boolean queriedColumnNames = false;
		Class searchType = Class.forName(gameType);
		for(Game game : games.values()) {
			if (searchType == game.getClass()) {
				if (!queriedColumnNames) {
					queriedColumnNames = true;
					for(String column : game.columnNamesSync())						
						columnNames.add(column);
				}
				GameListResponse.Games.Game tmp_game = game.descriptionSync();
				gameList.add(tmp_game);				
			}
			else {
				Logger.get().fine("Class " + game.getClass().getName() + " didn't match " +
						gameType);
			}
		}
	}
	
	/**
	 * Asks for detailed information from a single game.
	 * @param gres Message content object describing details of a game.
	 * @param gameID ID to identify a given game.
	 */
	public void getGameDetails(GameDetailsResponse gres, Long gameID) {
		Game game = games.get(gameID);
		if(game == null)
		{
			Logger.get().fine("User requested information about a game that does not exist.");
			
			OptionalStringParameters fake = new OptionalStringParameters();
			gres.setColumnNames(fake);
			gres.setPlayers(new Players());
			gres.setRules(new OptionalStringParameters());
			
			return;
		}
		
		game.detailsSync(gres);
	}
	
	
	/**
	 * Getter for database game connection.
	 * @return the database game connection
	 */
	
	public Connection getDbGameConnection() {
		
		try {
		Class.forName(Configuration.getGameDriverString());
		} catch (Exception e) {System.out.println(e);}
		try {
			return DriverManager.getConnection(
					Configuration.getGameConnectionString(),
					Configuration.getGameUsername(),
					Configuration.getGamePassword());
		} catch (SQLException se) {
			Logger.get().throwing("DriverManager", "getConnection", se);
			return null;
		}
	}

	/**
	 * Checks wether a game instance is futile. If a game is not used in a long time
	 * by anyone, the game is considered trash and removed.
	 * @param id Game instance id.
	 * @param task The timer ask that invokes this check.
	 */
	public void checkDelete(long id, GameTimerTask task) {
		Game game = this.games.get(id);
		
		GameDetailsResponse a = new GameDetailsResponse();
		game.detailsSync(a);
		
		if(a.getPlayers().getPlayer().size() == 0) {
			if(game.age++ > 6) {
				game.dieSync();
				games.remove(id);
				task.cancel();
			}	
		}
		else {
			game.age = 0;
		}
	}
	
	
	/**
	 * Finalize GameManager by canceling timer
	 */
	@Override
	public void finalize() {
		Logger.get().severe("FINALIZE OF GAMEMANAGER CALLED!! Not good!!11++ lol.");
		timer.cancel();
	}
	
}
