package com.asdf.server;

import java.util.Timer;

import com.asdf.common.GameTimerTask;
import com.asdf.common.Logger;

/**
 * Timer class providing thread safe timer for
 * Game instances.
 */
public class GameTimer {

	private Timer timer = new Timer();
	
	public GameTimer() {
		super();
	}
	
	/**
     * Schedules the specified task for execution after the specified delay.
     *
     * @param task  task to be scheduled.
     * @param delay delay in milliseconds before task is to be executed.
     * @throws IllegalArgumentException if <tt>delay</tt> is negative, or
     *         <tt>delay + System.currentTimeMillis()</tt> is negative.
     * @throws IllegalStateException if task was already scheduled or
     *         cancelled, or timer was cancelled.
     */
	public synchronized void schedule(GameTimerTask task, long delay) {
		try {
			timer.schedule(task, delay);
		}
		catch(IllegalStateException e) {
			Logger.get().severe("Caught timer already cancelled. Doing something about it..");
			timer = new Timer();
			timer.schedule(task, delay);
		}
	}
	
	/**
     * Schedules the specified task for repeated <i>fixed-delay execution</i>,
     * beginning after the specified delay.  Subsequent executions take place
     * at approximately regular intervals separated by the specified period.
     *
     * <p>In fixed-delay execution, each execution is scheduled relative to
     * the actual execution time of the previous execution.  If an execution
     * is delayed for any reason (such as garbage collection or other
     * background activity), subsequent executions will be delayed as well.
     * In the long run, the frequency of execution will generally be slightly
     * lower than the reciprocal of the specified period (assuming the system
     * clock underlying <tt>Object.wait(long)</tt> is accurate).
     *
     * <p>Fixed-delay execution is appropriate for recurring activities
     * that require "smoothness."  In other words, it is appropriate for
     * activities where it is more important to keep the frequency accurate
     * in the short run than in the long run.  This includes most animation
     * tasks, such as blinking a cursor at regular intervals.  It also includes
     * tasks wherein regular activity is performed in response to human
     * input, such as automatically repeating a character as long as a key
     * is held down.
     *
     * @param task   task to be scheduled.
     * @param delay  delay in milliseconds before task is to be executed.
     * @param period time in milliseconds between successive task executions.
     * @throws IllegalArgumentException if <tt>delay</tt> is negative, or
     *         <tt>delay + System.currentTimeMillis()</tt> is negative.
     * @throws IllegalStateException if task was already scheduled or
     *         cancelled, timer was cancelled, or timer thread terminated.
     */
	public synchronized void schedule(GameTimerTask task, long delay, long period) {
		timer.schedule(task, delay, period);
	}

	/**
	 * Cancel all current tasks.
	 */
	public synchronized void cancel() {
		Logger.get().severe("Someone is trying to kill teh timer!!");
		timer.cancel();
	}

}
