package com.asdf.server;

import com.asdf.server.GameManager.GameJoinException;
import com.asdf.utils.XMLTools;
import com.asdf.common.Logger;
import com.asdf.parser.*;

import javax.xml.bind.JAXBException;
import java.io.IOException;
import java.util.List;


/**
 * Handles incoming messages.
 */
public class MessageHandler {
	
	/** Handle to the Server instance */
	private ClientConnection client;
	private Server server;
	private AccountManager accountManager;
	private GameManager gameManager;
	private Message currentMessage;
	private XMLTools xmltools;
	
	public MessageHandler(ClientConnection client) {
		this.client = client;
		
		xmltools = new XMLTools();
		server = client.getServer();
		accountManager = server.getAccountManager();
		gameManager = server.getGameManager();
	}
	
	/**
	 * Generate Message with general error response set
	 * to given value.
	 * @param success Value of the success attribute
	 * @return Generated Message
	 */
	public Message generateGeneralErrorResponse(boolean success) {
		Message msg = new Message();
		Response resp = new Response();
		GenericResponse gres = new GenericResponse();
		gres.setSuccess(success);
		resp.setGeneral(gres);
		msg.setResponse(resp);
		return msg;
	}
	
	/**
	 * Generate Message with general error response set
	 * to given value.
	 * @param success Value of the success attribute
	 * @param description Value of the description element
	 * @return Generated Message
	 */
	public Message generateGeneralErrorResponse(boolean success, String description) {
		Message msg = generateGeneralErrorResponse(success);
		msg.getResponse().getGeneral().setDescription(description);
		return msg;
	}
	
	/**
	 * Checks that session is valid and if ID haven't been seen
	 * before it is stored. This method also checks that client
	 * isn't trying to impersonate another client.
	 * @param msg
	 * @return 
	 */
	private boolean processSession(Message msg) {
		long id = msg.getSessionID();
		
		if (id == 0) {
			Logger.get().warning(
					"Client from " + client.getAddressString() + " tried to use " +
					" zero sessionID");
			Message respMsg = generateGeneralErrorResponse(false, "Session ID must be defined");
			client.send(respMsg);
			return false;
		} 
		
		ClientConnection cc = server.getSessionManager().getBySessionID(id);
		if (cc != null && cc != this.client) {
			Logger.get().severe(
					"Client tried to use session ID owned by another connection! " +
					"Banning client.");
			Message respMsg = generateGeneralErrorResponse(false, "Busted & banned");
			client.send(respMsg);
			server.ban(this.client);
			return false;
		}
		else if (cc == null) {
			server.getSessionManager().put(id, this.client);
		}
		
		return true;	
	}
	
	/**
	 * Process message. 
	 * @param rawData the message
	 */
	public void process(byte[] rawData) throws IOException {
		Message msg = null;
		try {
			msg = xmltools.bytesToMessage(rawData);
		} catch (JAXBException e) {
			Message resp = generateGeneralErrorResponse(false,
					"Malformed message: " + e.getCause());
			client.send(resp);
			throw new IOException(e);
		}
		currentMessage = msg;
		
		
		
		if (!processSession(msg))
			return;
		
		if (msg.getRequest() != null) {
			if(!handleRequest(rawData, msg.getRequest())) {
				Logger.get().warning("Couldn't handle received message");
				Message respMsg = generateGeneralErrorResponse(false,
						"Message not supported");
				client.send(respMsg);
			}
		}
		else if(msg.getResponse() != null) {
			handleResponse(rawData, msg.getResponse());
		}
		
	}
	
	/**
	 * Handle Request
	 * @param rawData Document as raw byte array
	 * @param request De-serialized Request object
	 */
	private boolean handleRequest(byte[] rawData, Request request) {		
		if (request.getAccountManagement() != null)
			return handleAccountManagement(rawData, request.getAccountManagement());
		else if (request.getAuthentication() != null)
			return handleAuthentication(rawData, request.getAuthentication());
		else if (request.getGame() != null)
			return handleGame(rawData, request.getGame());
		else if(request.getLobby() != null)
			return handleLobbyRequest(rawData, request.getLobby());
		return false;
	}
	
	
	private boolean handleLobbyRequest(byte[] rawData, LobbyRequest lobbyreq) {
		
		if(lobbyreq.getAvailableGames() != null)
		{
			handleAvailableGamesRequest(lobbyreq, currentMessage.getSessionID(), client);
			return true;
		}
		
		if(lobbyreq.getGameList() != null)
		{
			handleGetGameList(lobbyreq, currentMessage.getSessionID(), client);
			return true;
		}
		
		if(lobbyreq.getGameDetails() != null)
		{
			handleGetGameDetails(lobbyreq, currentMessage.getSessionID(), client);
			return true;
		}
		
		return false;
	}
	
	
	/**
	 * Handle AccountManagement
	 * @param rawData Document as raw byte array
	 * @param management De-serialized AccountManagement object
	 */
	private boolean handleAccountManagement(byte[] rawData, AccountManagementRequest management) {
		Message msg = new Message();
		Response r = new Response();
		msg.setResponse(r);
		msg.setSessionID(currentMessage.getSessionID());
		
		if (management.getCreate() != null) {
			AccountManagementRequest.Create c = management.getCreate();
			AccountManagementResponse amRes = accountManager.createAccount(
					client, c.getAccountName(), c.getPassword(), 3);
			r.setAccountManagement(amRes);
			client.send(msg);
			return true;
		}
		else if (management.getChangePassword() != null) {
			AccountManagementRequest.ChangePassword c = management.getChangePassword();
			accountManager.changePassword(c.getAccountName(), c.getOldPassword(), c.getNewPassword());
			return true;
		} else if (management.getDelete() != null) {
			AccountManagementRequest.Delete c = management.getDelete();
			accountManager.deleteAccount(c.getAccountName(), c.getPassword());
			return true;
		}
		
		// Unknown request.
		return false;
	}
	
	/**
	 * Handle Authentication
	 * @param rawData Document as raw byte array
	 * @param authentication De-serialized Authentication object
	 */
	private boolean handleAuthentication(byte[] rawData, AuthenticationRequest authentication) {
		Message msg = new Message();
		Response r = new Response();
		msg.setResponse(r);
		msg.setSessionID(currentMessage.getSessionID());
		
		if (authentication.getLogin() != null) {
			AuthenticationRequest.Login login = authentication.getLogin();
			AuthenticationResponse aresp = accountManager.logIn(client, login.getAccountName(), login.getPassword());
			r.setAuthentication(aresp);
			client.send(msg);
			return true;
		}
		else if (authentication.getLogout() != null) {
			AuthenticationResponse aresp = accountManager.logOut(client);
			r.setAuthentication(aresp);
			client.send(msg);
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * Handle Game
	 * @param rawData Document as raw byte array
	 * @param game De-serialized Game object
	 */
	private boolean handleGame(byte[] rawData, GameRequest game) {
		if (game.getCreate() != null) {
			GameResponse gameResp = gameManager.createGame(game, currentMessage.getSessionID(), client);
			Logger.get().fine("Tried to create a game, sending response of success");
			Message msg = new Message();
			Response resp = new Response();
			resp.setGame(gameResp);
			msg.setResponse(resp);
			msg.setSessionID(currentMessage.getSessionID());
			client.send(msg);
			return true;
		}
		else if (game.getJoin() != null) {	
			try {
				gameManager.joinGame(game, currentMessage.getSessionID(), client);
			} catch (GameJoinException e) {
				Logger.get().warning("Got a join game message related to a game that does not exist.");
				Message msg = new Message();
				Response resp = new Response();
				GameResponse gameResp = new GameResponse();
				resp.setGame(gameResp);
				msg.setResponse(resp);
				msg.setSessionID(currentMessage.getSessionID());
				GenericResponse gres = new GenericResponse();
				gameResp.setJoin(gres);
				gres.setSuccess(false);				
				gres.setDescription(e.getMessage());
				client.send(msg);
			}
			return true;
		}
		else if (game.getAction() != null) {
			gameManager.deliverAction(game, currentMessage.getSessionID(), client);
			return true;
		}
		else if (game.getPart() != null) {
			gameManager.partGame(game, currentMessage.getSessionID(), client);
			return true;
		}
		else if (game.getDelete() != null) {
			gameManager.deleteGame(game, currentMessage.getSessionID(), client);
			return true;
		} 
		else if (game.getChat() != null) {
			gameManager.chat(game, currentMessage.getSessionID(), client);
			return true;
		}
		else if (game.getVoice() != null) {
			gameManager.voice(game, currentMessage.getSessionID(), client);
			return true;
		}
		
		return false;
	}
	
	/**
	 * Handle response
	 * @param rawData Document as raw byte array
	 * @param response De-serialized Response object
	 */
	private void handleResponse(byte[] rawData, Response response) {
		if(response.getAuthentication() != null)
			return; // don't give a shit about authentication replies.
		if(response.getAccountManagement() != null)
			return; // don't give a shit about account management replies.
	
		if(response.getGame() != null) {
			// this is something we actually care a little bit about.
			gameManager.handleGameResponse(response.getGame(), currentMessage.getSessionID(), client);
			return;
		}
		
		if(response.getGeneral() != null)
			return; // for now, don't give a shit.
		
		if(response.getLobby() != null)
			return; // for now, don't give a shit.
			
	}
	
	/**
	 * Handles requests that ask for a list of possible game types known to this server.
	 * @param lobbyreq Request instance.
	 * @param sessionID Session ID for the requestor.
	 * @param client ClientConnection to the requestor.
	 */
	private void handleAvailableGamesRequest(LobbyRequest lobbyreq, long sessionID, ClientConnection client) {
		Message msg = new Message();
		Response rsp = new Response();
		LobbyResponse res = new LobbyResponse();
		AvailableGamesResponse ares = new AvailableGamesResponse();
		res.setAvailableGames(ares);
		List<AvailableGamesResponse.Game> games = ares.getGame();
		
		gameManager.getGameTypes(games);
		rsp.setLobby(res);
		msg.setResponse(rsp);
		msg.setSessionID(sessionID);
		client.send(msg);
	}
	
	/**
	 * Obtains a list of Game information objects about active gaims and sends it to client.
	 * @param lobbyreq Information about the request.
	 * @param sessionID Specifies the session which sent the request.
	 * @param client Client information.
	 */
	private void handleGetGameList(LobbyRequest lobbyreq, long sessionID, ClientConnection client) {
		Message msg = new Message();
		Response rsp = new Response();
		LobbyResponse res = new LobbyResponse();
		GameListResponse gameRes = new GameListResponse();
		GameListResponse.Games games = new GameListResponse.Games();
		gameRes.setGames(games);
		games.setRequestedGameType(lobbyreq.getGameList().getGameType());
		games.setRequestedGameVariant(lobbyreq.getGameList().getGameVariant());
		
		OptionalStringParameters rsPars = new OptionalStringParameters();
		List<String> columnNames = rsPars.getString();
		
		gameRes.setColumnNames(rsPars);
		
		// TODO need a method to get column names to be listed in game browser in lobby
		
		List<GameListResponse.Games.Game> lstGames = games.getGame();
		
		try {
			gameManager.listGames(lstGames, columnNames, lobbyreq.getGameList().getGameType());
		} catch (ClassNotFoundException e) {
			Logger.get().throwing("GameManager", "listGames", e);
		}
		
		res.setGameList(gameRes);
		rsp.setLobby(res);
		msg.setResponse(rsp);
		msg.setSessionID(sessionID);
		client.send(msg);
		
	}

	/**
	 * Obtains details of one game and sends it to the client who asked for this information.
	 * @param lobbyreq Request instance
	 * @param sessionID Session ID of the requestor.
	 * @param client Client connection of the requestor.
	 */
	private void handleGetGameDetails(LobbyRequest lobbyreq, long sessionID, ClientConnection client) {
		Message msg = new Message();
		Response rsp = new Response();
		LobbyResponse res = new LobbyResponse();
		GameDetailsResponse gameRes = new GameDetailsResponse();

		gameManager.getGameDetails(gameRes, lobbyreq.getGameDetails().getGameID());
		
		res.setGameDetails(gameRes);
		rsp.setLobby(res);
		msg.setResponse(rsp);
		msg.setSessionID(sessionID);
		client.send(msg);
	}
	
}
