package com.asdf.server;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.spi.SelectorProvider;
import java.nio.channels.SelectionKey;
import java.util.*;

import com.asdf.common.ChangeRequest;
import com.asdf.common.Logger;
import com.asdf.utils.ClientServerConstants;

/**
 * Contains server and client socket handling
 * functionality.
 */
public class Server {

	/** Socket selector */
	private Selector selector;
	
	/** ServerChannel for accepting new clients */
	private ServerSocketChannel serverChannel;

	/** 
	 * Switch for running
	 * Can be set false from anywhere to stop execution
	 * gracefully.
	 */
	public boolean running;
	
	/** SessionManager used to store sessionID - client associations */
	private SessionManager sessionManager;
	
	/** AccountManager used to manage removing, creating and authentication of accounts */
	private AccountManager accountManager;
	
	/** GameManager to handle managing and listing of games */
	private GameManager gameManager;
	
	//TODO: old bans should be removed? One mistake shouldn't ban the user forever.
	/** Banlist, contains IP addresses */
	Vector<InetAddress> banList;
	
	// A list of PendingChange instances
	private LinkedList<ChangeRequest> pendingChanges = new LinkedList<ChangeRequest>();

	// Maps a SocketChannel to a list of ByteBuffer instances
	private HashMap<SocketChannel, LinkedList<ByteBuffer>> pendingData = 
		new HashMap<SocketChannel, LinkedList<ByteBuffer>>();


	public Server() {
		accountManager = new AccountManager();
		gameManager = new GameManager();
		sessionManager = new SessionManager();
		banList = new Vector<InetAddress>();
		
		try {
			selector = initSelector();
		} catch (IOException e) {
			Logger.get().throwing("Server", "Constructor", e);
		}
	}

	/**
	 * Initializes server socket and selector. Server socket is initialized
	 * to listen on localhost on port PORT. Method initializes socket as non-blocking. 
	 * Accept events are registered to be captured for the server socket.
	 * @return Configured Selector instance
	 * @throws IOException
	 */
	private Selector initSelector() throws IOException {

		// Create a new selector
		Selector socketSelector = SelectorProvider.provider().openSelector();

		// Create a new non-blocking server socket channel
		serverChannel = ServerSocketChannel.open();

		serverChannel.configureBlocking(false);

		// Bind the server socket to the specified address and port
		int port = ClientServerConstants.PORT;
		InetSocketAddress isa = new InetSocketAddress((InetAddress) null, port);
		serverChannel.socket().bind(isa);

		// Register the server socket channel, indicating an interest in 
		// accepting new connections
		serverChannel.register(socketSelector, SelectionKey.OP_ACCEPT);

		return socketSelector;
	}

	/**
	 * Server event loop. Execution is looping on this loop
	 * for the whole execution time. Loop asks events from the selector
	 * and handles them accordingly.
	 * @throws IOException
	 */
	public void run() throws IOException {
		running = true;
		try {
			while (running) {
				runOnce();
			}
		} finally {
			// Do this to prevent future socket binding errors
			dispose();
		}
	}

	/**
	 * Main event loop of the server. Select is called continuously
	 * to catch events on server and client sockets. Events may
	 * include read, accept and write available. Only read and
	 * accept events are listened on default. Default may be overridden
	 * by add ChangeRequest to the pendingChanges queue.
	 * @throws IOException
	 */
	public void runOnce() throws IOException {
		
		/*
		 * Process any pending changes. Currently only
		 * setting to listen sockets on write events is
		 * supported.
		 */
		synchronized (this.pendingChanges) {
			Iterator<ChangeRequest> changes = this.pendingChanges.iterator();
			while (changes.hasNext()) {
				ChangeRequest change = changes.next();
				switch (change.type) {
					case ChangeRequest.CHANGEOPS:
						SelectionKey key = change.socket.keyFor(this.selector);
						key.interestOps(change.ops);
				}
			}
			this.pendingChanges.clear();
		}
		
		this.selector.select(ClientServerConstants.SELECT_TIMEOUT);
		Iterator<SelectionKey> selectedKeys = this.selector.selectedKeys().iterator();
		while (selectedKeys.hasNext()) {
			SelectionKey key = selectedKeys.next();
			selectedKeys.remove();

			if (!key.isValid())
				continue;

			if (key.isAcceptable()) {
				Accept(key);
			} else if (key.isReadable()) {
				read(key);
			} else if (key.isWritable()) {
				write(key);
			}
		}
	}

	/**
	 * Handles the accept event on key. Accepts new client and
	 * registers reading ev	ents for capturing. Remote address
	 * is printed to log. This function disconnects banned clients.
	 * @param key
	 * @throws IOException
	 */
	private void Accept(SelectionKey key) throws IOException {
		// For an accept to be pending the channel must be a server socket channel.
		ServerSocketChannel serverSocketChannel = (ServerSocketChannel) key.channel();

		// Accept the connection and make it non-blocking
		SocketChannel socketChannel = serverSocketChannel.accept();
		socketChannel.configureBlocking(false);
		InetSocketAddress address = (InetSocketAddress)socketChannel.socket().getRemoteSocketAddress();
		socketChannel.socket().setKeepAlive(true);
		
		if (banList.contains(address.getAddress())) {
			Logger.get().warning(String.format("Banned host (%s) tried to connect.", 
					address));
			socketChannel.socket().shutdownInput();
			socketChannel.socket().shutdownOutput();
			socketChannel.socket().close();
		}
		else {
			Logger.get().info("Accepted connection from " + address.toString());
			// Register the new SocketChannel with our Selector, indicating
			// we'd like to be notified when there's data waiting to be read
			SelectionKey clientKey = socketChannel.register(this.selector,
					SelectionKey.OP_READ);
			clientKey.attach(new ClientConnection(this, clientKey, address));
		}
	}

	/**
	 * Handles read event on key. ClientConnection instance attached
	 * to the key specifies whether it is message size or the actual
	 * message being received.
	 * @param key
	 * @throws IOException
	 */
	private void read(SelectionKey key) throws IOException {
		SocketChannel channel = (SocketChannel) key.channel();
		ClientConnection client = (ClientConnection) key.attachment();
		ClientConnection.State state = client.getReceivingState();

		try {

			if (state == ClientConnection.State.WAITINGSIZE) {
				
				if (channel.read(client.sizeBuffer) == -1) {
					Logger.get().warning("Other end closed");
					disposeClient(key);
					return;
				}
				
				if (client.sizeBuffer.remaining() == 0)
					client.sizeReceived();
				
			} 
			else if (state == ClientConnection.State.WAITINGDATA) {
							
				
				if (channel.read(client.msgBuffer) == -1) {
					Logger.get().warning("Other end closed");
					disposeClient(key);
					return;
				}
				
				if (client.msgBuffer.remaining() == 0) {
					client.msgReceived();
				}
			}
		} 
		catch (IOException e) {
			Logger.get().throwing("Server",
					"An exception occured when reading message", e);
			disposeClient(key);
		}
	}

	/**
	 * Adds data to socket's queue for sending. Selector is waken up
	 * with a wakeup call and select returns write available response
	 * in runOnce method.
	 * @param socket
	 * @param data Data to be sent
	 * @throws IOException
	 */
	public void send(SocketChannel socket, ByteBuffer data) throws IOException {
		synchronized(this.pendingChanges) {
			
			this.pendingChanges.add(
					new ChangeRequest(socket, ChangeRequest.CHANGEOPS, SelectionKey.OP_WRITE));
			synchronized(this.pendingData) {
				LinkedList<ByteBuffer> list = this.pendingData.get(socket);
				if (list == null) {
					list = new LinkedList<ByteBuffer>();
					this.pendingData.put(socket, list);
				}
				list.add(data);
			}
		}
		
		// Finally, wake up our selecting thread so it can make the required changes
		this.selector.wakeup();
		
	}

	/**
	 * Method for handling the writable event occurring in
	 * runOnce method.
	 * @param key Key on which the event occurred
	 */
	private void write(SelectionKey key) {
		SocketChannel socketChannel = null;
		
		try {
			socketChannel = (SocketChannel)key.channel();
	
			synchronized (this.pendingData) {
				LinkedList<ByteBuffer> queue = (LinkedList<ByteBuffer>)this.pendingData.get(socketChannel);
	
				// Write until there's not more data ...
				while (!queue.isEmpty()) {
					ByteBuffer buf = queue.get(0);
					socketChannel.write(buf);
					if (buf.remaining() > 0) {
						// ... or the socket's buffer fills up
						break;
					}
					queue.remove(0);
				}
	
				if (queue.isEmpty()) {
					// We wrote away all data, so we're no longer interested
					// in writing on this socket. Switch back to waiting for
					// data.
					key.interestOps(SelectionKey.OP_READ);
					
					// Key might be marked for disposal
					ClientConnection client = (ClientConnection)key.attachment();
					if (client.disposing) {
						disposeClient(key);
					}
				}
			}
		} catch (IOException e) {
			Logger.get().warning(
					"Couldn't send data to client " + 
					socketChannel.socket().getRemoteSocketAddress());
			Logger.get().throwing("Server", "Send", e);
			synchronized(this.pendingData) {
				LinkedList<ByteBuffer> list = this.pendingData.get(key.channel());
				list.clear();
			}
			disposeClient(key);
		}

	}

	/**
	 * Get number of clients connected. Number of clients
	 * is number of keys on selector minus one. Minus one because
	 * serversocket's key isn't a client.
	 * @return number of clients connected
	 */
	public int clientsConnected() {
		int count = 0;
		for(SelectionKey key : selector.keys()) {
			if (key.isValid())
				count++;
		}
		return count - 1;
	}

	/**
	 * Thoroughly dispose client along with socket and session
	 * @param key Key identifying the user
	 */
	private void disposeClient(SelectionKey key) {
		Logger.get().info("Disposing client");
		ClientConnection client = (ClientConnection) key.attachment();
		sessionManager.remove(client);
		accountManager.logOut(client);
		boolean unsent_data = false;
		synchronized(this.pendingData) {
			LinkedList<ByteBuffer> list = this.pendingData.get(key.channel());
			if (list != null && !list.isEmpty()) {
				unsent_data = true;
				Logger.get().fine("Client has unsent data, postponing disposing");
			}
		}
		if (unsent_data) {
			client.disposing = true;
		} else {
			key.cancel();
			try {
				SocketChannel chan = (SocketChannel)key.channel();
				chan.socket().close();
			}
			catch (IOException e) {
				Logger.get().throwing("Server", "disposeClient", e);
			}
		}
	}
	
	/**
	 * Ban client by IP.
	 * @param client Client to ban
	 */
	public void ban(ClientConnection client) {
		Logger.get().info("Banning client " + client.getAddress());
		banList.add(client.getRawAddress());
		disposeClient(client.getKey());
	}
	
	/**
	 * Clean up serversocket so next initialization doesn't
	 * throw socket in use exception.
	 */
	@Override
	protected void finalize() throws IOException {
		this.serverChannel.close();
	}
	
	/**
	 * Getter for GameManager
	 * @return the GameManager
	 */
	public GameManager getGameManager() {
		return this.gameManager;
	}
	
	/**
	 * Getter for AccountManager
	 * @return the AccountManager
	 */
	public AccountManager getAccountManager() {
		return this.accountManager;
	}
	
	/**
	 * Getter for SessionManager
	 * @return SessionManager
	 */
	public SessionManager getSessionManager() {
		return this.sessionManager;
	}

	/**
	 * Can be called to free resources.
	 */
	public void dispose() {
		Logger.get().fine("Disposing server");
		if (serverChannel != null) {
			try {
				serverChannel.socket().close();
				serverChannel.close();
			} catch (IOException e) {
				Logger.get().throwing("Server", "dispose", e);
			}
		}
	}
}
