package com.asdf.server;

import java.util.HashMap;
import java.util.LinkedList;

import com.asdf.common.Logger;

/**
 * Stores session - client association.
 */
public class SessionManager {
	
	/** For finding out ClientConnection by session ID */
	private HashMap<Long, ClientConnection> clients;
	
	/** For finding sessions by ClientConnection */
	private HashMap<ClientConnection, LinkedList<Long>> sessions;
	
	/**
	 * Initializes empty lists
	 */
	public SessionManager() {
		clients = new HashMap<Long, ClientConnection>();
		sessions = new HashMap<ClientConnection, LinkedList<Long>>();
	}
	
	/**
	 * Get ClientConnection by session ID. There can only be
	 * one or none.
	 * @param sessionID ClientConnection having this session ID
	 * @return ClientConnection associated to given session ID
	 */
	public ClientConnection getBySessionID(long sessionID) {
		Logger.get().finest("Getting connection for sessionID " + sessionID);
		ClientConnection cc = clients.get(sessionID);
		if (cc != null && !cc.getKey().isValid()) {
			Logger.get().fine("Noticed invalid key, removing");
			remove(cc);
			return null;
		}
		return cc;		
	}
	
	/**
	 * Get Session IDs by ClientConnection. There can be any amount
	 * of ids including none.
	 * @param client ClientConnection used as search key
	 * @return Array of session ids or null
	 */
	public Long[] getByClientConnection(ClientConnection client) {
		if (!client.getKey().isValid()) {
			remove(client);
			return null;
		}
		
		LinkedList<Long> ids = sessions.get(client);
		Long[] list = new Long[ids.size()];
		ids.toArray(list);
		return list;
	}
	
	/**
	 * Add new session ID for given ClientConnection
	 * @param sessionID
	 * @param client
	 */
	public void put(long sessionID, ClientConnection client) {
		Logger.get().fine("Adding sessionID " + sessionID);
		clients.put(sessionID, client);
		if (!sessions.containsKey(client))
			sessions.put(client, new LinkedList<Long>());
		sessions.get(client).add(sessionID);
	}
	
	/**
	 * Remove session ID
	 * @param sessionID ID to remove
	 */
	public void remove(long sessionID) {
		Logger.get().fine("Removing sessionID " + sessionID);
		ClientConnection conn = clients.remove(sessionID);
		if (conn != null) {
			sessions.get(conn).remove(sessionID);
		}
		
	}
	
	/**
	 * Remove ClientConnection and associated session IDs
	 * @param client ClientConnection to remove
	 */
	public void remove(ClientConnection client) {
		if (sessions.containsKey(client)) {
			LinkedList<Long> clientSessions = sessions.remove(client);
			for(long sessid : clientSessions) {
				clients.remove(sessid);
			}
		}
	}
}
