package com.asdf.test;

import java.lang.Thread;
import java.util.concurrent.atomic.AtomicBoolean;
import com.asdf.server.Server;
import com.asdf.test.TestConstants;
import com.asdf.utils.Callback;

/**
 * ServerRunner
 * Executes server component in it's own thread
 * as long as necessary and handles stopping it
 * when necessary.
 */
public class ServerRunner implements Runnable {
	
	private Exception exception;
	private Error error;
	private AtomicBoolean running;
	private Thread thread;
	private Callback handler;
	
	/**
	 * Initialize ServerRunner and thread for it.
	 * Then keep running event loop in run() method
	 * until stop() is called or execution has occurred.
	 * @return ServerRunner instance
	 */
	public static ServerRunner start(Callback handler) {
		ServerRunner runner = new ServerRunner(handler);
		Thread t = new Thread(runner);
		runner.thread = t;
		t.start();
		try {
			synchronized(runner) {
				runner.wait();
			}
		} catch(Exception e) {
			System.err.println("Exception in ServerRunner: " + e.toString());
			e.printStackTrace();
		} catch(Error e) {
			System.err.println("Error in ServerRunner: " + e.toString());
			e.printStackTrace();
		}
		return runner;
	}
	
	public ServerRunner(Callback handler) {
		this.running = new AtomicBoolean();
		this.handler = handler;
	}
	
	/**
	 * Run event loop continuously until
	 * stop() is called. Occurring exceptions
	 * are stored silently in exception member
	 * and can be later retrieved for inspection.
	 */
	@Override
	public void run() {
		running.set(true);
		Server s = null;
		try {
			s = new Server();
			synchronized(this) {
				this.notify();
			}
			while (running.get()) {
				s.runOnce();
				handler.call(s);
			}
		}
		catch(Exception e) {
			System.err.println("Exception in ServerRunner: " + e.toString());
			e.printStackTrace();
			exception = e;
		}
		catch(Error e) {
			System.err.println("Error in ServerRunner: " + e.toString());
			e.printStackTrace();
			error = e;
		}
		finally {
			if (s != null)
				s.dispose();
		}
	}
	
	/**
	 * Stop running event loop. Running stops
	 * on next cycle and function returns after
	 * thread has been resumed. If thread doesn't resume
	 * in FREEZE_THRESHOLD seconds it's considered an error.
	 * Thread is then interrupted.
	 */
	public void stop() throws Exception {
		running.set(false);
		thread.join(TestConstants.FREEZE_THRESHOLD);
		if (thread.isAlive())
		{
			thread.interrupt();
			throw new Exception("ServerRunner thread froze");
		}
		
		if (this.exception != null)
			throw this.exception;
		

		if (this.error != null)
			throw this.error;
	}
}
