package com.asdf.test;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.net.Socket;

import org.junit.Assert;
import org.junit.Test;

import com.asdf.parser.*;
import com.asdf.server.Server;
import com.asdf.utils.ByteUtils;
import com.asdf.utils.ClientServerConstants;
import com.asdf.utils.XMLTools;

/**
 * Tries to find synchronization errors from server by
 * executing 100 simultaneous connections.
 */
public class ServerStressTest extends ServerTest {
	
	protected static int THREADS = 100;
	
	protected XMLTools xmltools = new XMLTools();
		
	/**
	 * Inner class representing one client connection.
	 */
	protected class StressRunner implements Runnable {
		
		public Exception exception;
		public Error error;
		protected int index;
		
		public StressRunner(int index) {
			this.index = index;
		}
		
		/**
		 * Read message from inputstream
		 * @param input DataInputStream from where the message is erad
		 * @return Message
		 * @throws Exception
		 */
		private Message readMsg(DataInputStream input) throws Exception {
			byte[] sizebuffer = new byte[4];
			Assert.assertEquals(sizebuffer.length, input.read(sizebuffer));
			int size = ByteUtils.bytesToInt(sizebuffer);
			byte[] msgbuffer = new byte[size];
			Assert.assertEquals(msgbuffer.length, input.read(msgbuffer));
			System.out.println("Received {" + new String(msgbuffer) + "}");
			synchronized(xmltools) {
				return xmltools.bytesToMessage(msgbuffer);
			}
		}
		
		/**
		 * Send message to given outputstream
		 * @param msg Message to send
		 * @param output OutputStream
 		 * @throws Exception
		 */
		private void sendMsg(Message msg, DataOutputStream output) throws Exception {
			byte[] msgData;
			synchronized(xmltools) {
				msgData = xmltools.messageToBytes(msg);
			}
			output.write(ByteUtils.intToBytes(msgData.length));
			output.write(msgData);
			String s = new String(msgData);
			System.out.println("Sent: {" + s + "}");
		}
		
		public void run() {
			try {
				Socket socket = new Socket("localhost", ClientServerConstants.PORT);
				//socket.setSoTimeout(1000);
				DataInputStream input = new DataInputStream(socket.getInputStream());
				DataOutputStream output = new DataOutputStream(socket.getOutputStream());
				
				Message response;
				Message msg = new Message();
				msg.setSessionID((long)index + 100);
				Request req = new Request();
				
				AuthenticationRequest authReq = new AuthenticationRequest();
				AuthenticationRequest.Login loginReq = new AuthenticationRequest.Login();
				loginReq.setAccountName("ernesti");
				loginReq.setPassword("salakala");
				authReq.setLogin(loginReq);
				req.setAuthentication(authReq);
				msg.setRequest(req);
				
				sendMsg(msg, output);
				response = readMsg(input);
				Assert.assertEquals(true, response.getResponse().getAuthentication().getLogin().isSuccess());
				
				req.setAuthentication(null);
				GameRequest gameReq = new GameRequest();
				CreateGameRequest cgReq = new CreateGameRequest();
				cgReq.setGameType("com.asdf.test.TestGamePlugin");
				cgReq.setName("Game " + index);
				gameReq.setCreate(cgReq);
				req.setGame(gameReq);
				msg.setRequest(req);
				
				sendMsg(msg, output);
				response = readMsg(input);
				Assert.assertEquals(true, response.getResponse().getGame().getCreate().isSuccess());
				gameReq.setGameID(response.getResponse().getGame().getCreate().getGameID());
				
				
				OptionalParameters join = new OptionalParameters();
				gameReq.setCreate(null);
				gameReq.setJoin(join);
				
				sendMsg(msg, output);
				response = readMsg(input);
				Assert.assertEquals(true, response.getResponse().getGame().getJoin().isSuccess());
				
				
				input.close();
				output.close();
				socket.close();
				
			} catch (Exception e) {
				this.exception = e;
			} catch(Error e) {
				this.error = e;
			}
		}
	}
	
	@Override
	protected void onServerLoop(Server server) {
	}
	
	@Override
	@Test(timeout=90000)
	public void testServer() throws Exception{
		startServer();
		
		long start = System.currentTimeMillis();
		
		Thread[] threads = new Thread[THREADS];
		StressRunner[] runners = new StressRunner[THREADS];
		for(int i = 0; i < THREADS; i++) {
			runners[i] = new StressRunner(i);
			threads[i] = new Thread(runners[i]);
			threads[i].start();
		}
		for(int i = 0; i < THREADS; i++) {
			threads[i].join();
			if (runners[i].exception != null)
				throw runners[i].exception;
			if (runners[i].error != null)
				throw runners[i].error;
		}
		
		double seconds = (System.currentTimeMillis() - start) / 1000.0;
		System.out.println("Running " + THREADS + 
				" concurrent sessions that authenticate, create game and join it" +
				" took " + seconds + " seconds.");
		
		stopServer();
		
	}

}
