package com.asdf.test;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.net.Socket;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;
import org.junit.Test;
import org.junit.Assert;
import com.asdf.parser.Message;
import com.asdf.server.Server;
import com.asdf.utils.ByteUtils;
import com.asdf.utils.Callback;
import com.asdf.utils.XMLTools;

/**
 * Integration test framework for server.
 */
public abstract class ServerTest {
	
	/**
	 * Handler which is called on every iteration
	 * on ServerRunner event loop.
	 */
	protected Callback serverHandler;
	
	/**
	 * Semaphore for determining when shutting down
	 * can begin.
	 */
	protected Semaphore semaphore;
	
	/**
	 * Instance of ServerRunner
	 */
	protected ServerRunner server;
	
	/**
	 * XMLTools handling (un)marshalling of messages
	 */
	protected XMLTools xmltools = new XMLTools();
	
	protected Socket socket;
	protected DataInputStream input;
	protected DataOutputStream output;
	
	/**
	 * Initialize serverhandler and semaphore.
	 */
	public ServerTest() {
		serverHandler = new Callback(){
			public void call(Object o) { onServerLoop((Server)o);}
		};
		semaphore = new Semaphore(1);
	}
	
	/**
	 * Is called at the end of every event cycle
	 * on ServerRunner.
	 * @param Server instance
	 */
	abstract protected void onServerLoop(Server server);
	
	/**
	 * Runs server as long as semaphore is released.
	 */
	protected void runServer() throws Exception {
		semaphore.acquire();
		server = ServerRunner.start(serverHandler);
		boolean success = semaphore.tryAcquire(
				TestConstants.TESTCASE_TIMEOUT, TimeUnit.MILLISECONDS);
		server.stop();
		if (!success)
			Assert.fail("Testcase didn't release semaphore in 10 seconds");	
		System.gc();
	}
	
	/**
	 * Starts server in it's own thread and returns.
	 * @throws Exception
	 */
	protected void startServer() throws Exception {
		server = ServerRunner.start(serverHandler);
	}
	
	/**
	 * Stops running server.
	 * @throws Exception
	 */
	protected void stopServer() throws Exception {
		server.stop();
		System.gc();
	}
	
	/**
	 * Actual test case. May be overridden.
	 */
	@Test(timeout=10000)
	public void testServer() throws Exception {
		runServer();
	}
	
	/**
	 * Read file from given path and send it to
	 * the socket. Length of the file is sent in
	 * binary form before the actual message.
	 * @param path Path to the message
	 * @throws Exception
	 */
	protected void sendMessage(String path) throws Exception {
		sendMessage(path, this.output);
	}
	
	/**
	 * Read file from given path and send it to
	 * the socket. Length of the file is sent in
	 * binary form before the actual message.
	 * @param path Path to the message
	 * @param output DataOutputStream
	 * @throws Exception
	 */
	protected void sendMessage(String path, DataOutputStream output) throws Exception {
		File file = new File(path);
		FileInputStream f = new FileInputStream(file);
		byte[] contents = new byte[(int)file.length()];
		f.read(contents);
		String s = new String(contents);
		s.trim();
		
		System.out.println("Sending: {" + new String(contents) + "}");
		output.write(ByteUtils.intToBytes((int)file.length()));
		output.write(contents);
	}
	
	/**
	 * Receive response Message from the socket. Other end
	 * always send size of the message first. Raw data received
	 * from the socket is converted to Message object.
	 * @return Message object containing response
	 * @throws Exception
	 */
	protected Message receiveResponse() throws Exception {
		return receiveResponse(this.input);
	}
	
	/**
	 * Receive response Message from the socket. Other end
	 * always send size of the message first. Raw data received
	 * from the socket is converted to Message object.
	 * @param input DataInputStream
	 * @return Message object containing response
	 * @throws Exception
	 */
	protected Message receiveResponse(DataInputStream input) throws Exception {
		byte[] sizebuffer = new byte[4];
		Assert.assertEquals(sizebuffer.length, input.read(sizebuffer));
		int size = ByteUtils.bytesToInt(sizebuffer);
		byte[] msgbuffer = new byte[size];
		Assert.assertEquals(msgbuffer.length, input.read(msgbuffer));
		System.out.println("Received {" + new String(msgbuffer) + "}");
		return xmltools.bytesToMessage(msgbuffer);
	}
}
