package com.asdf.test;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;
import org.w3c.dom.Element;

import com.asdf.parser.AuthenticationRequest;
import com.asdf.parser.CreateGameRequest;
import com.asdf.parser.GameRequest;
import com.asdf.parser.Message;
import com.asdf.parser.OptionalParameters;
import com.asdf.parser.Parameters;
import com.asdf.parser.Request;
import com.asdf.parser.RequireParameters;
import com.asdf.server.Server;
import com.asdf.utils.BasicElement;
import com.asdf.utils.ByteUtils;
import com.asdf.utils.ClientServerConstants;
import com.asdf.utils.XMLTools;

public class ServerTimerStressTest extends ServerStressTest {
	protected static int THREADS = 4;
	
	protected XMLTools xmltools = new XMLTools();
	
	/**
	 * Read message from inputstream
	 * @param input DataInputStream from where the message is erad
	 * @return Message
	 * @throws Exception
	 */
	protected Message readMsg(DataInputStream input) throws Exception {
		byte[] sizebuffer = new byte[4];
		Assert.assertEquals(sizebuffer.length, input.read(sizebuffer));
		int size = ByteUtils.bytesToInt(sizebuffer);
		byte[] msgbuffer = new byte[size];
		Assert.assertEquals(msgbuffer.length, input.read(msgbuffer));
		System.out.println("Received {" + new String(msgbuffer) + "}");
		synchronized(xmltools) {
			return xmltools.bytesToMessage(msgbuffer);
		}
	}
	
	/**
	 * Send message to given outputstream
	 * @param msg Message to send
	 * @param output OutputStream
	 * @throws Exception
	 */
	protected void sendMsg(Message msg, DataOutputStream output) throws Exception {
		byte[] msgData;
		synchronized(xmltools) {
			msgData = xmltools.messageToBytes(msg);
		}
		output.write(ByteUtils.intToBytes(msgData.length));
		output.write(msgData);
		String s = new String(msgData);
		System.out.println("Sent: {" + s + "}");
	}
	
	/**
	 * Inner class representing one client connection.
	 */
	protected class StressRunner implements Runnable {
		
		public Exception exception;
		public Error error;
		protected int index;
		
		private Socket socket;
		private DataInputStream input;
		private DataOutputStream output;
		
		public StressRunner(int index) throws UnknownHostException, IOException {
			this.index = index;
			socket = new Socket("localhost", ClientServerConstants.PORT);
			input = new DataInputStream(socket.getInputStream());
			output = new DataOutputStream(socket.getOutputStream());
		}
		
		protected void action(long sessionID, long gameID) throws Exception {
			Message msg = new Message();
			msg.setSessionID(sessionID);
			Request req = new Request();
			GameRequest gameReq = new GameRequest();
			RequireParameters reqPars = new RequireParameters();
			Parameters pars = new Parameters();
			gameReq.setGameID(gameID);
			msg.setRequest(req);
			req.setGame(gameReq);
			gameReq.setAction(reqPars);
			reqPars.setParameters(pars);
			
			List<Element> list = pars.getAny();
			BasicElement e = new BasicElement("settimer");
			list.add(e);
			
			sendMsg(msg, output);
			Message response = readMsg(input);
			Assert.assertEquals(true, response.getResponse().getGame().getAction().isSuccess());
		}
		
		protected void authenticate(long sessionID) throws Exception {
			Message msg = new Message();
			msg.setSessionID(sessionID);
			Request req = new Request();
			AuthenticationRequest authReq = new AuthenticationRequest();
			AuthenticationRequest.Login loginReq = new AuthenticationRequest.Login();
			loginReq.setAccountName("ernesti");
			loginReq.setPassword("salakala");
			authReq.setLogin(loginReq);
			req.setAuthentication(authReq);
			msg.setRequest(req);
			
			sendMsg(msg, output);
			Message response = readMsg(input);
			Assert.assertEquals(true, response.getResponse().getAuthentication().getLogin().isSuccess());
		}
		
		protected void joinGame(long sessionID, long gameID) throws Exception {
			Message msg = new Message();
			Request req = new Request();
			GameRequest gameReq = new GameRequest();
			OptionalParameters join = new OptionalParameters();
			
			msg.setSessionID(sessionID);
			
			msg.setRequest(req);
			req.setGame(gameReq);
			gameReq.setJoin(join);
			gameReq.setGameID(gameID);
			
			sendMsg(msg, output);
			Message response = readMsg(input);
			Assert.assertEquals(true, response.getResponse().getGame().getJoin().isSuccess());
		}
	
		protected long createGame(long sessionID) throws Exception {
			Message msg = new Message();
			msg.setSessionID(sessionID);
			Request req = new Request();
			GameRequest gameReq = new GameRequest();
			CreateGameRequest cgReq = new CreateGameRequest();
			cgReq.setGameType("com.asdf.test.TestGamePlugin");
			cgReq.setName("Game " + index);
			gameReq.setCreate(cgReq);
			req.setGame(gameReq);
			msg.setRequest(req);
			
			sendMsg(msg, output);
			Message response = readMsg(input);
			Assert.assertEquals(true, response.getResponse().getGame().getCreate().isSuccess());
			return response.getResponse().getGame().getCreate().getGameID();
		}
		
		@Override
		public void finalize() throws IOException {
			input.close();
			output.close();
			socket.close();
		}
		
		public void run() {
			try {
						
				long sessionID = (long)index + 100;
				authenticate(sessionID);
				long gameID = createGame(sessionID);
				joinGame(sessionID, gameID);
				action(sessionID, gameID);

				
			} catch (Exception e) {
				this.exception = e;
			} catch(Error e) {
				this.error = e;
			}
		}
	}
	
	@Override
	protected void onServerLoop(Server server) {
	}
	
	@Override
	@Test(timeout=10000)
	public void testServer() throws Exception{
		startServer();
		
		long start = System.currentTimeMillis();
		
		Thread[] threads = new Thread[THREADS];
		StressRunner[] runners = new StressRunner[THREADS];
		for(int i = 0; i < THREADS; i++) {
			runners[i] = new StressRunner(i);
			threads[i] = new Thread(runners[i]);
			threads[i].start();
		}
		for(int i = 0; i < THREADS; i++) {
			threads[i].join();
			if (runners[i].exception != null)
				throw runners[i].exception;
			if (runners[i].error != null)
				throw runners[i].error;
		}
		
		double seconds = (System.currentTimeMillis() - start) / 1000.0;
		System.out.println("Running " + THREADS + 
				" concurrent sessions that authenticate, create game and join it" +
				" took " + seconds + " seconds.");
		
		stopServer();
		
	}
}
