package com.asdf.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.asdf.parser.Message;

/**
 * Tools for (un)marshalling XML data from objects to
 * byte arrays and from byte array to objects.
 */
public class XMLTools {

	/** Default Jaxb context string to top level server parser. */
	private static final String JAXBCONTEXT = "com.asdf.parser";
	
	/** Default schema location for top level server parser */
	private static final String SCHEMA_LOCATION = "../Protocol/message.xsd";
	
	// Marshaller used to convert objects to byte arrays
	private Marshaller marshaller;
	// Unmarshaller used to convert byte arrays to objects
	private Unmarshaller unmarshaller;
	// JAXB context defining (un)marshalling contexts
	private JAXBContext context;
	
	/**
	 * Constructs XMLTools object. Version of constructor that uses
	 * default XML schema file and JAXB context.
	 */
	public XMLTools() {
		try {
			SchemaFactory sf = SchemaFactory.newInstance(
					javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI);
			Schema s = sf.newSchema(new File(SCHEMA_LOCATION));
			
			context = JAXBContext.newInstance(JAXBCONTEXT);
			
			marshaller = context.createMarshaller();
			marshaller.setSchema(s);
			
			unmarshaller = context.createUnmarshaller();
			unmarshaller.setSchema(s);
		} catch(Exception e) {
			throw new Error(e);
		}
	}
	
	/**
	 * Constructs XMLTools object. Version of constructor that lets
	 * user to define XML schema and JAXB context.
	 * @param jaxbContext Name of the JAXB context
	 * @param schemaLocation Path to the XML schema file
	 * @throws SAXException If schema is invalid
	 * @throws JAXBException If context can't be initialized
	 */
	public XMLTools(String jaxbContext, String schemaLocation) throws SAXException, JAXBException {
		
		SchemaFactory sf = SchemaFactory.newInstance(
				javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI);
		Schema s = sf.newSchema(new File(schemaLocation));
		
		context = JAXBContext.newInstance(jaxbContext);
		
		marshaller = context.createMarshaller();
		marshaller.setSchema(s);
		
		unmarshaller = context.createUnmarshaller();
		unmarshaller.setSchema(s);
	}
	
	/**
	 * Converts byte array into Message object
	 * @param bytes Byte array
	 * @return Unmarshalled object
	 * @throws JAXBException In case of not conforming to schema or otherwise
	 * invalid data
	 */
	public synchronized Object unmarshallBytes(byte[] bytes) throws JAXBException {
		InputStream is = new ByteArrayInputStream(bytes);
		return unmarshaller.unmarshal(is);
	}
	
	/**
	 * Convert given Node to object
	 * @param e Node to convert
	 * @return Converted object
	 * @throws JAXBException
	 */
	public synchronized Object unmarshallNode(Node e) throws JAXBException {
		return unmarshaller.unmarshal(e);
	}
	
	/**
	 * Converts object into byte array
	 * @param obj Object to convert
	 * @return Object converted to byte array
	 * @throws JAXBException In case of not conforming to schema or otherwise
	 * invalid message
	 */
	public synchronized byte[] marshallObject(Object obj) throws JAXBException {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		marshaller.marshal(obj, os);
		return os.toByteArray();
	}
	
	public synchronized Element marshallObjectToElement(Object obj) throws JAXBException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		DocumentBuilder db;
		try {
			db = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		}
		Document doc = db.newDocument();
		
		marshaller.marshal(obj, doc);
		return (Element)doc.getChildNodes().item(0);
	}

	/**
	 * Unmarshalls object from file and casts it to Message.
	 * @param path Path to the file
	 * @return Unmarshalled Message
	 * @throws JAXBException In case of not conforming to schema or otherwise
	 * invalid message
	 */
	public synchronized Message loadMessageFromFile(String path) throws JAXBException {
		return (Message) loadObjectFromFile(path);
	}

	/**
	 * Unmarshalls object from file.
	 * @param path Path to the file
	 * @return Unmarshalled object
	 * @throws JAXBException In case of not conforming to schema or otherwise
	 * invalid data
	 */
	public synchronized Object loadObjectFromFile(String path) throws JAXBException {
		return unmarshaller.unmarshal(new File(path));
	}

	/**
	 * Converts byte array to Message instance
	 * @param rawData Raw data to convert
	 * @return Instance of Message
	 * @throws JAXBException In case of not conforming to schema or otherwise
	 * invalid message
	 */
	public synchronized Message bytesToMessage(byte[] rawData) throws JAXBException {
		return (Message)unmarshallBytes(rawData);
	}

	/**
	 * Converts Message to byte array.
	 * @param msg Message to convert
	 * @return Message marshalled to byte array
	 * @throws JAXBException In case of not conforming to schema or otherwise
	 * invalid message
	 */
	public synchronized byte[] messageToBytes(Message msg) throws JAXBException {
		return marshallObject(msg);
	}

}
